@@ -73,36 +73,36 @@
      BX_TOKEN_FPU = 289,
      BX_TOKEN_SSE = 290,
      BX_TOKEN_MMX = 291,
-     BX_TOKEN_ALL = 292,
-     BX_TOKEN_IDT = 293,
-     BX_TOKEN_IVT = 294,
-     BX_TOKEN_GDT = 295,
-     BX_TOKEN_LDT = 296,
-     BX_TOKEN_TSS = 297,
-     BX_TOKEN_TAB = 298,
-     BX_TOKEN_DIRTY = 299,
-     BX_TOKEN_LINUX = 300,
-     BX_TOKEN_CONTROL_REGS = 301,
-     BX_TOKEN_SEGMENT_REGS = 302,
-     BX_TOKEN_EXAMINE = 303,
-     BX_TOKEN_XFORMAT = 304,
-     BX_TOKEN_DISFORMAT = 305,
-     BX_TOKEN_RESTORE = 306,
-     BX_TOKEN_SETPMEM = 307,
-     BX_TOKEN_SYMBOLNAME = 308,
-     BX_TOKEN_QUERY = 309,
-     BX_TOKEN_PENDING = 310,
-     BX_TOKEN_TAKE = 311,
-     BX_TOKEN_DMA = 312,
-     BX_TOKEN_IRQ = 313,
-     BX_TOKEN_DISASSEMBLE = 314,
-     BX_TOKEN_INSTRUMENT = 315,
-     BX_TOKEN_STRING = 316,
-     BX_TOKEN_STOP = 317,
-     BX_TOKEN_DOIT = 318,
-     BX_TOKEN_CRC = 319,
-     BX_TOKEN_TRACE = 320,
-     BX_TOKEN_TRACEREG = 321,
+     BX_TOKEN_IDT = 292,
+     BX_TOKEN_IVT = 293,
+     BX_TOKEN_GDT = 294,
+     BX_TOKEN_LDT = 295,
+     BX_TOKEN_TSS = 296,
+     BX_TOKEN_TAB = 297,
+     BX_TOKEN_DIRTY = 298,
+     BX_TOKEN_LINUX = 299,
+     BX_TOKEN_CONTROL_REGS = 300,
+     BX_TOKEN_SEGMENT_REGS = 301,
+     BX_TOKEN_EXAMINE = 302,
+     BX_TOKEN_XFORMAT = 303,
+     BX_TOKEN_DISFORMAT = 304,
+     BX_TOKEN_RESTORE = 305,
+     BX_TOKEN_SETPMEM = 306,
+     BX_TOKEN_SYMBOLNAME = 307,
+     BX_TOKEN_QUERY = 308,
+     BX_TOKEN_PENDING = 309,
+     BX_TOKEN_TAKE = 310,
+     BX_TOKEN_DMA = 311,
+     BX_TOKEN_IRQ = 312,
+     BX_TOKEN_DISASSEMBLE = 313,
+     BX_TOKEN_INSTRUMENT = 314,
+     BX_TOKEN_STRING = 315,
+     BX_TOKEN_STOP = 316,
+     BX_TOKEN_DOIT = 317,
+     BX_TOKEN_CRC = 318,
+     BX_TOKEN_TRACE = 319,
+     BX_TOKEN_TRACEREG = 320,
+     BX_TOKEN_TRACEMEM = 321,
      BX_TOKEN_SWITCH_MODE = 322,
      BX_TOKEN_SIZE = 323,
      BX_TOKEN_PTIME = 324,
@@ -177,36 +177,36 @@
 #define BX_TOKEN_FPU 289
 #define BX_TOKEN_SSE 290
 #define BX_TOKEN_MMX 291
-#define BX_TOKEN_ALL 292
-#define BX_TOKEN_IDT 293
-#define BX_TOKEN_IVT 294
-#define BX_TOKEN_GDT 295
-#define BX_TOKEN_LDT 296
-#define BX_TOKEN_TSS 297
-#define BX_TOKEN_TAB 298
-#define BX_TOKEN_DIRTY 299
-#define BX_TOKEN_LINUX 300
-#define BX_TOKEN_CONTROL_REGS 301
-#define BX_TOKEN_SEGMENT_REGS 302
-#define BX_TOKEN_EXAMINE 303
-#define BX_TOKEN_XFORMAT 304
-#define BX_TOKEN_DISFORMAT 305
-#define BX_TOKEN_RESTORE 306
-#define BX_TOKEN_SETPMEM 307
-#define BX_TOKEN_SYMBOLNAME 308
-#define BX_TOKEN_QUERY 309
-#define BX_TOKEN_PENDING 310
-#define BX_TOKEN_TAKE 311
-#define BX_TOKEN_DMA 312
-#define BX_TOKEN_IRQ 313
-#define BX_TOKEN_DISASSEMBLE 314
-#define BX_TOKEN_INSTRUMENT 315
-#define BX_TOKEN_STRING 316
-#define BX_TOKEN_STOP 317
-#define BX_TOKEN_DOIT 318
-#define BX_TOKEN_CRC 319
-#define BX_TOKEN_TRACE 320
-#define BX_TOKEN_TRACEREG 321
+#define BX_TOKEN_IDT 292
+#define BX_TOKEN_IVT 293
+#define BX_TOKEN_GDT 294
+#define BX_TOKEN_LDT 295
+#define BX_TOKEN_TSS 296
+#define BX_TOKEN_TAB 297
+#define BX_TOKEN_DIRTY 298
+#define BX_TOKEN_LINUX 299
+#define BX_TOKEN_CONTROL_REGS 300
+#define BX_TOKEN_SEGMENT_REGS 301
+#define BX_TOKEN_EXAMINE 302
+#define BX_TOKEN_XFORMAT 303
+#define BX_TOKEN_DISFORMAT 304
+#define BX_TOKEN_RESTORE 305
+#define BX_TOKEN_SETPMEM 306
+#define BX_TOKEN_SYMBOLNAME 307
+#define BX_TOKEN_QUERY 308
+#define BX_TOKEN_PENDING 309
+#define BX_TOKEN_TAKE 310
+#define BX_TOKEN_DMA 311
+#define BX_TOKEN_IRQ 312
+#define BX_TOKEN_DISASSEMBLE 313
+#define BX_TOKEN_INSTRUMENT 314
+#define BX_TOKEN_STRING 315
+#define BX_TOKEN_STOP 316
+#define BX_TOKEN_DOIT 317
+#define BX_TOKEN_CRC 318
+#define BX_TOKEN_TRACE 319
+#define BX_TOKEN_TRACEREG 320
+#define BX_TOKEN_TRACEMEM 321
 #define BX_TOKEN_SWITCH_MODE 322
 #define BX_TOKEN_SIZE 323
 #define BX_TOKEN_PTIME 324
