@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: bit32.cc,v 1.3 2007/12/23 17:21:27 sshwarts Exp $
+// $Id: bit32.cc,v 1.8 2008/04/25 07:40:51 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -25,16 +25,14 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 /////////////////////////////////////////////////////////////////////////
 
-
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
-
 #if BX_CPU_LEVEL >= 3
 
-void BX_CPU_C::BSF_GdEd(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BSF_GdEd(bxInstruction_c *i)
 {
   /* for 32 bit operand size mode */
   Bit32u op1_32, op2_32;
@@ -44,6 +42,7 @@ void BX_CPU_C::BSF_GdEd(bxInstruction_c *i)
     op2_32 = BX_READ_32BIT_REG(i->rm());
   }
   else {
+    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
     op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
   }
@@ -54,7 +53,7 @@ void BX_CPU_C::BSF_GdEd(bxInstruction_c *i)
   }
 
   op1_32 = 0;
-  while ( (op2_32 & 0x01) == 0 ) {
+  while ((op2_32 & 0x01) == 0) {
     op1_32++;
     op2_32 >>= 1;
   }
@@ -66,7 +65,7 @@ void BX_CPU_C::BSF_GdEd(bxInstruction_c *i)
   BX_WRITE_32BIT_REGZ(i->nnn(), op1_32);
 }
 
-void BX_CPU_C::BSR_GdEd(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BSR_GdEd(bxInstruction_c *i)
 {
   /* for 32 bit operand size mode */
   Bit32u op1_32, op2_32;
@@ -76,6 +75,7 @@ void BX_CPU_C::BSR_GdEd(bxInstruction_c *i)
     op2_32 = BX_READ_32BIT_REG(i->rm());
   }
   else {
+    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
     op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
   }
@@ -86,7 +86,7 @@ void BX_CPU_C::BSR_GdEd(bxInstruction_c *i)
   }
 
   op1_32 = 31;
-  while ( (op2_32 & 0x80000000) == 0 ) {
+  while ((op2_32 & 0x80000000) == 0) {
     op1_32--;
     op2_32 <<= 1;
   }
@@ -98,17 +98,24 @@ void BX_CPU_C::BSR_GdEd(bxInstruction_c *i)
   BX_WRITE_32BIT_REGZ(i->nnn(), op1_32);
 }
 
-void BX_CPU_C::BT_EdGdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EdGdM(bxInstruction_c *i)
 {
   bx_address op1_addr;
   Bit32u op1_32, op2_32, index;
   Bit32s displacement32;
 
-  op2_32 = BX_READ_32BIT_REG(i->nnn());
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
+  op2_32 = BX_READ_32BIT_REG(i->nnn());
   index = op2_32 & 0x1f;
   displacement32 = ((Bit32s) (op2_32&0xffffffe0)) / 32;
   op1_addr = RMAddr(i) + 4 * displacement32;
+  if (! i->as32L())
+    op1_addr = (Bit16u) op1_addr;
+#if BX_SUPPORT_X86_64
+  else if (! i->as64L())
+    op1_addr = (Bit32u) op1_addr;
+#endif
 
   /* pointer, segment address pair */
   op1_32 = read_virtual_dword(i->seg(), op1_addr);
@@ -116,7 +123,7 @@ void BX_CPU_C::BT_EdGdM(bxInstruction_c *i)
   set_CF((op1_32 >> index) & 0x01);
 }
 
-void BX_CPU_C::BT_EdGdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EdGdR(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
@@ -127,18 +134,25 @@ void BX_CPU_C::BT_EdGdR(bxInstruction_c *i)
   set_CF((op1_32 >> op2_32) & 0x01);
 }
 
-void BX_CPU_C::BTS_EdGdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EdGdM(bxInstruction_c *i)
 {
   bx_address op1_addr;
   Bit32u op1_32, op2_32, index;
   Bit32s displacement32;
   bx_bool bit_i;
 
-  op2_32 = BX_READ_32BIT_REG(i->nnn());
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
+  op2_32 = BX_READ_32BIT_REG(i->nnn());
   index = op2_32 & 0x1f;
   displacement32 = ((Bit32s) (op2_32&0xffffffe0)) / 32;
   op1_addr = RMAddr(i) + 4 * displacement32;
+  if (! i->as32L())
+    op1_addr = (Bit16u) op1_addr;
+#if BX_SUPPORT_X86_64
+  else if (! i->as64L())
+    op1_addr = (Bit32u) op1_addr;
+#endif
 
   /* pointer, segment address pair */
   op1_32 = read_RMW_virtual_dword(i->seg(), op1_addr);
@@ -151,7 +165,7 @@ void BX_CPU_C::BTS_EdGdM(bxInstruction_c *i)
   set_CF(bit_i);
 }
 
-void BX_CPU_C::BTS_EdGdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EdGdR(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
@@ -165,17 +179,24 @@ void BX_CPU_C::BTS_EdGdR(bxInstruction_c *i)
   BX_WRITE_32BIT_REGZ(i->rm(), op1_32);
 }
 
-void BX_CPU_C::BTR_EdGdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EdGdM(bxInstruction_c *i)
 {
   bx_address op1_addr;
   Bit32u op1_32, op2_32, index;
   Bit32s displacement32;
 
-  op2_32 = BX_READ_32BIT_REG(i->nnn());
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
+  op2_32 = BX_READ_32BIT_REG(i->nnn());
   index = op2_32 & 0x1f;
   displacement32 = ((Bit32s) (op2_32&0xffffffe0)) / 32;
   op1_addr = RMAddr(i) + 4 * displacement32;
+  if (! i->as32L())
+    op1_addr = (Bit16u) op1_addr;
+#if BX_SUPPORT_X86_64
+  else if (! i->as64L())
+    op1_addr = (Bit32u) op1_addr;
+#endif
 
   /* pointer, segment address pair */
   op1_32 = read_RMW_virtual_dword(i->seg(), op1_addr);
@@ -189,7 +210,7 @@ void BX_CPU_C::BTR_EdGdM(bxInstruction_c *i)
   set_CF(temp_cf);
 }
 
-void BX_CPU_C::BTR_EdGdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EdGdR(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
@@ -203,19 +224,27 @@ void BX_CPU_C::BTR_EdGdR(bxInstruction_c *i)
   BX_WRITE_32BIT_REGZ(i->rm(), op1_32);
 }
 
-void BX_CPU_C::BTC_EdGdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EdGdM(bxInstruction_c *i)
 {
   bx_address op1_addr;
   Bit32u op1_32, op2_32, index_32;
   Bit32s displacement32;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op2_32 = BX_READ_32BIT_REG(i->nnn());
   index_32 = op2_32 & 0x1f;
 
   displacement32 = ((Bit32s) (op2_32 & 0xffffffe0)) / 32;
   op1_addr = RMAddr(i) + 4 * displacement32;
-  op1_32 = read_RMW_virtual_dword(i->seg(), op1_addr);
+  if (! i->as32L())
+    op1_addr = (Bit16u) op1_addr;
+#if BX_SUPPORT_X86_64
+  else if (! i->as64L())
+    op1_addr = (Bit32u) op1_addr;
+#endif
 
+  op1_32 = read_RMW_virtual_dword(i->seg(), op1_addr);
   bx_bool temp_CF = (op1_32 >> index_32) & 0x01;
   op1_32 ^= (((Bit32u) 1) << index_32);  /* toggle bit */
   set_CF(temp_CF);
@@ -223,7 +252,7 @@ void BX_CPU_C::BTC_EdGdM(bxInstruction_c *i)
   write_RMW_virtual_dword(op1_32);
 }
 
-void BX_CPU_C::BTC_EdGdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EdGdR(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
@@ -238,15 +267,17 @@ void BX_CPU_C::BTC_EdGdR(bxInstruction_c *i)
   BX_WRITE_32BIT_REGZ(i->rm(), op1_32);
 }
 
-void BX_CPU_C::BT_EdIbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EdIbM(bxInstruction_c *i)
 {
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit32u op1_32 = read_virtual_dword(i->seg(), RMAddr(i));
   Bit8u  op2_8  = i->Ib() & 0x1f;
 
   set_CF((op1_32 >> op2_8) & 0x01);
 }
 
-void BX_CPU_C::BT_EdIbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EdIbR(bxInstruction_c *i)
 {
   Bit32u op1_32 = BX_READ_32BIT_REG(i->rm());
   Bit8u  op2_8  = i->Ib() & 0x1f;
@@ -254,10 +285,12 @@ void BX_CPU_C::BT_EdIbR(bxInstruction_c *i)
   set_CF((op1_32 >> op2_8) & 0x01);
 }
 
-void BX_CPU_C::BTS_EdIbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EdIbM(bxInstruction_c *i)
 {
   Bit8u op2_8 = i->Ib() & 0x1f;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit32u op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
   bx_bool temp_CF = (op1_32 >> op2_8) & 0x01;
   op1_32 |= (((Bit32u) 1) << op2_8);
@@ -266,7 +299,7 @@ void BX_CPU_C::BTS_EdIbM(bxInstruction_c *i)
   set_CF(temp_CF);
 }
 
-void BX_CPU_C::BTS_EdIbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EdIbR(bxInstruction_c *i)
 {
   Bit8u op2_8 = i->Ib() & 0x1f;
 
@@ -278,10 +311,12 @@ void BX_CPU_C::BTS_EdIbR(bxInstruction_c *i)
   set_CF(temp_CF);
 }
 
-void BX_CPU_C::BTC_EdIbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EdIbM(bxInstruction_c *i)
 {
   Bit8u op2_8 = i->Ib() & 0x1f;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit32u op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
   bx_bool temp_CF = (op1_32 >> op2_8) & 0x01;
   op1_32 ^= (((Bit32u) 1) << op2_8);  /* toggle bit */
@@ -290,7 +325,7 @@ void BX_CPU_C::BTC_EdIbM(bxInstruction_c *i)
   set_CF(temp_CF);
 }
 
-void BX_CPU_C::BTC_EdIbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EdIbR(bxInstruction_c *i)
 {
   Bit8u op2_8 = i->Ib() & 0x1f;
 
@@ -302,10 +337,12 @@ void BX_CPU_C::BTC_EdIbR(bxInstruction_c *i)
   set_CF(temp_CF);
 }
 
-void BX_CPU_C::BTR_EdIbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EdIbM(bxInstruction_c *i)
 {
   Bit8u op2_8 = i->Ib() & 0x1f;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit32u op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
   bx_bool temp_CF = (op1_32 >> op2_8) & 0x01;
   op1_32 &= ~(((Bit32u) 1) << op2_8);
@@ -314,7 +351,7 @@ void BX_CPU_C::BTR_EdIbM(bxInstruction_c *i)
   set_CF(temp_CF);
 }
 
-void BX_CPU_C::BTR_EdIbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EdIbR(bxInstruction_c *i)
 {
   Bit8u op2_8 = i->Ib() & 0x1f;
 
@@ -327,7 +364,7 @@ void BX_CPU_C::BTR_EdIbR(bxInstruction_c *i)
 }
 
 /* 0F B8 */
-void BX_CPU_C::POPCNT_GdEd(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::POPCNT_GdEd(bxInstruction_c *i)
 {
 #if BX_SUPPORT_POPCNT || (BX_SUPPORT_SSE >= 5) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
   Bit32u op1_32, op2_32;
@@ -337,6 +374,7 @@ void BX_CPU_C::POPCNT_GdEd(bxInstruction_c *i)
     op2_32 = BX_READ_32BIT_REG(i->rm());
   }
   else {
+    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
     op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
   }
