@@ -1,5 +1,5 @@
-/////////////////////////////////////////////////////////////////////////
-// $Id: ctrl_xfer64.cc,v 1.59 2007/12/21 17:30:49 sshwarts Exp $
+////////c/////////////////////////////////////////////////////////////////
+// $Id: ctrl_xfer64.cc,v 1.68 2008/05/11 20:46:11 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -25,22 +25,37 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 /////////////////////////////////////////////////////////////////////////
 
-
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
-
 #if BX_SUPPORT_X86_64
 
-void BX_CPU_C::RETnear64_Iw(bxInstruction_c *i)
+BX_CPP_INLINE void BX_CPP_AttrRegparmN(1) BX_CPU_C::branch_near64(bxInstruction_c *i)
+{
+  Bit64u new_RIP = RIP + (Bit32s) i->Id();
+
+  if (! IsCanonical(new_RIP)) {
+    BX_ERROR(("branch_near64: canonical RIP violation"));
+    exception(BX_GP_EXCEPTION, 0, 0);
+  }
+
+#if BX_SUPPORT_TRACE_CACHE && !defined(BX_TRACE_CACHE_NO_SPECULATIVE_TRACING)
+  // assert magic async_event to stop trace execution
+  BX_CPU_THIS_PTR async_event |= BX_ASYNC_EVENT_STOP_TRACE;
+#endif
+
+  RIP = new_RIP;
+}
+
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::RETnear64_Iw(bxInstruction_c *i)
 {
 #if BX_DEBUGGER
   BX_CPU_THIS_PTR show_flag |= Flag_ret;
 #endif
 
-  Bit64u return_RIP = read_virtual_qword(BX_SEG_REG_SS, RSP);
+  Bit64u return_RIP = read_virtual_qword_64(BX_SEG_REG_SS, RSP);
 
   if (! IsCanonical(return_RIP)) {
     BX_ERROR(("RETnear64_Iw: canonical RIP violation"));
@@ -53,16 +68,16 @@ void BX_CPU_C::RETnear64_Iw(bxInstruction_c *i)
   BX_INSTR_UCNEAR_BRANCH(BX_CPU_ID, BX_INSTR_IS_RET, RIP);
 }
 
-void BX_CPU_C::RETnear64(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::RETnear64(bxInstruction_c *i)
 {
 #if BX_DEBUGGER
   BX_CPU_THIS_PTR show_flag |= Flag_ret;
 #endif
 
-  Bit64u return_RIP = read_virtual_qword(BX_SEG_REG_SS, RSP);
+  Bit64u return_RIP = read_virtual_qword_64(BX_SEG_REG_SS, RSP);
 
   if (! IsCanonical(return_RIP)) {
-    BX_ERROR(("RETnear64_Iw: canonical RIP violation"));
+    BX_ERROR(("RETnear64: canonical RIP violation %08x%08x", GET32H(return_RIP), GET32L(return_RIP)));
     exception(BX_GP_EXCEPTION, 0, 0);
   }
 
@@ -72,7 +87,7 @@ void BX_CPU_C::RETnear64(bxInstruction_c *i)
   BX_INSTR_UCNEAR_BRANCH(BX_CPU_ID, BX_INSTR_IS_RET, RIP);
 }
 
-void BX_CPU_C::RETfar64_Iw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::RETfar64_Iw(bxInstruction_c *i)
 {
   invalidate_prefetch_q();
 
@@ -94,7 +109,7 @@ void BX_CPU_C::RETfar64_Iw(bxInstruction_c *i)
                       BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value, RIP);
 }
 
-void BX_CPU_C::RETfar64(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::RETfar64(bxInstruction_c *i)
 {
   invalidate_prefetch_q();
 
@@ -116,7 +131,7 @@ void BX_CPU_C::RETfar64(bxInstruction_c *i)
                       BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value, RIP);
 }
 
-void BX_CPU_C::CALL_Jq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CALL_Jq(bxInstruction_c *i)
 {
   Bit64u new_RIP = RIP + (Bit32s) i->Id();
 
@@ -136,20 +151,15 @@ void BX_CPU_C::CALL_Jq(bxInstruction_c *i)
   BX_INSTR_UCNEAR_BRANCH(BX_CPU_ID, BX_INSTR_IS_CALL, RIP);
 }
 
-void BX_CPU_C::CALL_Eq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CALL_EqM(bxInstruction_c *i)
 {
-  Bit64u op1_64;
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
 #if BX_DEBUGGER
   BX_CPU_THIS_PTR show_flag |= Flag_call;
 #endif
 
-  if (i->modC0()) {
-    op1_64 = BX_READ_64BIT_REG(i->rm());
-  }
-  else {
-    op1_64 = read_virtual_qword(i->seg(), RMAddr(i));
-  }
+  Bit64u op1_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
 
   if (! IsCanonical(op1_64))
   {
@@ -163,20 +173,39 @@ void BX_CPU_C::CALL_Eq(bxInstruction_c *i)
   BX_INSTR_UCNEAR_BRANCH(BX_CPU_ID, BX_INSTR_IS_CALL, RIP);
 }
 
-void BX_CPU_C::CALL64_Ep(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CALL_EqR(bxInstruction_c *i)
 {
-  Bit16u cs_raw;
-  Bit32u op1_32;
+#if BX_DEBUGGER
+  BX_CPU_THIS_PTR show_flag |= Flag_call;
+#endif
 
+  Bit64u op1_64 = BX_READ_64BIT_REG(i->rm());
+
+  if (! IsCanonical(op1_64))
+  {
+    BX_ERROR(("CALL_Eq: canonical RIP violation"));
+    exception(BX_GP_EXCEPTION, 0, 0);
+  }
+
+  push_64(RIP);
+  RIP = op1_64;
+
+  BX_INSTR_UCNEAR_BRANCH(BX_CPU_ID, BX_INSTR_IS_CALL, RIP);
+}
+
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CALL64_Ep(bxInstruction_c *i)
+{
   invalidate_prefetch_q();
 
 #if BX_DEBUGGER
   BX_CPU_THIS_PTR show_flag |= Flag_call;
 #endif
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   /* pointer, segment address pair */
-  op1_32 = read_virtual_dword(i->seg(), RMAddr(i));
-  cs_raw = read_virtual_word (i->seg(), RMAddr(i)+8);
+  Bit64u op1_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
+  Bit16u cs_raw = read_virtual_word_64(i->seg(), RMAddr(i)+8);
 
   BX_ASSERT(protected_mode());
 
@@ -184,7 +213,7 @@ void BX_CPU_C::CALL64_Ep(bxInstruction_c *i)
   BX_CPU_THIS_PTR prev_rsp = RSP;
 
   // call_protected is not RSP safe
-  call_protected(i, cs_raw, op1_32);
+  call_protected(i, cs_raw, op1_64);
 
   BX_CPU_THIS_PTR speculative_rsp = 0;
 
@@ -192,13 +221,21 @@ void BX_CPU_C::CALL64_Ep(bxInstruction_c *i)
                       BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value, RIP);
 }
 
-void BX_CPU_C::JMP_Jq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::JMP_Jq(bxInstruction_c *i)
 {
-  branch_near64(i);
+  Bit64u new_RIP = RIP + (Bit32s) i->Id();
+
+  if (! IsCanonical(new_RIP)) {
+    BX_ERROR(("JMP_Jq: canonical RIP violation"));
+    exception(BX_GP_EXCEPTION, 0, 0);
+  }
+
+  RIP = new_RIP;
+
   BX_INSTR_UCNEAR_BRANCH(BX_CPU_ID, BX_INSTR_IS_JMP, RIP);
 }
 
-void BX_CPU_C::JO_Jq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::JO_Jq(bxInstruction_c *i)
 {
   if (get_OF()) {
     branch_near64(i);
@@ -208,7 +245,7 @@ void BX_CPU_C::JO_Jq(bxInstruction_c *i)
   BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
 }
 
-void BX_CPU_C::JNO_Jq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::JNO_Jq(bxInstruction_c *i)
 {
   if (! get_OF()) {
     branch_near64(i);
@@ -218,7 +255,7 @@ void BX_CPU_C::JNO_Jq(bxInstruction_c *i)
   BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
 }
 
-void BX_CPU_C::JB_Jq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::JB_Jq(bxInstruction_c *i)
 {
   if (get_CF()) {
     branch_near64(i);
@@ -228,7 +265,7 @@ void BX_CPU_C::JB_Jq(bxInstruction_c *i)
   BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
 }
 
-void BX_CPU_C::JNB_Jq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::JNB_Jq(bxInstruction_c *i)
 {
   if (! get_CF()) {
     branch_near64(i);
@@ -238,7 +275,7 @@ void BX_CPU_C::JNB_Jq(bxInstruction_c *i)
   BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
 }
 
-void BX_CPU_C::JZ_Jq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::JZ_Jq(bxInstruction_c *i)
 {
   if (get_ZF()) {
     branch_near64(i);
@@ -248,7 +285,7 @@ void BX_CPU_C::JZ_Jq(bxInstruction_c *i)
   BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
 }
 
-void BX_CPU_C::JNZ_Jq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::JNZ_Jq(bxInstruction_c *i)
 {
   if (! get_ZF()) {
     branch_near64(i);
@@ -258,7 +295,7 @@ void BX_CPU_C::JNZ_Jq(bxInstruction_c *i)
   BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
 }
 
-void BX_CPU_C::JBE_Jq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::JBE_Jq(bxInstruction_c *i)
 {
   if (get_CF() || get_ZF()) {
     branch_near64(i);
@@ -268,7 +305,7 @@ void BX_CPU_C::JBE_Jq(bxInstruction_c *i)
   BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
 }
 
-void BX_CPU_C::JNBE_Jq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::JNBE_Jq(bxInstruction_c *i)
 {
   if (! (get_CF() || get_ZF())) {
     branch_near64(i);
@@ -278,7 +315,7 @@ void BX_CPU_C::JNBE_Jq(bxInstruction_c *i)
   BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
 }
 
-void BX_CPU_C::JS_Jq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::JS_Jq(bxInstruction_c *i)
 {
   if (get_SF()) {
     branch_near64(i);
@@ -288,7 +325,7 @@ void BX_CPU_C::JS_Jq(bxInstruction_c *i)
   BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
 }
 
-void BX_CPU_C::JNS_Jq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::JNS_Jq(bxInstruction_c *i)
 {
   if (! get_SF()) {
     branch_near64(i);
@@ -298,7 +335,7 @@ void BX_CPU_C::JNS_Jq(bxInstruction_c *i)
   BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
 }
 
-void BX_CPU_C::JP_Jq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::JP_Jq(bxInstruction_c *i)
 {
   if (get_PF()) {
     branch_near64(i);
@@ -308,7 +345,7 @@ void BX_CPU_C::JP_Jq(bxInstruction_c *i)
   BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
 }
 
-void BX_CPU_C::JNP_Jq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::JNP_Jq(bxInstruction_c *i)
 {
   if (! get_PF()) {
     branch_near64(i);
@@ -318,7 +355,7 @@ void BX_CPU_C::JNP_Jq(bxInstruction_c *i)
   BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
 }
 
-void BX_CPU_C::JL_Jq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::JL_Jq(bxInstruction_c *i)
 {
   if (getB_SF() != getB_OF()) {
     branch_near64(i);
@@ -328,7 +365,7 @@ void BX_CPU_C::JL_Jq(bxInstruction_c *i)
   BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
 }
 
-void BX_CPU_C::JNL_Jq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::JNL_Jq(bxInstruction_c *i)
 {
   if (getB_SF() == getB_OF()) {
     branch_near64(i);
@@ -338,7 +375,7 @@ void BX_CPU_C::JNL_Jq(bxInstruction_c *i)
   BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
 }
 
-void BX_CPU_C::JLE_Jq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::JLE_Jq(bxInstruction_c *i)
 {
   if (get_ZF() || (getB_SF() != getB_OF())) {
     branch_near64(i);
@@ -348,7 +385,7 @@ void BX_CPU_C::JLE_Jq(bxInstruction_c *i)
   BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
 }
 
-void BX_CPU_C::JNLE_Jq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::JNLE_Jq(bxInstruction_c *i)
 {
   if (! get_ZF() && (getB_SF() == getB_OF())) {
     branch_near64(i);
@@ -358,17 +395,26 @@ void BX_CPU_C::JNLE_Jq(bxInstruction_c *i)
   BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(BX_CPU_ID);
 }
 
-void BX_CPU_C::JMP_Eq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::JMP_EqM(bxInstruction_c *i)
 {
-  Bit64u op1_64;
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  if (i->modC0()) {
-    op1_64 = BX_READ_64BIT_REG(i->rm());
-  }
-  else {
-    op1_64 = read_virtual_qword(i->seg(), RMAddr(i));
+  Bit64u op1_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
+
+  if (! IsCanonical(op1_64)) {
+    BX_ERROR(("JMP_Eq: canonical RIP violation"));
+    exception(BX_GP_EXCEPTION, 0, 0);
   }
 
+  RIP = op1_64;
+
+  BX_INSTR_UCNEAR_BRANCH(BX_CPU_ID, BX_INSTR_IS_JMP, RIP);
+}
+
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::JMP_EqR(bxInstruction_c *i)
+{
+  Bit64u op1_64 = BX_READ_64BIT_REG(i->rm());
+
   if (! IsCanonical(op1_64)) {
     BX_ERROR(("JMP_Eq: canonical RIP violation"));
     exception(BX_GP_EXCEPTION, 0, 0);
@@ -380,25 +426,24 @@ void BX_CPU_C::JMP_Eq(bxInstruction_c *i)
 }
 
 /* Far indirect jump */
-void BX_CPU_C::JMP64_Ep(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::JMP64_Ep(bxInstruction_c *i)
 {
-  Bit16u cs_raw;
-  Bit32u op1_32;
-
   invalidate_prefetch_q();
 
-  op1_32 = read_virtual_dword(i->seg(), RMAddr(i));
-  cs_raw = read_virtual_word (i->seg(), RMAddr(i)+4);
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  Bit64u op1_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
+  Bit16u cs_raw = read_virtual_word_64(i->seg(), RMAddr(i)+8);
 
   BX_ASSERT(protected_mode());
 
-  jump_protected(i, cs_raw, op1_32);
+  jump_protected(i, cs_raw, op1_64);
 
   BX_INSTR_FAR_BRANCH(BX_CPU_ID, BX_INSTR_IS_JMP,
                       BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value, RIP);
 }
 
-void BX_CPU_C::IRET64(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::IRET64(bxInstruction_c *i)
 {
   invalidate_prefetch_q();
 
@@ -422,7 +467,7 @@ void BX_CPU_C::IRET64(bxInstruction_c *i)
                       BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value, RIP);
 }
 
-void BX_CPU_C::JCXZ64_Jb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::JCXZ64_Jb(bxInstruction_c *i)
 {
   if (i->as64L()) {
     if (RCX == 0) {
@@ -446,13 +491,13 @@ void BX_CPU_C::JCXZ64_Jb(bxInstruction_c *i)
 // There is some weirdness in LOOP instructions definition. If an exception
 // was generated during the instruction execution (for example #GP fault
 // because EIP was beyond CS segment limits) CPU state should restore the
-// state prior to instruction execution. 
+// state prior to instruction execution.
 //
 // The final point that we are not allowed to decrement ECX register before
 // it is known that no exceptions can happen.
 //
 
-void BX_CPU_C::LOOPNE64_Jb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::LOOPNE64_Jb(bxInstruction_c *i)
 {
   if (i->as64L()) {
     Bit64u count = RCX;
@@ -486,7 +531,7 @@ void BX_CPU_C::LOOPNE64_Jb(bxInstruction_c *i)
   }
 }
 
-void BX_CPU_C::LOOPE64_Jb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::LOOPE64_Jb(bxInstruction_c *i)
 {
   if (i->as64L()) {
     Bit64u count = RCX;
@@ -520,7 +565,7 @@ void BX_CPU_C::LOOPE64_Jb(bxInstruction_c *i)
   }
 }
 
-void BX_CPU_C::LOOP64_Jb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::LOOP64_Jb(bxInstruction_c *i)
 {
   if (i->as64L()) {
     Bit64u count = RCX;
