@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: data_xfer32.cc,v 1.51 2007/12/21 18:24:19 sshwarts Exp $
+// $Id: data_xfer32.cc,v 1.55 2008/03/22 21:29:39 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -35,7 +35,7 @@
 #define RAX EAX
 #endif
 
-void BX_CPU_C::XCHG_ERXEAX(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XCHG_ERXEAX(bxInstruction_c *i)
 {
 #if BX_SUPPORT_X86_64
   if (i->opcodeReg() == 0)  // 'xchg eax, eax' is NOP even in 64-bit mode
@@ -47,66 +47,71 @@ void BX_CPU_C::XCHG_ERXEAX(bxInstruction_c *i)
   BX_WRITE_32BIT_REGZ(i->opcodeReg(), temp32);
 }
 
-void BX_CPU_C::MOV_ERXId(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_ERXId(bxInstruction_c *i)
 {
   BX_WRITE_32BIT_REGZ(i->opcodeReg(), i->Id());
 }
 
-void BX_CPU_C::MOV_EdGdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_EdGdM(bxInstruction_c *i)
 {
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   write_virtual_dword(i->seg(), RMAddr(i), BX_READ_32BIT_REG(i->nnn()));
 }
 
-void BX_CPU_C::MOV_EdGdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_EdGdR(bxInstruction_c *i)
 {
   BX_WRITE_32BIT_REGZ(i->rm(), BX_READ_32BIT_REG(i->nnn()));
 }
 
-void BX_CPU_C::MOV_GdEdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_GdEdR(bxInstruction_c *i)
 {
   BX_WRITE_32BIT_REGZ(i->nnn(), BX_READ_32BIT_REG(i->rm()));
 }
 
-void BX_CPU_C::MOV_GdEdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_GdEdM(bxInstruction_c *i)
 {
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit32u val32 = read_virtual_dword(i->seg(), RMAddr(i));
   BX_WRITE_32BIT_REGZ(i->nnn(), val32);
 }
 
-void BX_CPU_C::LEA_GdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::LEA_GdM(bxInstruction_c *i)
 {
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   BX_WRITE_32BIT_REGZ(i->nnn(), RMAddr(i));
 }
 
-void BX_CPU_C::MOV_EAXOd(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_EAXOd(bxInstruction_c *i)
 {
   RAX = read_virtual_dword(i->seg(), i->Id());
 }
 
-void BX_CPU_C::MOV_OdEAX(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_OdEAX(bxInstruction_c *i)
 {
   write_virtual_dword(i->seg(), i->Id(), EAX);
 }
 
-void BX_CPU_C::MOV_EdIdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_EdIdM(bxInstruction_c *i)
 {
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   write_virtual_dword(i->seg(), RMAddr(i), i->Id());
 }
 
-void BX_CPU_C::MOV_EdIdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVZX_GdEbM(bxInstruction_c *i)
 {
-  BX_WRITE_32BIT_REGZ(i->rm(), i->Id());
-}
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-void BX_CPU_C::MOVZX_GdEbM(bxInstruction_c *i)
-{
   Bit8u op2_8 = read_virtual_byte(i->seg(), RMAddr(i));
 
   /* zero extend byte op2 into dword op1 */
   BX_WRITE_32BIT_REGZ(i->nnn(), (Bit32u) op2_8);
 }
 
-void BX_CPU_C::MOVZX_GdEbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVZX_GdEbR(bxInstruction_c *i)
 {
   Bit8u op2_8 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
 
@@ -114,15 +119,17 @@ void BX_CPU_C::MOVZX_GdEbR(bxInstruction_c *i)
   BX_WRITE_32BIT_REGZ(i->nnn(), (Bit32u) op2_8);
 }
 
-void BX_CPU_C::MOVZX_GdEwM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVZX_GdEwM(bxInstruction_c *i)
 {
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit16u op2_16 = read_virtual_word(i->seg(), RMAddr(i));
 
   /* zero extend word op2 into dword op1 */
   BX_WRITE_32BIT_REGZ(i->nnn(), (Bit32u) op2_16);
 }
 
-void BX_CPU_C::MOVZX_GdEwR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVZX_GdEwR(bxInstruction_c *i)
 {
   Bit16u op2_16 = BX_READ_16BIT_REG(i->rm());
 
@@ -130,15 +137,17 @@ void BX_CPU_C::MOVZX_GdEwR(bxInstruction_c *i)
   BX_WRITE_32BIT_REGZ(i->nnn(), (Bit32u) op2_16);
 }
 
-void BX_CPU_C::MOVSX_GdEbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSX_GdEbM(bxInstruction_c *i)
 {
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit8u op2_8 = read_virtual_byte(i->seg(), RMAddr(i));
 
   /* sign extend byte op2 into dword op1 */
   BX_WRITE_32BIT_REGZ(i->nnn(), (Bit8s) op2_8);
 }
 
-void BX_CPU_C::MOVSX_GdEbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSX_GdEbR(bxInstruction_c *i)
 {
   Bit8u op2_8 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
 
@@ -146,15 +155,17 @@ void BX_CPU_C::MOVSX_GdEbR(bxInstruction_c *i)
   BX_WRITE_32BIT_REGZ(i->nnn(), (Bit8s) op2_8);
 }
 
-void BX_CPU_C::MOVSX_GdEwM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSX_GdEwM(bxInstruction_c *i)
 {
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit16u op2_16 = read_virtual_word(i->seg(), RMAddr(i));
 
   /* sign extend word op2 into dword op1 */
   BX_WRITE_32BIT_REGZ(i->nnn(), (Bit16s) op2_16);
 }
 
-void BX_CPU_C::MOVSX_GdEwR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSX_GdEwR(bxInstruction_c *i)
 {
   Bit16u op2_16 = BX_READ_16BIT_REG(i->rm());
 
@@ -162,10 +173,12 @@ void BX_CPU_C::MOVSX_GdEwR(bxInstruction_c *i)
   BX_WRITE_32BIT_REGZ(i->nnn(), (Bit16s) op2_16);
 }
 
-void BX_CPU_C::XCHG_EdGdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XCHG_EdGdM(bxInstruction_c *i)
 {
   Bit32u op2_32, op1_32;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
   op2_32 = BX_READ_32BIT_REG(i->nnn());
   write_RMW_virtual_dword(op2_32);
@@ -173,7 +186,7 @@ void BX_CPU_C::XCHG_EdGdM(bxInstruction_c *i)
   BX_WRITE_32BIT_REGZ(i->nnn(), op1_32);
 }
 
-void BX_CPU_C::XCHG_EdGdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XCHG_EdGdR(bxInstruction_c *i)
 {
   Bit32u op1_32 = BX_READ_32BIT_REG(i->rm());
   Bit32u op2_32 = BX_READ_32BIT_REG(i->nnn());
@@ -186,9 +199,11 @@ void BX_CPU_C::XCHG_EdGdR(bxInstruction_c *i)
 //       of whether condition is true or not.  Thus, exceptions may
 //       occur even if the MOV does not take place.
 
-void BX_CPU_C::CMOVO_GdEdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVO_GdEdM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
 
   if (get_OF())
@@ -201,7 +216,7 @@ void BX_CPU_C::CMOVO_GdEdM(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVO_GdEdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVO_GdEdR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   if (get_OF())
@@ -214,14 +229,16 @@ void BX_CPU_C::CMOVO_GdEdR(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVNO_GdEdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNO_GdEdM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
 
   if (!get_OF())
     BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-  
+
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVNO_GdEd: --enable-cpu-level=6 required"));
@@ -229,12 +246,12 @@ void BX_CPU_C::CMOVNO_GdEdM(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVNO_GdEdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNO_GdEdR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   if (!get_OF())
     BX_WRITE_32BIT_REGZ(i->nnn(), BX_READ_32BIT_REG(i->rm()));
-  
+
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVNO_GdEd: --enable-cpu-level=6 required"));
@@ -242,14 +259,16 @@ void BX_CPU_C::CMOVNO_GdEdR(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVB_GdEdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVB_GdEdM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
 
   if (get_CF())
     BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-  
+
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVB_GdEd: --enable-cpu-level=6 required"));
@@ -257,12 +276,12 @@ void BX_CPU_C::CMOVB_GdEdM(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVB_GdEdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVB_GdEdR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   if (get_CF())
     BX_WRITE_32BIT_REGZ(i->nnn(), BX_READ_32BIT_REG(i->rm()));
-  
+
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVB_GdEd: --enable-cpu-level=6 required"));
@@ -270,14 +289,16 @@ void BX_CPU_C::CMOVB_GdEdR(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVNB_GdEdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNB_GdEdM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
 
   if (!get_CF())
     BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-  
+
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVNB_GdEd: --enable-cpu-level=6 required"));
@@ -285,12 +306,12 @@ void BX_CPU_C::CMOVNB_GdEdM(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVNB_GdEdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNB_GdEdR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   if (!get_CF())
     BX_WRITE_32BIT_REGZ(i->nnn(), BX_READ_32BIT_REG(i->rm()));
-  
+
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVNB_GdEd: --enable-cpu-level=6 required"));
@@ -298,14 +319,16 @@ void BX_CPU_C::CMOVNB_GdEdR(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVZ_GdEdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVZ_GdEdM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
 
   if (get_ZF())
     BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-  
+
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVZ_GdEd: --enable-cpu-level=6 required"));
@@ -313,12 +336,12 @@ void BX_CPU_C::CMOVZ_GdEdM(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVZ_GdEdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVZ_GdEdR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   if (get_ZF())
     BX_WRITE_32BIT_REGZ(i->nnn(), BX_READ_32BIT_REG(i->rm()));
-  
+
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVZ_GdEd: --enable-cpu-level=6 required"));
@@ -326,14 +349,16 @@ void BX_CPU_C::CMOVZ_GdEdR(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVNZ_GdEdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNZ_GdEdM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
 
   if (!get_ZF())
     BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-  
+
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVNZ_GdEd: --enable-cpu-level=6 required"));
@@ -341,12 +366,12 @@ void BX_CPU_C::CMOVNZ_GdEdM(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVNZ_GdEdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNZ_GdEdR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   if (!get_ZF())
     BX_WRITE_32BIT_REGZ(i->nnn(), BX_READ_32BIT_REG(i->rm()));
-  
+
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVNZ_GdEd: --enable-cpu-level=6 required"));
@@ -354,14 +379,16 @@ void BX_CPU_C::CMOVNZ_GdEdR(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVBE_GdEdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVBE_GdEdM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
 
   if (get_CF() || get_ZF())
     BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-  
+
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVBE_GdEd: --enable-cpu-level=6 required"));
@@ -369,12 +396,12 @@ void BX_CPU_C::CMOVBE_GdEdM(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVBE_GdEdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVBE_GdEdR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   if (get_CF() || get_ZF())
     BX_WRITE_32BIT_REGZ(i->nnn(), BX_READ_32BIT_REG(i->rm()));
-  
+
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVBE_GdEd: --enable-cpu-level=6 required"));
@@ -382,14 +409,16 @@ void BX_CPU_C::CMOVBE_GdEdR(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVNBE_GdEdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNBE_GdEdM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
 
   if (! (get_CF() || get_ZF()))
     BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-  
+
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVNBE_GdEd: --enable-cpu-level=6 required"));
@@ -397,7 +426,7 @@ void BX_CPU_C::CMOVNBE_GdEdM(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVNBE_GdEdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNBE_GdEdR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   if (! (get_CF() || get_ZF()))
@@ -410,9 +439,11 @@ void BX_CPU_C::CMOVNBE_GdEdR(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVS_GdEdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVS_GdEdM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
 
   if (get_SF())
@@ -425,7 +456,7 @@ void BX_CPU_C::CMOVS_GdEdM(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVS_GdEdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVS_GdEdR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   if (get_SF())
@@ -438,9 +469,11 @@ void BX_CPU_C::CMOVS_GdEdR(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVNS_GdEdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNS_GdEdM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
 
   if (!get_SF())
@@ -453,7 +486,7 @@ void BX_CPU_C::CMOVNS_GdEdM(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVNS_GdEdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNS_GdEdR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   if (!get_SF())
@@ -466,9 +499,11 @@ void BX_CPU_C::CMOVNS_GdEdR(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVP_GdEdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVP_GdEdM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
 
   if (get_PF())
@@ -481,7 +516,7 @@ void BX_CPU_C::CMOVP_GdEdM(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVP_GdEdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVP_GdEdR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   if (get_PF())
@@ -494,9 +529,11 @@ void BX_CPU_C::CMOVP_GdEdR(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVNP_GdEdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNP_GdEdM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
 
   if (!get_PF())
@@ -509,7 +546,7 @@ void BX_CPU_C::CMOVNP_GdEdM(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVNP_GdEdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNP_GdEdR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   if (!get_PF())
@@ -522,9 +559,11 @@ void BX_CPU_C::CMOVNP_GdEdR(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVL_GdEdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVL_GdEdM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
 
   if (getB_SF() != getB_OF())
@@ -537,7 +576,7 @@ void BX_CPU_C::CMOVL_GdEdM(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVL_GdEdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVL_GdEdR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   if (getB_SF() != getB_OF())
@@ -550,9 +589,11 @@ void BX_CPU_C::CMOVL_GdEdR(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVNL_GdEdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNL_GdEdM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
 
   if (getB_SF() == getB_OF())
@@ -565,7 +606,7 @@ void BX_CPU_C::CMOVNL_GdEdM(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVNL_GdEdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNL_GdEdR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   if (getB_SF() == getB_OF())
@@ -578,9 +619,11 @@ void BX_CPU_C::CMOVNL_GdEdR(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVLE_GdEdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVLE_GdEdM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
 
   if (get_ZF() || (getB_SF() != getB_OF()))
@@ -593,7 +636,7 @@ void BX_CPU_C::CMOVLE_GdEdM(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVLE_GdEdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVLE_GdEdR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   if (get_ZF() || (getB_SF() != getB_OF()))
@@ -606,9 +649,11 @@ void BX_CPU_C::CMOVLE_GdEdR(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVNLE_GdEdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNLE_GdEdM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
 
   if (! get_ZF() && (getB_SF() == getB_OF()))
@@ -621,7 +666,7 @@ void BX_CPU_C::CMOVNLE_GdEdM(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::CMOVNLE_GdEdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNLE_GdEdR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
   if (! get_ZF() && (getB_SF() == getB_OF()))
