@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: flag_ctrl.cc,v 1.33 2007/12/20 18:29:38 sshwarts Exp $
+// $Id: flag_ctrl.cc,v 1.38 2008/04/08 17:58:56 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -23,8 +23,7 @@
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
-
-
+/////////////////////////////////////////////////////////////////////////
 
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
@@ -36,8 +35,7 @@
 #define RSP ESP
 #endif
 
-
-void BX_CPU_C::SAHF(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SAHF(bxInstruction_c *i)
 {
   set_SF((AH & 0x80) >> 7);
   set_ZF((AH & 0x40) >> 6);
@@ -46,22 +44,22 @@ void BX_CPU_C::SAHF(bxInstruction_c *i)
   set_PF((AH & 0x04) >> 2);
 }
 
-void BX_CPU_C::LAHF(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::LAHF(bxInstruction_c *i)
 {
-  AH = read_flags() & 0xFF;
+  AH = read_eflags() & 0xFF;
 }
 
-void BX_CPU_C::CLC(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CLC(bxInstruction_c *i)
 {
   clear_CF();
 }
 
-void BX_CPU_C::STC(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::STC(bxInstruction_c *i)
 {
   assert_CF();
 }
 
-void BX_CPU_C::CLI(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CLI(bxInstruction_c *i)
 {
   Bit32u IOPL = BX_CPU_THIS_PTR get_IOPL();
   Bit32u  cpl = CPL;
@@ -77,7 +75,7 @@ void BX_CPU_C::CLI(bxInstruction_c *i)
         return;
       }
     }
-    else 
+    else
 #endif
     {
       if (IOPL < cpl) {
@@ -106,7 +104,7 @@ void BX_CPU_C::CLI(bxInstruction_c *i)
   BX_CPU_THIS_PTR clear_IF();
 }
 
-void BX_CPU_C::STI(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::STI(bxInstruction_c *i)
 {
   Bit32u IOPL = BX_CPU_THIS_PTR get_IOPL();
   Bit32u  cpl = CPL;
@@ -159,24 +157,24 @@ void BX_CPU_C::STI(bxInstruction_c *i)
   }
 }
 
-void BX_CPU_C::CLD(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CLD(bxInstruction_c *i)
 {
   BX_CPU_THIS_PTR clear_DF();
 }
 
-void BX_CPU_C::STD(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::STD(bxInstruction_c *i)
 {
   BX_CPU_THIS_PTR assert_DF();
 }
 
-void BX_CPU_C::CMC(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMC(bxInstruction_c *i)
 {
   set_CF(! get_CF());
 }
 
-void BX_CPU_C::PUSHF_Fw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::PUSHF_Fw(bxInstruction_c *i)
 {
-  Bit16u flags = read_flags();
+  Bit16u flags = (Bit16u) read_eflags();
 
   if (v8086_mode()) {
     if ((BX_CPU_THIS_PTR get_IOPL() < 3) && (CR4_VME_ENABLED == 0)) {
@@ -197,7 +195,7 @@ void BX_CPU_C::PUSHF_Fw(bxInstruction_c *i)
   push_16(flags);
 }
 
-void BX_CPU_C::POPF_Fw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::POPF_Fw(bxInstruction_c *i)
 {
   // Build a mask of the following bits:
   // x,NT,IOPL,OF,DF,IF,TF,SF,ZF,x,AF,x,PF,x,CF
@@ -225,7 +223,7 @@ void BX_CPU_C::POPF_Fw(bxInstruction_c *i)
     flags16 = pop_16();
 #if BX_SUPPORT_VME
     if (CR4_VME_ENABLED && BX_CPU_THIS_PTR get_IOPL() < 3) {
-      if (((flags16 & EFlagsIFMask) && BX_CPU_THIS_PTR get_VIP()) || 
+      if (((flags16 & EFlagsIFMask) && BX_CPU_THIS_PTR get_VIP()) ||
            (flags16 & EFlagsTFMask))
       {
         BX_DEBUG(("POPFW: #GP(0) in VME mode"));
@@ -255,7 +253,7 @@ void BX_CPU_C::POPF_Fw(bxInstruction_c *i)
 
 #if BX_CPU_LEVEL >= 3
 
-void BX_CPU_C::PUSHF_Fd(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::PUSHF_Fd(bxInstruction_c *i)
 {
   if (v8086_mode() && (BX_CPU_THIS_PTR get_IOPL()<3)) {
     BX_DEBUG(("PUSHFD: #GP(0) in v8086 mode"));
@@ -266,11 +264,11 @@ void BX_CPU_C::PUSHF_Fd(bxInstruction_c *i)
   push_32(read_eflags() & 0x00fcffff);
 }
 
-void BX_CPU_C::POPF_Fd(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::POPF_Fd(bxInstruction_c *i)
 {
   // Build a mask of the following bits:
   // ID,VIP,VIF,AC,VM,RF,x,NT,IOPL,OF,DF,IF,TF,SF,ZF,x,AF,x,PF,x,CF
-  Bit32u changeMask = EFlagsOSZAPCMask | EFlagsTFMask | 
+  Bit32u changeMask = EFlagsOSZAPCMask | EFlagsTFMask |
                           EFlagsDFMask | EFlagsNTMask | EFlagsRFMask;
 #if BX_CPU_LEVEL >= 4
   changeMask |= (EFlagsIDMask | EFlagsACMask);  // ID/AC
@@ -306,13 +304,13 @@ void BX_CPU_C::POPF_Fd(bxInstruction_c *i)
 }
 
 #if BX_SUPPORT_X86_64
-void BX_CPU_C::PUSHF_Fq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::PUSHF_Fq(bxInstruction_c *i)
 {
   // VM & RF flags cleared in image stored on the stack
   push_64(read_eflags() & 0x00fcffff);
 }
 
-void BX_CPU_C::POPF_Fq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::POPF_Fq(bxInstruction_c *i)
 {
   // Build a mask of the following bits:
   // ID,VIP,VIF,AC,VM,RF,x,NT,IOPL,OF,DF,IF,TF,SF,ZF,x,AF,x,PF,x,CF
@@ -322,7 +320,7 @@ void BX_CPU_C::POPF_Fq(bxInstruction_c *i)
 
   BX_ASSERT (protected_mode());
 
-  Bit32u eflags = (Bit32u) pop_64();
+  Bit32u eflags32 = (Bit32u) pop_64();
 
   if (CPL==0)
     changeMask |= EFlagsIOPLMask;
@@ -330,13 +328,13 @@ void BX_CPU_C::POPF_Fq(bxInstruction_c *i)
     changeMask |= EFlagsIFMask;
 
   // VIF, VIP, VM are unaffected
-  writeEFlags(eflags, changeMask);
+  writeEFlags(eflags32, changeMask);
 }
 #endif
 
 #endif  // BX_CPU_LEVEL >= 3
 
-void BX_CPU_C::SALC(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SALC(bxInstruction_c *i)
 {
   if (get_CF()) {
     AL = 0xff;
