@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: flag_ctrl_pro.cc,v 1.29 2007/11/20 21:22:03 sshwarts Exp $
+// $Id: flag_ctrl_pro.cc,v 1.34 2008/04/25 08:19:36 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -23,14 +23,14 @@
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
-
+//
+/////////////////////////////////////////////////////////////////////////
 
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::setEFlags(Bit32u val)
 {
   // VM flag could not be set from long mode
@@ -41,13 +41,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::setEFlags(Bit32u val)
   }
 #endif
 
+  BX_CPU_THIS_PTR eflags = val;
+  BX_CPU_THIS_PTR lf_flags_status = 0; // OSZAPC flags are known.
+
 #if BX_CPU_LEVEL >= 4 && BX_SUPPORT_ALIGNMENT_CHECK
   handleAlignmentCheck();
 #endif
 
-  BX_CPU_THIS_PTR eflags.val32 = val;
-  BX_CPU_THIS_PTR lf_flags_status = 0; // OSZAPC flags are known.
-  BX_CPU_THIS_PTR set_VM(val & EFlagsVMMask);
+  handleCpuModeChange(); // VM flag might be changed
 }
 
   void BX_CPP_AttrRegparmN(2)
@@ -66,7 +67,7 @@ BX_CPU_C::writeEFlags(Bit32u flags, Bit32u changeMask)
   // Screen out changing of any unsupported bits.
   changeMask &= supportMask;
 
-  Bit32u newEFlags = (BX_CPU_THIS_PTR eflags.val32 & ~changeMask) |
+  Bit32u newEFlags = (BX_CPU_THIS_PTR eflags & ~changeMask) |
               (flags & changeMask);
   setEFlags(newEFlags);
   // OSZAPC flags are known - done in setEFlags(newEFlags)
@@ -110,40 +111,5 @@ Bit32u BX_CPU_C::force_flags(void)
     setEFlagsOSZAPC(newflags);
   }
 
-  return BX_CPU_THIS_PTR eflags.val32;
+  return BX_CPU_THIS_PTR eflags;
 }
-
-Bit16u BX_CPU_C::read_flags(void)
-{
-  Bit16u flags16 = (Bit16u) BX_CPU_THIS_PTR force_flags();
-
-  /* 8086: bits 12-15 always set to 1.
-   * 286: in real mode, bits 12-15 always cleared.
-   * 386+: real-mode: bit15 cleared, bits 14..12 are last loaded value
-   *       protected-mode: bit 15 clear, bit 14 = last loaded, IOPL?
-   */
-
-#if BX_CPU_LEVEL < 2
-  flags16 |= 0xF000;    /* 8086 nature */
-#elif BX_CPU_LEVEL == 2
-  if (real_mode()) {
-    flags16 &= 0x0FFF;  /* 80286 in real mode nature */
-  }
-#endif
-
-  return(flags16);
-}
-
-#if BX_CPU_LEVEL >= 3
-Bit32u BX_CPU_C::read_eflags(void)
-{
-  Bit32u flags32 = BX_CPU_THIS_PTR force_flags();
-
-  /*
-   * 386+: real-mode: bit15 cleared, bits 14..12 are last loaded value
-   *       protected-mode: bit 15 clear, bit 14 = last loaded, IOPL?
-   */
-
-  return(flags32);
-}
-#endif
