@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: fpu_emu.cc,v 1.5 2007/10/15 22:07:52 sshwarts Exp $
+// $Id: fpu_emu.cc,v 1.8 2008/04/03 18:13:26 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2004 Stanislav Shwartsman
@@ -20,14 +20,13 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 /////////////////////////////////////////////////////////////////////////
 
-
-#define LOG_THIS BX_CPU_THIS_PTR
+#define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
 /* 9B */
-void BX_CPU_C::FWAIT(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FWAIT(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   if (BX_CPU_THIS_PTR cr0.get_TS() && BX_CPU_THIS_PTR cr0.get_MP())
@@ -41,7 +40,7 @@ void BX_CPU_C::FWAIT(bxInstruction_c *i)
 
 /* relevant only when FPU support is disabled */
 #if BX_SUPPORT_FPU == 0
-void BX_CPU_C::FPU_ESC(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FPU_ESC(bxInstruction_c *i)
 {
   if (BX_CPU_THIS_PTR cr0.get_EM() || BX_CPU_THIS_PTR cr0.get_TS())
     exception(BX_NM_EXCEPTION, 0, 0);
