@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: logical32.cc,v 1.37 2007/12/20 20:58:37 sshwarts Exp $
+// $Id: logical32.cc,v 1.39 2008/03/22 21:29:40 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -35,10 +35,12 @@
 #define RAX EAX
 #endif
 
-void BX_CPU_C::XOR_EdGdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EdGdM(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
   op2_32 = BX_READ_32BIT_REG(i->nnn());
   op1_32 ^= op2_32;
@@ -47,7 +49,7 @@ void BX_CPU_C::XOR_EdGdM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::XOR_EdGdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EdGdR(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
@@ -59,10 +61,12 @@ void BX_CPU_C::XOR_EdGdR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::XOR_GdEdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_GdEdM(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1_32 = BX_READ_32BIT_REG(i->nnn());
   op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
   op1_32 ^= op2_32;
@@ -71,7 +75,7 @@ void BX_CPU_C::XOR_GdEdM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::XOR_GdEdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_GdEdR(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
@@ -83,7 +87,7 @@ void BX_CPU_C::XOR_GdEdR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::XOR_EAXId(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EAXId(bxInstruction_c *i)
 {
   Bit32u op1_32;
 
@@ -94,10 +98,12 @@ void BX_CPU_C::XOR_EAXId(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::XOR_EdIdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EdIdM(bxInstruction_c *i)
 {
   Bit32u op1_32;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
   op1_32 ^= i->Id();
   write_RMW_virtual_dword(op1_32);
@@ -105,7 +111,7 @@ void BX_CPU_C::XOR_EdIdM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::XOR_EdIdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EdIdR(bxInstruction_c *i)
 {
   Bit32u op1_32 = BX_READ_32BIT_REG(i->rm());
   op1_32 ^= i->Id();
@@ -114,10 +120,12 @@ void BX_CPU_C::XOR_EdIdR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::OR_EdIdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EdIdM(bxInstruction_c *i)
 {
   Bit32u op1_32;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
   op1_32 |= i->Id();
   write_RMW_virtual_dword(op1_32);
@@ -125,7 +133,7 @@ void BX_CPU_C::OR_EdIdM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::OR_EdIdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EdIdR(bxInstruction_c *i)
 {
   Bit32u op1_32 = BX_READ_32BIT_REG(i->rm());
   op1_32 |= i->Id();
@@ -134,26 +142,30 @@ void BX_CPU_C::OR_EdIdR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::NOT_EdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::NOT_EdM(bxInstruction_c *i)
 {
   Bit32u op1_32;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
   op1_32 = ~op1_32;
   write_RMW_virtual_dword(op1_32);
 }
 
-void BX_CPU_C::NOT_EdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::NOT_EdR(bxInstruction_c *i)
 {
   Bit32u op1_32 = BX_READ_32BIT_REG(i->rm());
   op1_32 = ~op1_32;
   BX_WRITE_32BIT_REGZ(i->rm(), op1_32);
 }
 
-void BX_CPU_C::OR_EdGdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EdGdM(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
   op2_32 = BX_READ_32BIT_REG(i->nnn());
   op1_32 |= op2_32;
@@ -162,7 +174,7 @@ void BX_CPU_C::OR_EdGdM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::OR_EdGdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EdGdR(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
@@ -174,10 +186,12 @@ void BX_CPU_C::OR_EdGdR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::OR_GdEdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_GdEdM(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1_32 = BX_READ_32BIT_REG(i->nnn());
   op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
   op1_32 |= op2_32;
@@ -186,7 +200,7 @@ void BX_CPU_C::OR_GdEdM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::OR_GdEdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_GdEdR(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
@@ -198,7 +212,7 @@ void BX_CPU_C::OR_GdEdR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::OR_EAXId(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EAXId(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
@@ -210,10 +224,12 @@ void BX_CPU_C::OR_EAXId(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::AND_EdGdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EdGdM(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
   op2_32 = BX_READ_32BIT_REG(i->nnn());
   op1_32 &= op2_32;
@@ -222,7 +238,7 @@ void BX_CPU_C::AND_EdGdM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::AND_EdGdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EdGdR(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
@@ -234,10 +250,12 @@ void BX_CPU_C::AND_EdGdR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::AND_GdEdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_GdEdM(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1_32 = BX_READ_32BIT_REG(i->nnn());
   op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
   op1_32 &= op2_32;
@@ -246,7 +264,7 @@ void BX_CPU_C::AND_GdEdM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::AND_GdEdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_GdEdR(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
@@ -258,7 +276,7 @@ void BX_CPU_C::AND_GdEdR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::AND_EAXId(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EAXId(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
@@ -270,10 +288,12 @@ void BX_CPU_C::AND_EAXId(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::AND_EdIdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EdIdM(bxInstruction_c *i)
 {
   Bit32u op1_32;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
   op1_32 &= i->Id();
   write_RMW_virtual_dword(op1_32);
@@ -281,7 +301,7 @@ void BX_CPU_C::AND_EdIdM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::AND_EdIdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EdIdR(bxInstruction_c *i)
 {
   Bit32u op1_32 = BX_READ_32BIT_REG(i->rm());
   op1_32 &= i->Id();
@@ -290,10 +310,12 @@ void BX_CPU_C::AND_EdIdR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::TEST_EdGdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EdGdM(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1_32 = read_virtual_dword(i->seg(), RMAddr(i));
   op2_32 = BX_READ_32BIT_REG(i->nnn());
   op1_32 &= op2_32;
@@ -301,7 +323,7 @@ void BX_CPU_C::TEST_EdGdM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::TEST_EdGdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EdGdR(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
@@ -312,7 +334,7 @@ void BX_CPU_C::TEST_EdGdR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::TEST_EAXId(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EAXId(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
@@ -323,14 +345,16 @@ void BX_CPU_C::TEST_EAXId(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::TEST_EdIdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EdIdM(bxInstruction_c *i)
 {
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit32u op1_32 = read_virtual_dword(i->seg(), RMAddr(i));
   op1_32 &= i->Id();
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPU_C::TEST_EdIdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EdIdR(bxInstruction_c *i)
 {
   Bit32u op1_32 = BX_READ_32BIT_REG(i->rm());
   op1_32 &= i->Id();
