@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: logical8.cc,v 1.36 2007/12/20 20:58:37 sshwarts Exp $
+// $Id: logical8.cc,v 1.39 2008/03/22 21:29:40 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -25,17 +25,17 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 /////////////////////////////////////////////////////////////////////////
 
-
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
-
-void BX_CPU_C::XOR_EbGbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EbGbM(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
   op2 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
   op1 ^= op2;
@@ -44,7 +44,7 @@ void BX_CPU_C::XOR_EbGbM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::XOR_EbGbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EbGbR(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
@@ -56,10 +56,12 @@ void BX_CPU_C::XOR_EbGbR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::XOR_GbEbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_GbEbM(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
   op2 = read_virtual_byte(i->seg(), RMAddr(i));
   op1 ^= op2;
@@ -68,7 +70,7 @@ void BX_CPU_C::XOR_GbEbM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::XOR_GbEbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_GbEbR(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
@@ -80,7 +82,7 @@ void BX_CPU_C::XOR_GbEbR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::XOR_ALIb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_ALIb(bxInstruction_c *i)
 {
   Bit8u op1;
 
@@ -91,10 +93,12 @@ void BX_CPU_C::XOR_ALIb(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::XOR_EbIbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EbIbM(bxInstruction_c *i)
 {
   Bit8u op1, op2 = i->Ib();
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
   op1 ^= op2;
   write_RMW_virtual_byte(op1);
@@ -102,7 +106,7 @@ void BX_CPU_C::XOR_EbIbM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::XOR_EbIbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EbIbR(bxInstruction_c *i)
 {
   Bit8u op1, op2 = i->Ib();
 
@@ -113,10 +117,12 @@ void BX_CPU_C::XOR_EbIbR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::OR_EbIbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EbIbM(bxInstruction_c *i)
 {
   Bit8u op1, op2 = i->Ib();
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
   op1 |= op2;
   write_RMW_virtual_byte(op1);
@@ -124,7 +130,7 @@ void BX_CPU_C::OR_EbIbM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::OR_EbIbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EbIbR(bxInstruction_c *i)
 {
   Bit8u op1, op2 = i->Ib();
 
@@ -135,26 +141,30 @@ void BX_CPU_C::OR_EbIbR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::NOT_EbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::NOT_EbM(bxInstruction_c *i)
 {
   Bit8u op1_8;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1_8 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
   op1_8 = ~op1_8;
   write_RMW_virtual_byte(op1_8);
 }
 
-void BX_CPU_C::NOT_EbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::NOT_EbR(bxInstruction_c *i)
 {
   Bit8u op1_8 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
   op1_8 = ~op1_8;
   BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), op1_8);
 }
 
-void BX_CPU_C::OR_EbGbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EbGbM(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
   op2 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
   op1 |= op2;
@@ -163,7 +173,7 @@ void BX_CPU_C::OR_EbGbM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::OR_EbGbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EbGbR(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
@@ -175,10 +185,12 @@ void BX_CPU_C::OR_EbGbR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::OR_GbEbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_GbEbM(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
   op2 = read_virtual_byte(i->seg(), RMAddr(i));
   op1 |= op2;
@@ -187,7 +199,7 @@ void BX_CPU_C::OR_GbEbM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::OR_GbEbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_GbEbR(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
@@ -199,7 +211,7 @@ void BX_CPU_C::OR_GbEbR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::OR_ALIb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_ALIb(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
@@ -211,10 +223,12 @@ void BX_CPU_C::OR_ALIb(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::AND_EbGbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EbGbM(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
   op2 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
   op1 &= op2;
@@ -223,7 +237,7 @@ void BX_CPU_C::AND_EbGbM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::AND_EbGbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EbGbR(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
@@ -235,10 +249,12 @@ void BX_CPU_C::AND_EbGbR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::AND_GbEbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_GbEbM(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
   op2 = read_virtual_byte(i->seg(), RMAddr(i));
   op1 &= op2;
@@ -247,7 +263,7 @@ void BX_CPU_C::AND_GbEbM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::AND_GbEbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_GbEbR(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
@@ -259,7 +275,7 @@ void BX_CPU_C::AND_GbEbR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::AND_ALIb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_ALIb(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
@@ -271,10 +287,12 @@ void BX_CPU_C::AND_ALIb(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::AND_EbIbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EbIbM(bxInstruction_c *i)
 {
   Bit8u op1, op2 = i->Ib();
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
   op1 &= op2;
   write_RMW_virtual_byte(op1);
@@ -282,7 +300,7 @@ void BX_CPU_C::AND_EbIbM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::AND_EbIbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EbIbR(bxInstruction_c *i)
 {
   Bit8u op1, op2 = i->Ib();
 
@@ -293,10 +311,12 @@ void BX_CPU_C::AND_EbIbR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::TEST_EbGbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EbGbM(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   op1 = read_virtual_byte(i->seg(), RMAddr(i));
   op2 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
   op1 &= op2;
@@ -304,7 +324,7 @@ void BX_CPU_C::TEST_EbGbM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::TEST_EbGbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EbGbR(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
@@ -315,7 +335,7 @@ void BX_CPU_C::TEST_EbGbR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::TEST_ALIb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_ALIb(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
@@ -326,14 +346,16 @@ void BX_CPU_C::TEST_ALIb(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::TEST_EbIbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EbIbM(bxInstruction_c *i)
 {
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit8u op1 = read_virtual_byte(i->seg(), RMAddr(i));
   op1 &= i->Ib();
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPU_C::TEST_EbIbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EbIbR(bxInstruction_c *i)
 {
   Bit8u op1 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
   op1 &= i->Ib();
