@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: mult64.cc,v 1.23 2007/12/23 17:21:27 sshwarts Exp $
+// $Id: mult64.cc,v 1.30 2008/05/24 10:26:03 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -48,10 +48,10 @@ void long_mul(Bit128u *product, Bit64u op1, Bit64u op2)
 
   int i,j,k;
 
-  op_1[0] = op1 & 0xffffffff;
-  op_1[1] = op1 >> 32;
-  op_2[0] = op2 & 0xffffffff;
-  op_2[1] = op2 >> 32;
+  op_1[0] = (Bit32u)(op1 & 0xffffffff);
+  op_1[1] = (Bit32u)(op1 >> 32);
+  op_2[0] = (Bit32u)(op2 & 0xffffffff);
+  op_2[1] = (Bit32u)(op2 >> 32);
 
   for (i = 0; i < 4; i++) result[i] = 0;
 
@@ -59,10 +59,10 @@ void long_mul(Bit128u *product, Bit64u op1, Bit64u op2)
     for (j = 0; j < 2; j++) {
       nn = (Bit64u) op_1[i] * (Bit64u) op_2[j];
       k = i + j;
-      c = partial_add(&result[k++],nn & 0xffffffff);
-      c = partial_add(&result[k++],(nn >> 32) + c);
+      c = partial_add(&result[k++], (Bit32u)(nn & 0xffffffff));
+      c = partial_add(&result[k++], (Bit32u)(nn >> 32) + c);
       while (k < 4 && c != 0) {
-        c = partial_add(&result[k++],c);
+        c = partial_add(&result[k++], c);
       }
     }
   }
@@ -74,7 +74,7 @@ void long_mul(Bit128u *product, Bit64u op1, Bit64u op2)
 void long_neg(Bit128s *n)
 {
   Bit64u t = n->lo;
-  n->lo = (Bit64u) -n->lo;
+  n->lo = - (Bit64s)(n->lo);
   if (t - 1 > t) --n->hi;
   n->hi = ~n->hi;
 }
@@ -206,11 +206,11 @@ void long_idiv(Bit128s *quotient,Bit64s *remainder,Bit128s *dividend,Bit64s divi
   }
 }
 
-void BX_CPU_C::MUL_RAXEq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MUL_RAXEq(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64;
   Bit128u product_128;
-  
+
   op1_64 = RAX;
 
   /* op2 is a register or memory reference */
@@ -218,8 +218,9 @@ void BX_CPU_C::MUL_RAXEq(bxInstruction_c *i)
     op2_64 = BX_READ_64BIT_REG(i->rm());
   }
   else {
+    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
-    op2_64 = read_virtual_qword(i->seg(), RMAddr(i));
+    op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
   }
 
   // product_128 = ((Bit128u) op1_64) * ((Bit128u) op2_64);
@@ -240,7 +241,7 @@ void BX_CPU_C::MUL_RAXEq(bxInstruction_c *i)
   }
 }
 
-void BX_CPU_C::IMUL_RAXEq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::IMUL_RAXEq(bxInstruction_c *i)
 {
   Bit64s op1_64, op2_64;
   Bit128s product_128;
@@ -252,8 +253,9 @@ void BX_CPU_C::IMUL_RAXEq(bxInstruction_c *i)
     op2_64 = BX_READ_64BIT_REG(i->rm());
   }
   else {
+    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
-    op2_64 = (Bit64s) read_virtual_qword(i->seg(), RMAddr(i));
+    op2_64 = (Bit64s) read_virtual_qword_64(i->seg(), RMAddr(i));
   }
 
   // product_128 = ((Bit128s) op1_64) * ((Bit128s) op2_64);
@@ -279,7 +281,7 @@ void BX_CPU_C::IMUL_RAXEq(bxInstruction_c *i)
   }
 }
 
-void BX_CPU_C::DIV_RAXEq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::DIV_RAXEq(bxInstruction_c *i)
 {
   Bit64u op2_64, remainder_64, quotient_64l;
   Bit128u op1_128, quotient_128;
@@ -292,8 +294,9 @@ void BX_CPU_C::DIV_RAXEq(bxInstruction_c *i)
     op2_64 = BX_READ_64BIT_REG(i->rm());
   }
   else {
+    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
-    op2_64 = read_virtual_qword(i->seg(), RMAddr(i));
+    op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
   }
 
   if (op2_64 == 0) {
@@ -319,7 +322,7 @@ void BX_CPU_C::DIV_RAXEq(bxInstruction_c *i)
   RDX = remainder_64;
 }
 
-void BX_CPU_C::IDIV_RAXEq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::IDIV_RAXEq(bxInstruction_c *i)
 {
   Bit64s op2_64, remainder_64, quotient_64l;
   Bit128s op1_128, quotient_128;
@@ -332,20 +335,18 @@ void BX_CPU_C::IDIV_RAXEq(bxInstruction_c *i)
     op2_64 = BX_READ_64BIT_REG(i->rm());
   }
   else {
+    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
-    op2_64 = (Bit64s) read_virtual_qword(i->seg(), RMAddr(i));
+    op2_64 = (Bit64s) read_virtual_qword_64(i->seg(), RMAddr(i));
   }
- 
+
   if (op2_64 == 0) {
     exception(BX_DE_EXCEPTION, 0, 0);
   }
 
-  /* check MIN_INT divided by -1 case */
-  if (op2_64 == -1)
-  {
-    if ((op1_128.hi == (Bit64s) BX_CONST64(0x8000000000000000)) && (!op1_128.lo))
-      exception(BX_DE_EXCEPTION, 0, 0);
-  }
+  /* check MIN_INT case */
+  if ((op1_128.hi == (Bit64s) BX_CONST64(0x8000000000000000)) && (!op1_128.lo))
+    exception(BX_DE_EXCEPTION, 0, 0);
 
   // quotient_128 = op1_128 / op2_64;
   // remainder_64 = (Bit64s) (op1_128 % op2_64);
@@ -369,7 +370,7 @@ void BX_CPU_C::IDIV_RAXEq(bxInstruction_c *i)
   RDX = remainder_64;
 }
 
-void BX_CPU_C::IMUL_GqEqId(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::IMUL_GqEqId(bxInstruction_c *i)
 {
   Bit64s op2_64, op3_64;
   Bit128s product_128;
@@ -381,8 +382,9 @@ void BX_CPU_C::IMUL_GqEqId(bxInstruction_c *i)
     op2_64 = BX_READ_64BIT_REG(i->rm());
   }
   else {
+    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
-    op2_64 = (Bit64s) read_virtual_qword(i->seg(), RMAddr(i));
+    op2_64 = (Bit64s) read_virtual_qword_64(i->seg(), RMAddr(i));
   }
 
   long_imul(&product_128,op2_64,op3_64);
@@ -398,7 +400,7 @@ void BX_CPU_C::IMUL_GqEqId(bxInstruction_c *i)
   }
 }
 
-void BX_CPU_C::IMUL_GqEq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::IMUL_GqEq(bxInstruction_c *i)
 {
   Bit64s op1_64, op2_64;
   Bit128s product_128;
@@ -408,8 +410,9 @@ void BX_CPU_C::IMUL_GqEq(bxInstruction_c *i)
     op2_64 = BX_READ_64BIT_REG(i->rm());
   }
   else {
+    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
-    op2_64 = (Bit64s) read_virtual_qword(i->seg(), RMAddr(i));
+    op2_64 = (Bit64s) read_virtual_qword_64(i->seg(), RMAddr(i));
   }
 
   op1_64 = BX_READ_64BIT_REG(i->nnn());
