@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: shift16.cc,v 1.39 2007/12/20 20:58:37 sshwarts Exp $
+// $Id: shift16.cc,v 1.46 2008/04/05 19:08:01 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -25,14 +25,12 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 /////////////////////////////////////////////////////////////////////////
 
-
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
-
-void BX_CPU_C::SHLD_EwGw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHLD_EwGwM(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, result_16;
   Bit32u temp_32, result_32;
@@ -40,21 +38,17 @@ void BX_CPU_C::SHLD_EwGw(bxInstruction_c *i)
   unsigned of, cf;
 
   /* op1:op2 << count.  result stored in op1 */
-  if (i->b1() == 0x1a4)
+  if (i->b1() == 0xa4) // 0x1a4
     count = i->Ib();
   else // 0x1a5
     count = CL;
 
   count &= 0x1f; // use only 5 LSB's
 
-  /* op1 is a register or memory reference */
-  if (i->modC0()) {
-    op1_16 = BX_READ_16BIT_REG(i->rm());
-  }
-  else {
-    /* pointer, segment address pair */
-    op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
-  }
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  /* pointer, segment address pair */
+  op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
 
   if (!count) return;
 
@@ -71,15 +65,9 @@ void BX_CPU_C::SHLD_EwGw(bxInstruction_c *i)
     result_32 |= (op1_16 << (count - 16));
   }
 
-  result_16 = result_32 >> 16;
+  result_16 = (Bit16u)(result_32 >> 16);
 
-  /* now write result back to destination */
-  if (i->modC0()) {
-    BX_WRITE_16BIT_REG(i->rm(), result_16);
-  }
-  else {
-    write_RMW_virtual_word(result_16);
-  }
+  write_RMW_virtual_word(result_16);
 
   SET_FLAGS_OSZAPC_LOGIC_16(result_16); /* handle SF, ZF and AF flags */
 
@@ -88,28 +76,66 @@ void BX_CPU_C::SHLD_EwGw(bxInstruction_c *i)
   SET_FLAGS_OxxxxC(of, cf);
 }
 
-void BX_CPU_C::SHRD_EwGw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHLD_EwGwR(bxInstruction_c *i)
+{
+  Bit16u op1_16, op2_16, result_16;
+  Bit32u temp_32, result_32;
+  unsigned count;
+  unsigned of, cf;
+
+  /* op1:op2 << count.  result stored in op1 */
+  if (i->b1() == 0xa4) // 0x1a4
+    count = i->Ib();
+  else // 0x1a5
+    count = CL;
+
+  count &= 0x1f; // use only 5 LSB's
+
+  if (!count) return;
+
+  op1_16 = BX_READ_16BIT_REG(i->rm());
+  op2_16 = BX_READ_16BIT_REG(i->nnn());
+
+  /* count < 32, since only lower 5 bits used */
+  temp_32 = ((Bit32u)(op1_16) << 16) | (op2_16); // double formed by op1:op2
+  result_32 = temp_32 << count;
+
+  // hack to act like x86 SHLD when count > 16
+  if (count > 16) {
+    // when count > 16 actually shifting op1:op2:op2 << count,
+    // it is the same as shifting op2:op2 by count-16
+    result_32 |= (op1_16 << (count - 16));
+  }
+
+  result_16 = (Bit16u)(result_32 >> 16);
+
+  BX_WRITE_16BIT_REG(i->rm(), result_16);
+
+  SET_FLAGS_OSZAPC_LOGIC_16(result_16); /* handle SF, ZF and AF flags */
+
+  cf = (temp_32 >> (32 - count)) & 0x1;
+  of = cf ^ (result_16 >> 15); // of = cf ^ result15
+  SET_FLAGS_OxxxxC(of, cf);
+}
+
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHRD_EwGwM(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, result_16;
   Bit32u temp_32, result_32;
   unsigned count;
   unsigned cf, of;
 
-  if (i->b1() == 0x1ac)
+  if (i->b1() == 0xac) // 0x1ac
     count = i->Ib();
   else // 0x1ad
     count = CL;
 
   count &= 0x1f; /* use only 5 LSB's */
 
-  /* op1 is a register or memory reference */
-  if (i->modC0()) {
-    op1_16 = BX_READ_16BIT_REG(i->rm());
-  }
-  else {
-    /* pointer, segment address pair */
-    op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
-  }
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  /* pointer, segment address pair */
+  op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
 
   if (!count) return;
 
@@ -125,16 +151,10 @@ void BX_CPU_C::SHRD_EwGw(bxInstruction_c *i)
     // it is the same as shifting op2:op2 by count-16
     result_32 |= (op1_16 << (32 - count));
   }
-  
-  result_16 = result_32;
 
-  /* now write result back to destination */
-  if (i->modC0()) {
-    BX_WRITE_16BIT_REG(i->rm(), result_16);
-  }
-  else {
-    write_RMW_virtual_word(result_16);
-  }
+  result_16 = (Bit16u) result_32;
+
+  write_RMW_virtual_word(result_16);
 
   SET_FLAGS_OSZAPC_LOGIC_16(result_16); /* handle SF, ZF and AF flags */
 
@@ -143,24 +163,64 @@ void BX_CPU_C::SHRD_EwGw(bxInstruction_c *i)
   SET_FLAGS_OxxxxC(of, cf);
 }
 
-void BX_CPU_C::ROL_Ew(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHRD_EwGwR(bxInstruction_c *i)
+{
+  Bit16u op1_16, op2_16, result_16;
+  Bit32u temp_32, result_32;
+  unsigned count;
+  unsigned cf, of;
+
+  if (i->b1() == 0xac) // 0x1ac
+    count = i->Ib();
+  else // 0x1ad
+    count = CL;
+
+  count &= 0x1f; /* use only 5 LSB's */
+
+  if (!count) return;
+
+  op1_16 = BX_READ_16BIT_REG(i->rm());
+  op2_16 = BX_READ_16BIT_REG(i->nnn());
+
+  /* count < 32, since only lower 5 bits used */
+  temp_32 = (op2_16 << 16) | op1_16; // double formed by op2:op1
+  result_32 = temp_32 >> count;
+
+  // hack to act like x86 SHRD when count > 16
+  if (count > 16) {
+    // when count > 16 actually shifting op2:op2:op1 >> count,
+    // it is the same as shifting op2:op2 by count-16
+    result_32 |= (op1_16 << (32 - count));
+  }
+
+  result_16 = (Bit16u) result_32;
+
+  BX_WRITE_16BIT_REG(i->rm(), result_16);
+
+  SET_FLAGS_OSZAPC_LOGIC_16(result_16); /* handle SF, ZF and AF flags */
+
+  cf = (op1_16 >> (count - 1)) & 0x1;
+  of = ((result_16 << 1) ^ result_16) >> 15; // of = result14 ^ result15
+  SET_FLAGS_OxxxxC(of, cf);
+}
+
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ROL_Ew(bxInstruction_c *i)
 {
   Bit16u op1_16, result_16;
   unsigned count;
   unsigned bit0, bit15;
 
-  if (i->b1() == 0xc1)
-    count = i->Ib();
-  else if (i->b1() == 0xd1)
-    count = 1;
-  else // 0xd3
+  if (i->b1() == 0xd3)
     count = CL;
+  else // 0xc1 or 0xd1
+    count = i->Ib();
 
   /* op1 is a register or memory reference */
   if (i->modC0()) {
     op1_16 = BX_READ_16BIT_REG(i->rm());
   }
   else {
+    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
     op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
   }
@@ -193,24 +253,23 @@ void BX_CPU_C::ROL_Ew(bxInstruction_c *i)
   SET_FLAGS_OxxxxC(bit0 ^ bit15, bit0);
 }
 
-void BX_CPU_C::ROR_Ew(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ROR_Ew(bxInstruction_c *i)
 {
   Bit16u op1_16, result_16;
   unsigned count;
   unsigned bit14, bit15;
 
-  if (i->b1() == 0xc1)
-    count = i->Ib();
-  else if (i->b1() == 0xd1)
-    count = 1;
-  else // 0xd3
+  if (i->b1() == 0xd3)
     count = CL;
+  else // 0xc1 or 0xd1
+    count = i->Ib();
 
   /* op1 is a register or memory reference */
   if (i->modC0()) {
     op1_16 = BX_READ_16BIT_REG(i->rm());
   }
   else {
+    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
     op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
   }
@@ -219,7 +278,7 @@ void BX_CPU_C::ROR_Ew(bxInstruction_c *i)
     if (count & 0x10) {
       bit14 = (op1_16 >> 14) & 1;
       bit15 = (op1_16 >> 15) & 1;
-      // of = result14 ^ result15 
+      // of = result14 ^ result15
       SET_FLAGS_OxxxxC(bit14 ^ bit15, bit15);
     }
     return;
@@ -236,25 +295,23 @@ void BX_CPU_C::ROR_Ew(bxInstruction_c *i)
   else {
     write_RMW_virtual_word(result_16);
   }
-  
+
   bit14 = (result_16 >> 14) & 1;
   bit15 = (result_16 >> 15) & 1;
-  // of = result14 ^ result15 
+  // of = result14 ^ result15
   SET_FLAGS_OxxxxC(bit14 ^ bit15, bit15);
 }
 
-void BX_CPU_C::RCL_Ew(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCL_Ew(bxInstruction_c *i)
 {
   Bit16u op1_16, result_16;
   unsigned count;
   unsigned of, cf;
 
-  if (i->b1() == 0xc1)
-    count = i->Ib();
-  else if (i->b1() == 0xd1)
-    count = 1;
-  else // 0xd3
+  if (i->b1() == 0xd3)
     count = CL;
+  else // 0xc1 or 0xd1
+    count = i->Ib();
 
   count = (count & 0x1f) % 17;
 
@@ -263,6 +320,7 @@ void BX_CPU_C::RCL_Ew(bxInstruction_c *i)
     op1_16 = BX_READ_16BIT_REG(i->rm());
   }
   else {
+    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
     op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
   }
@@ -293,18 +351,16 @@ void BX_CPU_C::RCL_Ew(bxInstruction_c *i)
   SET_FLAGS_OxxxxC(of, cf);
 }
 
-void BX_CPU_C::RCR_Ew(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_Ew(bxInstruction_c *i)
 {
   Bit16u op1_16, result_16;
   unsigned count;
   unsigned of, cf;
 
-  if (i->b1() == 0xc1)
-    count = i->Ib();
-  else if (i->b1() == 0xd1)
-    count = 1;
-  else // 0xd3
+  if (i->b1() == 0xd3)
     count = CL;
+  else // 0xc1 or 0xd1
+    count = i->Ib();
 
   count = (count & 0x1f) % 17;
 
@@ -313,6 +369,7 @@ void BX_CPU_C::RCR_Ew(bxInstruction_c *i)
     op1_16 = BX_READ_16BIT_REG(i->rm());
   }
   else {
+    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
     op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
   }
@@ -335,18 +392,16 @@ void BX_CPU_C::RCR_Ew(bxInstruction_c *i)
   SET_FLAGS_OxxxxC(of, cf);
 }
 
-void BX_CPU_C::SHL_Ew(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHL_Ew(bxInstruction_c *i)
 {
   Bit16u op1_16, result_16;
   unsigned count;
   unsigned of = 0, cf = 0;
 
-  if (i->b1() == 0xc1)
-    count = i->Ib();
-  else if (i->b1() == 0xd1)
-    count = 1;
-  else // 0xd3
+  if (i->b1() == 0xd3)
     count = CL;
+  else // 0xc1 or 0xd1
+    count = i->Ib();
 
   count &= 0x1f; /* use only 5 LSB's */
 
@@ -355,6 +410,7 @@ void BX_CPU_C::SHL_Ew(bxInstruction_c *i)
     op1_16 = BX_READ_16BIT_REG(i->rm());
   }
   else {
+    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
     op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
   }
@@ -382,18 +438,16 @@ void BX_CPU_C::SHL_Ew(bxInstruction_c *i)
   SET_FLAGS_OxxxxC(of, cf);
 }
 
-void BX_CPU_C::SHR_Ew(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHR_Ew(bxInstruction_c *i)
 {
   Bit16u op1_16, result_16;
   unsigned count;
   unsigned of, cf;
 
-  if (i->b1() == 0xc1)
-    count = i->Ib();
-  else if (i->b1() == 0xd1)
-    count = 1;
-  else // 0xd3
+  if (i->b1() == 0xd3)
     count = CL;
+  else // 0xc1 or 0xd1
+    count = i->Ib();
 
   count &= 0x1f; /* use only 5 LSB's */
 
@@ -402,6 +456,7 @@ void BX_CPU_C::SHR_Ew(bxInstruction_c *i)
     op1_16 = BX_READ_16BIT_REG(i->rm());
   }
   else {
+    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
     op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
   }
@@ -427,17 +482,15 @@ void BX_CPU_C::SHR_Ew(bxInstruction_c *i)
   SET_FLAGS_OxxxxC(of, cf);
 }
 
-void BX_CPU_C::SAR_Ew(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SAR_Ew(bxInstruction_c *i)
 {
   Bit16u op1_16, result_16;
   unsigned count, cf;
 
-  if (i->b1() == 0xc1)
-    count = i->Ib();
-  else if (i->b1() == 0xd1)
-    count = 1;
-  else // 0xd3
+  if (i->b1() == 0xd3)
     count = CL;
+  else // 0xc1 or 0xd1
+    count = i->Ib();
 
   count &= 0x1f;  /* use only 5 LSB's */
 
@@ -446,6 +499,7 @@ void BX_CPU_C::SAR_Ew(bxInstruction_c *i)
     op1_16 = BX_READ_16BIT_REG(i->rm());
   }
   else {
+    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
     op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
   }
@@ -475,7 +529,7 @@ void BX_CPU_C::SAR_Ew(bxInstruction_c *i)
 
   SET_FLAGS_OSZAPC_LOGIC_16(result_16); /* handle SF, ZF and AF flags */
   /* signed overflow cannot happen in SAR instruction */
-  SET_FLAGS_OxxxxC(0, cf); 
+  SET_FLAGS_OxxxxC(0, cf);
 
   /* now write result back to destination */
   if (i->modC0()) {
