@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: fpu.cc,v 1.30 2007/12/23 17:21:27 sshwarts Exp $
+// $Id: fpu.cc,v 1.42 2008/05/19 20:00:42 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2003 Stanislav Shwartsman
@@ -32,9 +32,8 @@
 #define UPDATE_LAST_OPCODE       1
 #define CHECK_PENDING_EXCEPTIONS 1
 
-
 #if BX_SUPPORT_FPU
-void BX_CPU_C::prepareFPU(bxInstruction_c *i, 
+void BX_CPU_C::prepareFPU(bxInstruction_c *i,
 	bx_bool check_pending_exceptions, bx_bool update_last_instruction)
 {
   if (BX_CPU_THIS_PTR cr0.get_EM() || BX_CPU_THIS_PTR cr0.get_TS())
@@ -45,16 +44,18 @@ void BX_CPU_C::prepareFPU(bxInstruction_c *i,
 
   if (update_last_instruction)
   {
-    BX_CPU_THIS_PTR the_i387.foo = ((Bit32u)(i->b1()) << 8) | (Bit32u)(i->modrm()) & 0x7ff;
+    BX_CPU_THIS_PTR the_i387.foo = (((Bit32u)(i->b1()) << 8) | i->modrm()) & 0x7ff;
     BX_CPU_THIS_PTR the_i387.fcs = BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value;
     BX_CPU_THIS_PTR the_i387.fip = BX_CPU_THIS_PTR prev_rip;
 
     if (! i->modC0()) {
-         BX_CPU_THIS_PTR the_i387.fds = BX_CPU_THIS_PTR sregs[i->seg()].selector.value;
-         BX_CPU_THIS_PTR the_i387.fdp = RMAddr(i);
+       BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+       BX_CPU_THIS_PTR the_i387.fds = BX_CPU_THIS_PTR sregs[i->seg()].selector.value;
+       BX_CPU_THIS_PTR the_i387.fdp = RMAddr(i);
     } else {
-         BX_CPU_THIS_PTR the_i387.fds = BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].selector.value;
-         BX_CPU_THIS_PTR the_i387.fdp = 0;
+       BX_CPU_THIS_PTR the_i387.fds = BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].selector.value;
+       BX_CPU_THIS_PTR the_i387.fdp = 0;
     }
   }
 }
@@ -83,6 +84,17 @@ void BX_CPU_C::FPU_check_pending_exceptions(void)
 
 int BX_CPU_C::fpu_save_environment(bxInstruction_c *i)
 {
+    /* read all registers in stack order and update x87 tag word */
+    for(int n=0;n<8;n++) {
+       // update tag only if it is not empty
+       if (! IS_TAG_EMPTY(n)) {
+           int tag = FPU_tagof(BX_READ_FPU_REG(n));
+           BX_CPU_THIS_PTR the_i387.FPU_settagi(tag, n);
+       }
+    }
+
+    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
     if (protected_mode())  /* Protected Mode */
     {
         if (i->os32L() || i->os64L())
@@ -95,12 +107,12 @@ int BX_CPU_C::fpu_save_environment(bxInstruction_c *i)
             write_virtual_dword(i->seg(), RMAddr(i) + 0x04, tmp);
             tmp = 0xffff0000 | BX_CPU_THIS_PTR the_i387.get_tag_word();
             write_virtual_dword(i->seg(), RMAddr(i) + 0x08, tmp);
-            tmp = (BX_CPU_THIS_PTR the_i387.fip) & 0xffffffff;
+            tmp = (Bit32u)(BX_CPU_THIS_PTR the_i387.fip);
             write_virtual_dword(i->seg(), RMAddr(i) + 0x0c, tmp);
             tmp  = (BX_CPU_THIS_PTR the_i387.fcs & 0xffff) |
                           ((Bit32u)(BX_CPU_THIS_PTR the_i387.foo)) << 16;
             write_virtual_dword(i->seg(), RMAddr(i) + 0x10, tmp);
-            tmp = (BX_CPU_THIS_PTR the_i387.fdp) & 0xffffffff;
+            tmp = (Bit32u)(BX_CPU_THIS_PTR the_i387.fdp);
             write_virtual_dword(i->seg(), RMAddr(i) + 0x14, tmp);
             tmp = 0xffff0000 | (BX_CPU_THIS_PTR the_i387.fds);
             write_virtual_dword(i->seg(), RMAddr(i) + 0x18, tmp);
@@ -117,11 +129,11 @@ int BX_CPU_C::fpu_save_environment(bxInstruction_c *i)
             write_virtual_word(i->seg(), RMAddr(i) + 0x02, tmp);
             tmp = BX_CPU_THIS_PTR the_i387.get_tag_word();
             write_virtual_word(i->seg(), RMAddr(i) + 0x04, tmp);
-            tmp = (BX_CPU_THIS_PTR the_i387.fip) & 0xffff;
+            tmp = (Bit16u)(BX_CPU_THIS_PTR the_i387.fip) & 0xffff;
             write_virtual_word(i->seg(), RMAddr(i) + 0x06, tmp);
             tmp = (BX_CPU_THIS_PTR the_i387.fcs);
             write_virtual_word(i->seg(), RMAddr(i) + 0x08, tmp);
-            tmp = (BX_CPU_THIS_PTR the_i387.fdp) & 0xffff;
+            tmp = (Bit16u)(BX_CPU_THIS_PTR the_i387.fdp) & 0xffff;
             write_virtual_word(i->seg(), RMAddr(i) + 0x0a, tmp);
             tmp = (BX_CPU_THIS_PTR the_i387.fds);
             write_virtual_word(i->seg(), RMAddr(i) + 0x0c, tmp);
@@ -139,7 +151,7 @@ int BX_CPU_C::fpu_save_environment(bxInstruction_c *i)
         if (i->os32L() || i->os64L())
         {
             Bit32u tmp;
-        
+
             tmp = 0xffff0000 | BX_CPU_THIS_PTR the_i387.get_control_word();
             write_virtual_dword(i->seg(), RMAddr(i), tmp);
             tmp = 0xffff0000 | BX_CPU_THIS_PTR the_i387.get_status_word();
@@ -148,8 +160,7 @@ int BX_CPU_C::fpu_save_environment(bxInstruction_c *i)
             write_virtual_dword(i->seg(), RMAddr(i) + 0x08, tmp);
             tmp = 0xffff0000 | (fp_ip & 0xffff);
             write_virtual_dword(i->seg(), RMAddr(i) + 0x0c, tmp);
-            tmp = ((fp_ip & 0xffff0000) >> 4) |
-                          (BX_CPU_THIS_PTR the_i387.foo & 0x7ff);
+            tmp = ((fp_ip & 0xffff0000) >> 4) | BX_CPU_THIS_PTR the_i387.foo;
             write_virtual_dword(i->seg(), RMAddr(i) + 0x10, tmp);
             tmp = 0xffff0000 | (fp_dp & 0xffff);
             write_virtual_dword(i->seg(), RMAddr(i) + 0x14, tmp);
@@ -170,8 +181,7 @@ int BX_CPU_C::fpu_save_environment(bxInstruction_c *i)
             write_virtual_word(i->seg(), RMAddr(i) + 0x04, tmp);
             tmp = fp_ip & 0xffff;
             write_virtual_word(i->seg(), RMAddr(i) + 0x06, tmp);
-            tmp = (Bit16u)(((fp_ip & 0xf0000) >> 4) |
-                          (BX_CPU_THIS_PTR the_i387.foo & 0x7ff));
+            tmp = (Bit16u)((fp_ip & 0xf0000) >> 4) | BX_CPU_THIS_PTR the_i387.foo;
             write_virtual_word(i->seg(), RMAddr(i) + 0x08, tmp);
             tmp = fp_dp & 0xffff;
             write_virtual_word(i->seg(), RMAddr(i) + 0x0a, tmp);
@@ -179,14 +189,16 @@ int BX_CPU_C::fpu_save_environment(bxInstruction_c *i)
             write_virtual_word(i->seg(), RMAddr(i) + 0x0c, tmp);
 
             return 0x0e;
-        }       
-    }   
+        }
+    }
 }
 
 int BX_CPU_C::fpu_load_environment(bxInstruction_c *i)
 {
     int offset;
 
+    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
     if (protected_mode())  /* Protected Mode */
     {
         if (i->os32L() || i->os64L())
@@ -293,6 +305,10 @@ int BX_CPU_C::fpu_load_environment(bxInstruction_c *i)
         }
     }
 
+    /* always set bit 6 as '1 */
+    BX_CPU_THIS_PTR the_i387.cwd =
+       (BX_CPU_THIS_PTR the_i387.cwd & ~FPU_CW_Reserved_Bits) | 0x0040;
+
     /* check for unmasked exceptions */
     if (FPU_PARTIAL_STATUS & ~FPU_CONTROL_WORD & FPU_CW_Exceptions_Mask)
     {
@@ -307,15 +323,17 @@ int BX_CPU_C::fpu_load_environment(bxInstruction_c *i)
 
     return offset;
 }
-#endif
 
 /* D9 /5 */
-void BX_CPU_C::FLDCW(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FLDCW(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i, CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
+
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   Bit16u cwd = read_virtual_word(i->seg(), RMAddr(i));
-  FPU_CONTROL_WORD = cwd;
+  FPU_CONTROL_WORD = (cwd & ~FPU_CW_Reserved_Bits) | 0x0040; // bit 6 is reserved as '1
 
   /* check for unmasked exceptions */
   if (FPU_PARTIAL_STATUS & ~FPU_CONTROL_WORD & FPU_CW_Exceptions_Mask)
@@ -334,11 +352,15 @@ void BX_CPU_C::FLDCW(bxInstruction_c *i)
 }
 
 /* D9 /7 */
-void BX_CPU_C::FNSTCW(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNSTCW(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
+
   Bit16u cwd = BX_CPU_THIS_PTR the_i387.get_control_word();
+
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   write_virtual_word(i->seg(), RMAddr(i), cwd);
 #else
   BX_INFO(("FNSTCW: required FPU, configure --enable-fpu"));
@@ -346,11 +368,15 @@ void BX_CPU_C::FNSTCW(bxInstruction_c *i)
 }
 
 /* DD /7 */
-void BX_CPU_C::FNSTSW(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNSTSW(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
+
   Bit16u swd = BX_CPU_THIS_PTR the_i387.get_status_word();
+
+  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   write_virtual_word(i->seg(), RMAddr(i), swd);
 #else
   BX_INFO(("FNSTSW: required FPU, configure --enable-fpu"));
@@ -358,7 +384,7 @@ void BX_CPU_C::FNSTSW(bxInstruction_c *i)
 }
 
 /* DF E0 */
-void BX_CPU_C::FNSTSW_AX(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNSTSW_AX(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
@@ -369,23 +395,21 @@ void BX_CPU_C::FNSTSW_AX(bxInstruction_c *i)
 }
 
 /* DD /4 */
-void BX_CPU_C::FRSTOR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FRSTOR(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i, CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
-  int offset = fpu_load_environment(i);
 
-  /* read all registers in stack order. */
+  int offset = fpu_load_environment(i);
+  floatx80 tmp;
+
+  /* read all registers in stack order */
   for(int n=0;n<8;n++)
   {
-     floatx80 tmp;
-
-     // read register only if its tag is not empty
-     if (! IS_TAG_EMPTY(n))
-     {
-         read_virtual_tword(i->seg(), RMAddr(i) + offset + n*10, &tmp);
-         BX_WRITE_FPU_REG(tmp, n);
-     }
+     read_virtual_tword(i->seg(), RMAddr(i) + offset + n*10, &tmp);
+     // update tag only if it is not empty
+     BX_WRITE_FPU_REGISTER_AND_TAG(tmp,
+              IS_TAG_EMPTY(n) ? FPU_Tag_Empty : FPU_tagof(tmp), n);
   }
 #else
   BX_INFO(("FRSTOR: required FPU, configure --enable-fpu"));
@@ -393,7 +417,7 @@ void BX_CPU_C::FRSTOR(bxInstruction_c *i)
 }
 
 /* DD /6 */
-void BX_CPU_C::FNSAVE(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNSAVE(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
@@ -414,7 +438,7 @@ void BX_CPU_C::FNSAVE(bxInstruction_c *i)
 }
 
 /* 9B E2 */
-void BX_CPU_C::FNCLEX(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNCLEX(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
@@ -430,7 +454,7 @@ void BX_CPU_C::FNCLEX(bxInstruction_c *i)
 }
 
 /* DB E3 */
-void BX_CPU_C::FNINIT(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNINIT(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
@@ -441,18 +465,27 @@ void BX_CPU_C::FNINIT(bxInstruction_c *i)
 }
 
 /* D9 /4 */
-void BX_CPU_C::FLDENV(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FLDENV(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i, CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
   fpu_load_environment(i);
+
+  /* read all registers in stack order and update x87 tag word */
+  for(int n=0;n<8;n++) {
+     // update tag only if it is not empty
+     if (! IS_TAG_EMPTY(n)) {
+         int tag = FPU_tagof(BX_READ_FPU_REG(n));
+         BX_CPU_THIS_PTR the_i387.FPU_settagi(tag, n);
+     }
+  }
 #else
   BX_INFO(("FLDENV: required FPU, configure --enable-fpu"));
 #endif
 }
 
 /* D9 /6 */
-void BX_CPU_C::FNSTENV(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNSTENV(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
@@ -467,7 +500,7 @@ void BX_CPU_C::FNSTENV(bxInstruction_c *i)
 }
 
 /* D9 D0 */
-void BX_CPU_C::FNOP(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNOP(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i, CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
@@ -480,7 +513,7 @@ void BX_CPU_C::FNOP(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FPLEGACY(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FPLEGACY(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
@@ -492,30 +525,62 @@ void BX_CPU_C::FPLEGACY(bxInstruction_c *i)
 #endif
 }
 
+#endif
 
 #if BX_SUPPORT_FPU
 
 #include <math.h>
 
-void BX_CPU_C::print_state_FPU()
+void BX_CPU_C::print_state_FPU(void)
 {
   static double scale_factor = pow(2.0, -63.0);
+  static char* cw_round_control[] = {
+    "NEAREST", "DOWN", "UP", "CHOP"
+  };
+  static char* cw_precision_control[] = {
+    "32", "RES", "64", "80"
+  };
 
   Bit32u reg;
-  reg = BX_CPU_THIS_PTR the_i387.get_control_word();
-  fprintf(stderr, "control word: 0x%04x\n", reg);
   reg = BX_CPU_THIS_PTR the_i387.get_status_word();
-  fprintf(stderr, "status  word: 0x%04x\n", reg);
-  fprintf(stderr, "        TOS : %d\n", FPU_TOS&7);
+  fprintf(stderr, "status  word: 0x%04x: ", reg);
+  fprintf(stderr, "%s %s TOS%d %s %s %s %s %s %s %s %s %s %s %s\n",
+    (reg & FPU_SW_Backward) ? "B" : "b",
+    (reg & FPU_SW_C3) ? "C3" : "c3", (FPU_TOS&7),
+    (reg & FPU_SW_C2) ? "C2" : "c2",
+    (reg & FPU_SW_C1) ? "C1" : "c1",
+    (reg & FPU_SW_C0) ? "C0" : "c0",
+    (reg & FPU_SW_Summary) ? "ES" : "es",
+    (reg & FPU_SW_Stack_Fault) ? "SF" : "sf",
+    (reg & FPU_SW_Precision) ? "PE" : "pe",
+    (reg & FPU_SW_Underflow) ? "UE" : "ue",
+    (reg & FPU_SW_Overflow) ? "OE" : "oe",
+    (reg & FPU_SW_Zero_Div) ? "ZE" : "ze",
+    (reg & FPU_SW_Denormal_Op) ? "DE" : "de",
+    (reg & FPU_SW_Invalid) ? "IE" : "ie");
+
+  reg = BX_CPU_THIS_PTR the_i387.get_control_word();
+  fprintf(stderr, "control word: 0x%04x: ", reg);
+  fprintf(stderr, "%s RC_%s PC_%s %s %s %s %s %s %s\n",
+    (reg & FPU_CW_Inf) ? "INF" : "inf",
+    (cw_round_control[(reg & FPU_CW_RC) >> 10]),
+    (cw_precision_control[(reg & FPU_CW_PC) >> 8]),
+    (reg & FPU_CW_Precision) ? "PM" : "pm",
+    (reg & FPU_CW_Underflow) ? "UM" : "um",
+    (reg & FPU_CW_Overflow)  ? "OM" : "om",
+    (reg & FPU_CW_Zero_Div)  ? "ZM" : "zm",
+    (reg & FPU_CW_Denormal)  ? "DM" : "dm",
+    (reg & FPU_CW_Invalid)   ? "IM" : "im");
+
   reg = BX_CPU_THIS_PTR the_i387.get_tag_word();
   fprintf(stderr, "tag word:     0x%04x\n", reg);
   reg = BX_CPU_THIS_PTR the_i387.foo;
   fprintf(stderr, "operand:      0x%04x\n", reg);
-  reg = BX_CPU_THIS_PTR the_i387.fip & 0xffffffff;
+  reg = (Bit32u)(BX_CPU_THIS_PTR the_i387.fip) & 0xffffffff;
   fprintf(stderr, "fip:          0x%08x\n", reg);
   reg = BX_CPU_THIS_PTR the_i387.fcs;
   fprintf(stderr, "fcs:          0x%04x\n", reg);
-  reg = BX_CPU_THIS_PTR the_i387.fdp & 0xffffffff;
+  reg = (Bit32u)(BX_CPU_THIS_PTR the_i387.fdp) & 0xffffffff;
   fprintf(stderr, "fdp:          0x%08x\n", reg);
   reg = BX_CPU_THIS_PTR the_i387.fds;
   fprintf(stderr, "fds:          0x%04x\n", reg);
@@ -524,6 +589,8 @@ void BX_CPU_C::print_state_FPU()
   int tos = FPU_TOS & 7;
   for (int i=0; i<8; i++) {
     const floatx80 &fp = BX_FPU_REG(i);
+    unsigned tag = BX_CPU_THIS_PTR the_i387.FPU_gettagi((i-tos)&7);
+    if (tag != FPU_Tag_Empty) tag = FPU_tagof(fp);
     double f = pow(2.0, ((0x7fff & fp.exp) - 0x3fff));
     if (fp.exp & 0x8000) f = -f;
 #ifdef _MSC_VER
@@ -531,11 +598,11 @@ void BX_CPU_C::print_state_FPU()
 #else
     f *= fp.fraction*scale_factor;
 #endif
-    fprintf(stderr, "%sFPR%d(%c):        %.10f (raw 0x%04x:%08lx%08lx)\n",
-          i==tos?"=>":"  ",
-          i, 
-          "v0s?"[BX_CPU_THIS_PTR the_i387.FPU_gettagi((i-tos)&7)],
-          f, fp.exp & 0xffff, fp.fraction >> 32, fp.fraction & 0xffffffff);
+    fprintf(stderr, "%sFP%d ST%d(%c):        raw 0x%04x:%08lx%08lx (%.10f)\n",
+          i==tos?"=>":"  ", i, (i-tos)&7,
+          "v0se"[tag],
+          fp.exp & 0xffff, GET32H(fp.fraction), GET32L(fp.fraction),
+          f);
   }
 }
 #endif
