@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: fpu_arith.cc,v 1.10 2007/12/20 20:58:38 sshwarts Exp $
+// $Id: fpu_arith.cc,v 1.14 2008/04/26 19:38:53 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2003 Stanislav Shwartsman
@@ -21,20 +21,19 @@
 //
 /////////////////////////////////////////////////////////////////////////
 
-
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #include "cpu/cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
-
 #if BX_SUPPORT_FPU
+
 float_status_t FPU_pre_exception_handling(Bit16u control_word)
 {
   float_status_t status;
 
   int precision = control_word & FPU_CW_PC;
- 
+
   switch(precision)
   {
      case FPU_PR_32_BITS:
@@ -47,8 +46,8 @@ float_status_t FPU_pre_exception_handling(Bit16u control_word)
        status.float_rounding_precision = 80;
        break;
      default:
-    /* With the precision control bits set to 01 "(reserved)", a 
-       real CPU behaves as if the precision control bits were 
+    /* With the precision control bits set to 01 "(reserved)", a
+       real CPU behaves as if the precision control bits were
        set to 11 "80 bits" */
        status.float_rounding_precision = 80;
   }
@@ -60,9 +59,8 @@ float_status_t FPU_pre_exception_handling(Bit16u control_word)
 
   return status;
 }
-#endif
 
-void BX_CPU_C::FADD_ST0_STj(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_ST0_STj(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -78,7 +76,7 @@ void BX_CPU_C::FADD_ST0_STj(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = BX_READ_FPU_REG(i->rm());
 
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_add(a, b, status);
@@ -92,7 +90,7 @@ void BX_CPU_C::FADD_ST0_STj(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FADD_STi_ST0(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_STi_ST0(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -110,7 +108,7 @@ void BX_CPU_C::FADD_STi_ST0(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(i->rm());
   floatx80 b = BX_READ_FPU_REG(0);
 
-  float_status_t status = 
+  float_status_t status =
 	FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_add(a, b, status);
@@ -119,32 +117,31 @@ void BX_CPU_C::FADD_STi_ST0(bxInstruction_c *i)
       return;
 
   BX_WRITE_FPU_REG(result, i->rm());
-  if (pop_stack) 
+  if (pop_stack)
      BX_CPU_THIS_PTR the_i387.FPU_pop();
 #else
   BX_INFO(("FADD(P)_STi_ST0: required FPU, configure --enable-fpu"));
 #endif
 }
 
-void BX_CPU_C::FADD_SINGLE_REAL(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_SINGLE_REAL(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
-
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_add(BX_READ_FPU_REG(0), 
+  floatx80 result = floatx80_add(BX_READ_FPU_REG(0),
 		float32_to_floatx80(load_reg, status), status);
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
@@ -156,25 +153,24 @@ void BX_CPU_C::FADD_SINGLE_REAL(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FADD_DOUBLE_REAL(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FADD_DOUBLE_REAL(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
-
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_add(BX_READ_FPU_REG(0), 
+  floatx80 result = floatx80_add(BX_READ_FPU_REG(0),
 		float64_to_floatx80(load_reg, status), status);
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
@@ -186,25 +182,24 @@ void BX_CPU_C::FADD_DOUBLE_REAL(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FIADD_WORD_INTEGER(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIADD_WORD_INTEGER(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
-
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = int32_to_floatx80((Bit32s)(load_reg));
 
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_add(a, b, status);
@@ -218,25 +213,24 @@ void BX_CPU_C::FIADD_WORD_INTEGER(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FIADD_DWORD_INTEGER(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIADD_DWORD_INTEGER(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
-
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = int32_to_floatx80(load_reg);
 
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_add(a, b, status);
@@ -250,7 +244,7 @@ void BX_CPU_C::FIADD_DWORD_INTEGER(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FMUL_ST0_STj(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FMUL_ST0_STj(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -266,7 +260,7 @@ void BX_CPU_C::FMUL_ST0_STj(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = BX_READ_FPU_REG(i->rm());
 
-  float_status_t status = 
+  float_status_t status =
 	FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_mul(a, b, status);
@@ -280,7 +274,7 @@ void BX_CPU_C::FMUL_ST0_STj(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FMUL_STi_ST0(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FMUL_STi_ST0(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -298,7 +292,7 @@ void BX_CPU_C::FMUL_STi_ST0(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(i->rm());
   floatx80 b = BX_READ_FPU_REG(0);
 
-  float_status_t status = 
+  float_status_t status =
 	FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_mul(a, b, status);
@@ -307,32 +301,31 @@ void BX_CPU_C::FMUL_STi_ST0(bxInstruction_c *i)
       return;
 
   BX_WRITE_FPU_REG(result, i->rm());
-  if (pop_stack) 
+  if (pop_stack)
      BX_CPU_THIS_PTR the_i387.FPU_pop();
 #else
   BX_INFO(("FMUL(P)_STi_ST0: required FPU, configure --enable-fpu"));
 #endif
 }
 
-void BX_CPU_C::FMUL_SINGLE_REAL(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FMUL_SINGLE_REAL(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
-
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_mul(BX_READ_FPU_REG(0), 
+  floatx80 result = floatx80_mul(BX_READ_FPU_REG(0),
 		float32_to_floatx80(load_reg, status), status);
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
@@ -344,25 +337,24 @@ void BX_CPU_C::FMUL_SINGLE_REAL(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FMUL_DOUBLE_REAL(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FMUL_DOUBLE_REAL(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
-
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_mul(BX_READ_FPU_REG(0), 
+  floatx80 result = floatx80_mul(BX_READ_FPU_REG(0),
 		float64_to_floatx80(load_reg, status), status);
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
@@ -374,25 +366,24 @@ void BX_CPU_C::FMUL_DOUBLE_REAL(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FIMUL_WORD_INTEGER(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIMUL_WORD_INTEGER(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
-
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = int32_to_floatx80((Bit32s)(load_reg));
 
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_mul(a, b, status);
@@ -406,25 +397,24 @@ void BX_CPU_C::FIMUL_WORD_INTEGER(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FIMUL_DWORD_INTEGER(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIMUL_DWORD_INTEGER(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
-
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = int32_to_floatx80(load_reg);
 
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_mul(a, b, status);
@@ -438,7 +428,7 @@ void BX_CPU_C::FIMUL_DWORD_INTEGER(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FSUB_ST0_STj(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUB_ST0_STj(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -454,7 +444,7 @@ void BX_CPU_C::FSUB_ST0_STj(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = BX_READ_FPU_REG(i->rm());
 
-  float_status_t status = 
+  float_status_t status =
 	FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_sub(a, b, status);
@@ -468,7 +458,7 @@ void BX_CPU_C::FSUB_ST0_STj(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FSUBR_ST0_STj(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUBR_ST0_STj(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -484,7 +474,7 @@ void BX_CPU_C::FSUBR_ST0_STj(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(i->rm());
   floatx80 b = BX_READ_FPU_REG(0);
 
-  float_status_t status = 
+  float_status_t status =
 	FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_sub(a, b, status);
@@ -498,7 +488,7 @@ void BX_CPU_C::FSUBR_ST0_STj(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FSUB_STi_ST0(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUB_STi_ST0(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -516,7 +506,7 @@ void BX_CPU_C::FSUB_STi_ST0(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(i->rm());
   floatx80 b = BX_READ_FPU_REG(0);
 
-  float_status_t status = 
+  float_status_t status =
 	FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_sub(a, b, status);
@@ -526,14 +516,14 @@ void BX_CPU_C::FSUB_STi_ST0(bxInstruction_c *i)
 
   BX_WRITE_FPU_REG(result, i->rm());
 
-  if (pop_stack) 
+  if (pop_stack)
      BX_CPU_THIS_PTR the_i387.FPU_pop();
 #else
   BX_INFO(("FSUB(P)_STi_ST0: required FPU, configure --enable-fpu"));
 #endif
 }
 
-void BX_CPU_C::FSUBR_STi_ST0(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUBR_STi_ST0(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -551,7 +541,7 @@ void BX_CPU_C::FSUBR_STi_ST0(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = BX_READ_FPU_REG(i->rm());
 
-  float_status_t status = 
+  float_status_t status =
 	FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_sub(a, b, status);
@@ -561,32 +551,31 @@ void BX_CPU_C::FSUBR_STi_ST0(bxInstruction_c *i)
 
   BX_WRITE_FPU_REG(result, i->rm());
 
-  if (pop_stack) 
+  if (pop_stack)
      BX_CPU_THIS_PTR the_i387.FPU_pop();
 #else
   BX_INFO(("FSUBR(P)_STi_ST0: required FPU, configure --enable-fpu"));
 #endif
 }
 
-void BX_CPU_C::FSUB_SINGLE_REAL(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUB_SINGLE_REAL(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
-
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_sub(BX_READ_FPU_REG(0), 
+  floatx80 result = floatx80_sub(BX_READ_FPU_REG(0),
 		float32_to_floatx80(load_reg, status), status);
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
@@ -598,25 +587,24 @@ void BX_CPU_C::FSUB_SINGLE_REAL(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FSUBR_SINGLE_REAL(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUBR_SINGLE_REAL(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
-
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_sub(float32_to_floatx80(load_reg, status), 
+  floatx80 result = floatx80_sub(float32_to_floatx80(load_reg, status),
 		BX_READ_FPU_REG(0), status);
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
@@ -628,25 +616,24 @@ void BX_CPU_C::FSUBR_SINGLE_REAL(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FSUB_DOUBLE_REAL(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUB_DOUBLE_REAL(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
-
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_sub(BX_READ_FPU_REG(0), 
+  floatx80 result = floatx80_sub(BX_READ_FPU_REG(0),
 		float64_to_floatx80(load_reg, status), status);
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
@@ -658,25 +645,24 @@ void BX_CPU_C::FSUB_DOUBLE_REAL(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FSUBR_DOUBLE_REAL(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSUBR_DOUBLE_REAL(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
-
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_sub(float64_to_floatx80(load_reg, status), 
+  floatx80 result = floatx80_sub(float64_to_floatx80(load_reg, status),
 		BX_READ_FPU_REG(0), status);
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
@@ -688,25 +674,24 @@ void BX_CPU_C::FSUBR_DOUBLE_REAL(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FISUB_WORD_INTEGER(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUB_WORD_INTEGER(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
-
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = int32_to_floatx80((Bit32s)(load_reg));
 
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_sub(a, b, status);
@@ -720,25 +705,24 @@ void BX_CPU_C::FISUB_WORD_INTEGER(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FISUBR_WORD_INTEGER(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUBR_WORD_INTEGER(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
-
   floatx80 a = int32_to_floatx80((Bit32s)(load_reg));
   floatx80 b = BX_READ_FPU_REG(0);
 
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_sub(a, b, status);
@@ -752,28 +736,27 @@ void BX_CPU_C::FISUBR_WORD_INTEGER(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FISUB_DWORD_INTEGER(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUB_DWORD_INTEGER(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
-
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = int32_to_floatx80(load_reg);
 
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_sub(BX_READ_FPU_REG(0), 
+  floatx80 result = floatx80_sub(BX_READ_FPU_REG(0),
 	      int32_to_floatx80(load_reg), status);
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
@@ -785,25 +768,24 @@ void BX_CPU_C::FISUB_DWORD_INTEGER(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FISUBR_DWORD_INTEGER(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FISUBR_DWORD_INTEGER(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
-
   floatx80 a = int32_to_floatx80(load_reg);
   floatx80 b = BX_READ_FPU_REG(0);
 
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_sub(a, b, status);
@@ -817,7 +799,7 @@ void BX_CPU_C::FISUBR_DWORD_INTEGER(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FDIV_ST0_STj(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIV_ST0_STj(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -833,7 +815,7 @@ void BX_CPU_C::FDIV_ST0_STj(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = BX_READ_FPU_REG(i->rm());
 
-  float_status_t status = 
+  float_status_t status =
 	FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_div(a, b, status);
@@ -847,7 +829,7 @@ void BX_CPU_C::FDIV_ST0_STj(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FDIVR_ST0_STj(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIVR_ST0_STj(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -863,7 +845,7 @@ void BX_CPU_C::FDIVR_ST0_STj(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(i->rm());
   floatx80 b = BX_READ_FPU_REG(0);
 
-  float_status_t status = 
+  float_status_t status =
 	FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_div(a, b, status);
@@ -877,7 +859,7 @@ void BX_CPU_C::FDIVR_ST0_STj(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FDIV_STi_ST0(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIV_STi_ST0(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -895,7 +877,7 @@ void BX_CPU_C::FDIV_STi_ST0(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(i->rm());
   floatx80 b = BX_READ_FPU_REG(0);
 
-  float_status_t status = 
+  float_status_t status =
 	FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_div(a, b, status);
@@ -904,14 +886,14 @@ void BX_CPU_C::FDIV_STi_ST0(bxInstruction_c *i)
       return;
 
   BX_WRITE_FPU_REG(result, i->rm());
-  if (pop_stack) 
+  if (pop_stack)
      BX_CPU_THIS_PTR the_i387.FPU_pop();
 #else
   BX_INFO(("FDIV(P)_STi_ST0: required FPU, configure --enable-fpu"));
 #endif
 }
 
-void BX_CPU_C::FDIVR_STi_ST0(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIVR_STi_ST0(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -929,7 +911,7 @@ void BX_CPU_C::FDIVR_STi_ST0(bxInstruction_c *i)
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = BX_READ_FPU_REG(i->rm());
 
-  float_status_t status = 
+  float_status_t status =
 	FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_div(a, b, status);
@@ -938,32 +920,31 @@ void BX_CPU_C::FDIVR_STi_ST0(bxInstruction_c *i)
       return;
 
   BX_WRITE_FPU_REG(result, i->rm());
-  if (pop_stack) 
+  if (pop_stack)
      BX_CPU_THIS_PTR the_i387.FPU_pop();
 #else
   BX_INFO(("FDIVR(P)_STi_ST0: required FPU, configure --enable-fpu"));
 #endif
 }
 
-void BX_CPU_C::FDIV_SINGLE_REAL(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIV_SINGLE_REAL(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
-
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_div(BX_READ_FPU_REG(0), 
+  floatx80 result = floatx80_div(BX_READ_FPU_REG(0),
 		float32_to_floatx80(load_reg, status), status);
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
@@ -975,25 +956,24 @@ void BX_CPU_C::FDIV_SINGLE_REAL(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FDIVR_SINGLE_REAL(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIVR_SINGLE_REAL(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  float32 load_reg = read_virtual_dword(i->seg(), RMAddr(i));
-
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_div(float32_to_floatx80(load_reg, status), 
+  floatx80 result = floatx80_div(float32_to_floatx80(load_reg, status),
 		BX_READ_FPU_REG(0), status);
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
@@ -1005,25 +985,24 @@ void BX_CPU_C::FDIVR_SINGLE_REAL(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FDIV_DOUBLE_REAL(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIV_DOUBLE_REAL(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
-
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_div(BX_READ_FPU_REG(0), 
+  floatx80 result = floatx80_div(BX_READ_FPU_REG(0),
 		float64_to_floatx80(load_reg, status), status);
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
@@ -1035,25 +1014,24 @@ void BX_CPU_C::FDIV_DOUBLE_REAL(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FDIVR_DOUBLE_REAL(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDIVR_DOUBLE_REAL(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  float64 load_reg = read_virtual_qword(i->seg(), RMAddr(i));
-
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
-  floatx80 result = floatx80_div(float64_to_floatx80(load_reg, status), 
+  floatx80 result = floatx80_div(float64_to_floatx80(load_reg, status),
 		BX_READ_FPU_REG(0), status);
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
@@ -1065,25 +1043,24 @@ void BX_CPU_C::FDIVR_DOUBLE_REAL(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FIDIV_WORD_INTEGER(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIV_WORD_INTEGER(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
-
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = int32_to_floatx80((Bit32s)(load_reg));
 
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_div(a, b, status);
@@ -1097,25 +1074,24 @@ void BX_CPU_C::FIDIV_WORD_INTEGER(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FIDIVR_WORD_INTEGER(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIVR_WORD_INTEGER(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  Bit16s load_reg = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
-
   floatx80 a = int32_to_floatx80((Bit32s)(load_reg));
   floatx80 b = BX_READ_FPU_REG(0);
 
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_div(a, b, status);
@@ -1129,25 +1105,24 @@ void BX_CPU_C::FIDIVR_WORD_INTEGER(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FIDIV_DWORD_INTEGER(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIV_DWORD_INTEGER(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
-
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = int32_to_floatx80(load_reg);
 
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_div(a, b, status);
@@ -1161,25 +1136,24 @@ void BX_CPU_C::FIDIV_DWORD_INTEGER(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FIDIVR_DWORD_INTEGER(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FIDIVR_DWORD_INTEGER(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
+  Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
+
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  Bit32s load_reg = (Bit32s) read_virtual_dword(i->seg(), RMAddr(i));
-
   floatx80 a = int32_to_floatx80(load_reg);
   floatx80 b = BX_READ_FPU_REG(0);
 
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_div(a, b, status);
@@ -1193,7 +1167,7 @@ void BX_CPU_C::FIDIVR_DWORD_INTEGER(bxInstruction_c *i)
 #endif
 }
 
-void BX_CPU_C::FSQRT(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSQRT(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -1205,7 +1179,7 @@ void BX_CPU_C::FSQRT(bxInstruction_c *i)
      return;
   }
 
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_sqrt(BX_READ_FPU_REG(0), status);
@@ -1220,7 +1194,7 @@ void BX_CPU_C::FSQRT(bxInstruction_c *i)
 }
 
 /* D9 FC */
-void BX_CPU_C::FRNDINT(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FRNDINT(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -1232,7 +1206,7 @@ void BX_CPU_C::FRNDINT(bxInstruction_c *i)
      return;
   }
 
-  float_status_t status = 
+  float_status_t status =
       FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_round_to_int(BX_READ_FPU_REG(0), status);
@@ -1245,3 +1219,5 @@ void BX_CPU_C::FRNDINT(bxInstruction_c *i)
   BX_INFO(("FRNDINT: required FPU, configure --enable-fpu"));
 #endif
 }
+
+#endif
