@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: fpu_const.cc,v 1.8 2007/03/23 21:27:13 sshwarts Exp $
+// $Id: fpu_const.cc,v 1.12 2008/05/10 13:34:01 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2003 Stanislav Shwartsman
@@ -21,13 +21,11 @@
 //
 /////////////////////////////////////////////////////////////////////////
 
-
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #include "cpu/cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
-
 #if BX_SUPPORT_FPU
 
 #include "softfloatx80.h"
@@ -54,9 +52,7 @@ BX_CPP_INLINE floatx80 FPU_round_const(const floatx80 &a, int adj)
   return result;
 }
 
-#endif
-
-void BX_CPU_C::FLDL2T(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FLDL2T(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -66,18 +62,17 @@ void BX_CPU_C::FLDL2T(bxInstruction_c *i)
   if (! IS_TAG_EMPTY(-1))
   {
       BX_CPU_THIS_PTR FPU_stack_overflow();
-      return; 
+      return;
   }
 
   BX_CPU_THIS_PTR the_i387.FPU_push();
-  BX_WRITE_FPU_REGISTER_AND_TAG(FPU_round_const(Const_L2T, 
-	(FPU_CONTROL_WORD == FPU_RC_UP) ? 1 : 0), FPU_Tag_Valid, 0);
+  BX_WRITE_FPU_REG(FPU_round_const(Const_L2T, (FPU_CONTROL_WORD == FPU_RC_UP) ? 1 : 0), 0);
 #else
   BX_INFO(("FLDL2T: required FPU, configure --enable-fpu"));
 #endif
 }
 
-void BX_CPU_C::FLDL2E(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FLDL2E(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -87,18 +82,17 @@ void BX_CPU_C::FLDL2E(bxInstruction_c *i)
   if (! IS_TAG_EMPTY(-1))
   {
       BX_CPU_THIS_PTR FPU_stack_overflow();
-      return; 
+      return;
   }
 
   BX_CPU_THIS_PTR the_i387.FPU_push();
-  BX_WRITE_FPU_REGISTER_AND_TAG(FPU_round_const(Const_L2E, 
-	DOWN_OR_CHOP() ? -1 : 0), FPU_Tag_Valid, 0);
+  BX_WRITE_FPU_REG(FPU_round_const(Const_L2E, DOWN_OR_CHOP() ? -1 : 0), 0);
 #else
   BX_INFO(("FLDL2E: required FPU, configure --enable-fpu"));
 #endif
 }
 
-void BX_CPU_C::FLDPI(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FLDPI(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -108,18 +102,17 @@ void BX_CPU_C::FLDPI(bxInstruction_c *i)
   if (! IS_TAG_EMPTY(-1))
   {
       BX_CPU_THIS_PTR FPU_stack_overflow();
-      return; 
+      return;
   }
 
   BX_CPU_THIS_PTR the_i387.FPU_push();
-  BX_WRITE_FPU_REGISTER_AND_TAG(FPU_round_const(Const_PI, 
-	DOWN_OR_CHOP() ? -1 : 0), FPU_Tag_Valid, 0);
+  BX_WRITE_FPU_REG(FPU_round_const(Const_PI, DOWN_OR_CHOP() ? -1 : 0), 0);
 #else
   BX_INFO(("FLDPI: required FPU, configure --enable-fpu"));
 #endif
 }
 
-void BX_CPU_C::FLDLG2(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FLDLG2(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -129,18 +122,17 @@ void BX_CPU_C::FLDLG2(bxInstruction_c *i)
   if (! IS_TAG_EMPTY(-1))
   {
       BX_CPU_THIS_PTR FPU_stack_overflow();
-      return; 
+      return;
   }
 
   BX_CPU_THIS_PTR the_i387.FPU_push();
-  BX_WRITE_FPU_REGISTER_AND_TAG(FPU_round_const(Const_LG2, 
-	DOWN_OR_CHOP() ? -1 : 0), FPU_Tag_Valid, 0);
+  BX_WRITE_FPU_REG(FPU_round_const(Const_LG2, DOWN_OR_CHOP() ? -1 : 0), 0);
 #else
   BX_INFO(("FLDLG2: required FPU, configure --enable-fpu"));
 #endif
 }
 
-void BX_CPU_C::FLDLN2(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FLDLN2(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -150,18 +142,17 @@ void BX_CPU_C::FLDLN2(bxInstruction_c *i)
   if (! IS_TAG_EMPTY(-1))
   {
       BX_CPU_THIS_PTR FPU_stack_overflow();
-      return; 
+      return;
   }
 
   BX_CPU_THIS_PTR the_i387.FPU_push();
-  BX_WRITE_FPU_REGISTER_AND_TAG(FPU_round_const(Const_LN2, 
-	DOWN_OR_CHOP() ? -1 : 0), FPU_Tag_Valid, 0);
+  BX_WRITE_FPU_REG(FPU_round_const(Const_LN2, DOWN_OR_CHOP() ? -1 : 0), 0);
 #else
   BX_INFO(("FLDLN2: required FPU, configure --enable-fpu"));
 #endif
 }
 
-void BX_CPU_C::FLD1(bxInstruction_c *i) 
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FLD1(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -171,17 +162,17 @@ void BX_CPU_C::FLD1(bxInstruction_c *i)
   if (! IS_TAG_EMPTY(-1))
   {
       BX_CPU_THIS_PTR FPU_stack_overflow();
-      return; 
+      return;
   }
 
   BX_CPU_THIS_PTR the_i387.FPU_push();
-  BX_WRITE_FPU_REGISTER_AND_TAG(Const_1, FPU_Tag_Valid, 0);
+  BX_WRITE_FPU_REG(Const_1, 0);
 #else
   BX_INFO(("FLD1: required FPU, configure --enable-fpu"));
 #endif
 }
 
-void BX_CPU_C::FLDZ(bxInstruction_c *i)                      
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FLDZ(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -191,12 +182,14 @@ void BX_CPU_C::FLDZ(bxInstruction_c *i)
   if (! IS_TAG_EMPTY(-1))
   {
       BX_CPU_THIS_PTR FPU_stack_overflow();
-      return; 
+      return;
   }
 
   BX_CPU_THIS_PTR the_i387.FPU_push();
-  BX_WRITE_FPU_REGISTER_AND_TAG(Const_Z, FPU_Tag_Zero, 0);
+  BX_WRITE_FPU_REG(Const_Z, 0);
 #else
   BX_INFO(("FLDZ: required FPU, configure --enable-fpu"));
 #endif
 }
+
+#endif
