@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: fpu_trans.cc,v 1.11 2007/03/23 21:27:13 sshwarts Exp $
+// $Id: fpu_trans.cc,v 1.17 2008/05/10 13:34:01 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2003 Stanislav Shwartsman
@@ -21,38 +21,38 @@
 //
 /////////////////////////////////////////////////////////////////////////
 
-
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #include "cpu/cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
+#if BX_SUPPORT_FPU
+
 #include "softfloatx80.h"
 #include "softfloat-specialize.h"
 
 extern float_status_t FPU_pre_exception_handling(Bit16u control_word);
 
 /* D9 F0 */
-void BX_CPU_C::F2XM1(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::F2XM1(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
   clear_C1();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  float_status_t status = 
-	FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word() | FPU_PR_80_BITS);
+  float_status_t status =
+     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word() | FPU_PR_80_BITS);
 
   floatx80 result = f2xm1(BX_READ_FPU_REG(0), status);
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
-      return;
+     return;
 
   BX_WRITE_FPU_REG(result, 0);
 #else
@@ -61,7 +61,7 @@ void BX_CPU_C::F2XM1(bxInstruction_c *i)
 }
 
 /* D9 F1 */
-void BX_CPU_C::FYL2X(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FYL2X(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -74,13 +74,13 @@ void BX_CPU_C::FYL2X(bxInstruction_c *i)
      return;
   }
 
-  float_status_t status = 
-	FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word() | FPU_PR_80_BITS);
+  float_status_t status =
+     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word() | FPU_PR_80_BITS);
 
   floatx80 result = fyl2x(BX_READ_FPU_REG(0), BX_READ_FPU_REG(1), status);
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
-      return;
+     return;
 
   BX_CPU_THIS_PTR the_i387.FPU_pop();
   BX_WRITE_FPU_REG(result, 0);
@@ -90,7 +90,7 @@ void BX_CPU_C::FYL2X(bxInstruction_c *i)
 }
 
 /* D9 F2 */
-void BX_CPU_C::FPTAN(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FPTAN(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -98,58 +98,61 @@ void BX_CPU_C::FPTAN(bxInstruction_c *i)
   clear_C1();
   clear_C2();
 
-  if (! IS_TAG_EMPTY(-1) || IS_TAG_EMPTY(0))
+  if (IS_TAG_EMPTY(0) || ! IS_TAG_EMPTY(-1))
   {
-      BX_CPU_THIS_PTR FPU_exception(FPU_EX_Stack_Overflow);
+     if(IS_TAG_EMPTY(0))
+       BX_CPU_THIS_PTR FPU_exception(FPU_EX_Stack_Underflow);
+     else
+       BX_CPU_THIS_PTR FPU_exception(FPU_EX_Stack_Overflow);
 
-      /* The masked response */
-      if (BX_CPU_THIS_PTR the_i387.is_IA_masked())
-      {
-          BX_WRITE_FPU_REGISTER_AND_TAG(floatx80_default_nan, FPU_Tag_Special, 0);
-          BX_CPU_THIS_PTR the_i387.FPU_push();
-          BX_WRITE_FPU_REGISTER_AND_TAG(floatx80_default_nan, FPU_Tag_Special, 0);
-      }
+     /* The masked response */
+     if (BX_CPU_THIS_PTR the_i387.is_IA_masked())
+     {
+         BX_WRITE_FPU_REG(floatx80_default_nan, 0);
+         BX_CPU_THIS_PTR the_i387.FPU_push();
+         BX_WRITE_FPU_REG(floatx80_default_nan, 0);
+     }
 
-      return; 
+     return;
   }
 
   extern const floatx80 Const_1;
 
-  float_status_t status = 
-	FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word() | FPU_PR_80_BITS);
+  float_status_t status =
+     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word() | FPU_PR_80_BITS);
 
   floatx80 y = BX_READ_FPU_REG(0);
   if (ftan(y, status) == -1)
   {
-      FPU_PARTIAL_STATUS |= FPU_SW_C2;
-      return;
+     FPU_PARTIAL_STATUS |= FPU_SW_C2;
+     return;
   }
 
   if (floatx80_is_nan(y))
   {
-      if (! BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
-      {
-          BX_WRITE_FPU_REGISTER_AND_TAG(y, FPU_Tag_Special, 0);
-          BX_CPU_THIS_PTR the_i387.FPU_push();
-          BX_WRITE_FPU_REGISTER_AND_TAG(y, FPU_Tag_Special, 0);
-      }
+     if (! BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
+     {
+         BX_WRITE_FPU_REG(y, 0);
+         BX_CPU_THIS_PTR the_i387.FPU_push();
+         BX_WRITE_FPU_REG(y, 0);
+     }
 
-      return;
+     return;
   }
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
-      return;
+     return;
 
   BX_WRITE_FPU_REG(y, 0);
   BX_CPU_THIS_PTR the_i387.FPU_push();
-  BX_WRITE_FPU_REGISTER_AND_TAG(Const_1, FPU_Tag_Valid, 0);
+  BX_WRITE_FPU_REG(Const_1, 0);
 #else
   BX_INFO(("FPTAN: required FPU, configure --enable-fpu"));
 #endif
 }
 
 /* D9 F3 */
-void BX_CPU_C::FPATAN(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FPATAN(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
 
@@ -161,13 +164,13 @@ void BX_CPU_C::FPATAN(bxInstruction_c *i)
      return;
   }
 
-  float_status_t status = 
-	FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word() | FPU_PR_80_BITS);
+  float_status_t status =
+     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word() | FPU_PR_80_BITS);
 
   floatx80 result = fpatan(BX_READ_FPU_REG(0), BX_READ_FPU_REG(1), status);
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
-      return;
+     return;
 
   BX_CPU_THIS_PTR the_i387.FPU_pop();
   BX_WRITE_FPU_REG(result, 0);
@@ -177,47 +180,50 @@ void BX_CPU_C::FPATAN(bxInstruction_c *i)
 }
 
 /* D9 F4 */
-void BX_CPU_C::FXTRACT(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FXTRACT(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
 
   clear_C1();
 
-  if (! IS_TAG_EMPTY(-1) || IS_TAG_EMPTY(0))
+  if (IS_TAG_EMPTY(0) || ! IS_TAG_EMPTY(-1))
   {
-      BX_CPU_THIS_PTR FPU_exception(FPU_EX_Stack_Overflow);
+     if(IS_TAG_EMPTY(0))
+       BX_CPU_THIS_PTR FPU_exception(FPU_EX_Stack_Underflow);
+     else
+       BX_CPU_THIS_PTR FPU_exception(FPU_EX_Stack_Overflow);
 
-      /* The masked response */
-      if (BX_CPU_THIS_PTR the_i387.is_IA_masked())
-      {
-          BX_WRITE_FPU_REGISTER_AND_TAG(floatx80_default_nan, FPU_Tag_Special, 0);
-          BX_CPU_THIS_PTR the_i387.FPU_push();
-          BX_WRITE_FPU_REGISTER_AND_TAG(floatx80_default_nan, FPU_Tag_Special, 0);
-      }
+     /* The masked response */
+     if (BX_CPU_THIS_PTR the_i387.is_IA_masked())
+     {
+         BX_WRITE_FPU_REG(floatx80_default_nan, 0);
+         BX_CPU_THIS_PTR the_i387.FPU_push();
+         BX_WRITE_FPU_REG(floatx80_default_nan, 0);
+     }
 
-      return; 
+     return;
   }
 
-  float_status_t status = 
-      FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+  float_status_t status =
+     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 a = BX_READ_FPU_REG(0);
   floatx80 b = floatx80_extract(a, status);
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
-      return;
+     return;
 
-  BX_WRITE_FPU_REG(b, 0);	// exponent
+  BX_WRITE_FPU_REG(b, 0);     // exponent
   BX_CPU_THIS_PTR the_i387.FPU_push();
-  BX_WRITE_FPU_REG(a, 0);	// fraction
+  BX_WRITE_FPU_REG(a, 0);     // fraction
 #else
   BX_INFO(("FXTRACT: required FPU, configure --enable-fpu"));
 #endif
 }
 
 /* D9 F5 */
-void BX_CPU_C::FPREM1(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FPREM1(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -230,8 +236,8 @@ void BX_CPU_C::FPREM1(bxInstruction_c *i)
      return;
   }
 
-  float_status_t status = 
-	FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+  float_status_t status =
+     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   Bit64u quotient;
 
@@ -241,15 +247,15 @@ void BX_CPU_C::FPREM1(bxInstruction_c *i)
   floatx80 result = floatx80_ieee754_remainder(a, b, quotient, status);
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
-      return;
+     return;
 
   int cc = 0;
   if (quotient == (Bit64u) -1) cc = FPU_SW_C2;
   else
   {
-      if (quotient & 1) cc |= FPU_SW_C1;
-      if (quotient & 2) cc |= FPU_SW_C3;
-      if (quotient & 4) cc |= FPU_SW_C0;
+     if (quotient & 1) cc |= FPU_SW_C1;
+     if (quotient & 2) cc |= FPU_SW_C3;
+     if (quotient & 4) cc |= FPU_SW_C0;
   }
   setcc(cc);
 
@@ -260,7 +266,7 @@ void BX_CPU_C::FPREM1(bxInstruction_c *i)
 }
 
 /* D9 F8 */
-void BX_CPU_C::FPREM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FPREM(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -273,8 +279,8 @@ void BX_CPU_C::FPREM(bxInstruction_c *i)
      return;
   }
 
-  float_status_t status = 
-	FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+  float_status_t status =
+     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   Bit64u quotient;
 
@@ -284,15 +290,15 @@ void BX_CPU_C::FPREM(bxInstruction_c *i)
   floatx80 result = floatx80_remainder(a, b, quotient, status);
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
-      return;
+     return;
 
   int cc = 0;
   if (quotient == (Bit64u) -1) cc = FPU_SW_C2;
   else
   {
-      if (quotient & 1) cc |= FPU_SW_C1;
-      if (quotient & 2) cc |= FPU_SW_C3;
-      if (quotient & 4) cc |= FPU_SW_C0;
+     if (quotient & 1) cc |= FPU_SW_C1;
+     if (quotient & 2) cc |= FPU_SW_C3;
+     if (quotient & 4) cc |= FPU_SW_C0;
   }
   setcc(cc);
 
@@ -303,7 +309,7 @@ void BX_CPU_C::FPREM(bxInstruction_c *i)
 }
 
 /* D9 F9 */
-void BX_CPU_C::FYL2XP1(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FYL2XP1(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -316,13 +322,13 @@ void BX_CPU_C::FYL2XP1(bxInstruction_c *i)
      return;
   }
 
-  float_status_t status = 
-	FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word() | FPU_PR_80_BITS);
+  float_status_t status =
+     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word() | FPU_PR_80_BITS);
 
   floatx80 result = fyl2xp1(BX_READ_FPU_REG(0), BX_READ_FPU_REG(1), status);
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
-      return;
+     return;
 
   BX_CPU_THIS_PTR the_i387.FPU_pop();
   BX_WRITE_FPU_REG(result, 0);
@@ -332,7 +338,7 @@ void BX_CPU_C::FYL2XP1(bxInstruction_c *i)
 }
 
 /* D9 FB */
-void BX_CPU_C::FSINCOS(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSINCOS(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -340,34 +346,37 @@ void BX_CPU_C::FSINCOS(bxInstruction_c *i)
   clear_C1();
   clear_C2();
 
-  if (! IS_TAG_EMPTY(-1) || IS_TAG_EMPTY(0))
+  if (IS_TAG_EMPTY(0) || ! IS_TAG_EMPTY(-1))
   {
-      BX_CPU_THIS_PTR FPU_exception(FPU_EX_Stack_Overflow);
+     if(IS_TAG_EMPTY(0))
+       BX_CPU_THIS_PTR FPU_exception(FPU_EX_Stack_Underflow);
+     else
+       BX_CPU_THIS_PTR FPU_exception(FPU_EX_Stack_Overflow);
 
-      /* The masked response */
-      if (BX_CPU_THIS_PTR the_i387.is_IA_masked())
-      {
-          BX_WRITE_FPU_REGISTER_AND_TAG(floatx80_default_nan, FPU_Tag_Special, 0);
-          BX_CPU_THIS_PTR the_i387.FPU_push();
-          BX_WRITE_FPU_REGISTER_AND_TAG(floatx80_default_nan, FPU_Tag_Special, 0);
-      }
+     /* The masked response */
+     if (BX_CPU_THIS_PTR the_i387.is_IA_masked())
+     {
+         BX_WRITE_FPU_REG(floatx80_default_nan, 0);
+         BX_CPU_THIS_PTR the_i387.FPU_push();
+         BX_WRITE_FPU_REG(floatx80_default_nan, 0);
+     }
 
-      return; 
+     return;
   }
 
-  float_status_t status = 
-	FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word() | FPU_PR_80_BITS);
+  float_status_t status =
+     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word() | FPU_PR_80_BITS);
 
   floatx80 y = BX_READ_FPU_REG(0);
   floatx80 sin_y, cos_y;
-  if (fsincos(y, &sin_y, &cos_y, status) == -1) 
+  if (fsincos(y, &sin_y, &cos_y, status) == -1)
   {
-      FPU_PARTIAL_STATUS |= FPU_SW_C2;
-      return;
+     FPU_PARTIAL_STATUS |= FPU_SW_C2;
+     return;
   }
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
-      return;
+     return;
 
   BX_WRITE_FPU_REG(sin_y, 0);
   BX_CPU_THIS_PTR the_i387.FPU_push();
@@ -378,7 +387,7 @@ void BX_CPU_C::FSINCOS(bxInstruction_c *i)
 }
 
 /* D9 FD */
-void BX_CPU_C::FSCALE(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSCALE(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -391,8 +400,8 @@ void BX_CPU_C::FSCALE(bxInstruction_c *i)
      return;
   }
 
-  float_status_t status = 
-	FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
+  float_status_t status =
+     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word());
 
   floatx80 result = floatx80_scale(BX_READ_FPU_REG(0), BX_READ_FPU_REG(1), status);
 
@@ -406,7 +415,7 @@ void BX_CPU_C::FSCALE(bxInstruction_c *i)
 }
 
 /* D9 FE */
-void BX_CPU_C::FSIN(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FSIN(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -414,24 +423,23 @@ void BX_CPU_C::FSIN(bxInstruction_c *i)
   clear_C1();
   clear_C2();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  float_status_t status = 
-	FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word() | FPU_PR_80_BITS);
+  float_status_t status =
+     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word() | FPU_PR_80_BITS);
 
   floatx80 y = BX_READ_FPU_REG(0);
   if (fsin(y, status) == -1)
   {
-      FPU_PARTIAL_STATUS |= FPU_SW_C2;
-      return;
+     FPU_PARTIAL_STATUS |= FPU_SW_C2;
+     return;
   }
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
-      return;
+     return;
 
   BX_WRITE_FPU_REG(y, 0);
 #else
@@ -440,7 +448,7 @@ void BX_CPU_C::FSIN(bxInstruction_c *i)
 }
 
 /* D9 FF */
-void BX_CPU_C::FCOS(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::FCOS(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
@@ -448,27 +456,28 @@ void BX_CPU_C::FCOS(bxInstruction_c *i)
   clear_C1();
   clear_C2();
 
-  if (IS_TAG_EMPTY(0))
-  {
+  if (IS_TAG_EMPTY(0)) {
      BX_CPU_THIS_PTR FPU_stack_underflow(0);
      return;
   }
 
-  float_status_t status = 
-	FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word() | FPU_PR_80_BITS);
+  float_status_t status =
+     FPU_pre_exception_handling(BX_CPU_THIS_PTR the_i387.get_control_word() | FPU_PR_80_BITS);
 
   floatx80 y = BX_READ_FPU_REG(0);
   if (fcos(y, status) == -1)
   {
-      FPU_PARTIAL_STATUS |= FPU_SW_C2;
-      return;
+     FPU_PARTIAL_STATUS |= FPU_SW_C2;
+     return;
   }
 
   if (BX_CPU_THIS_PTR FPU_exception(status.float_exception_flags))
-      return;
+     return;
 
   BX_WRITE_FPU_REG(y, 0);
 #else
   BX_INFO(("FCOS: required FPU, configure --enable-fpu"));
 #endif
 }
+
+#endif
