@@ -1,3 +1,10 @@
+/////////////////////////////////////////////////////////////////////////
+// $Id: sdl.h,v 1.6 2005/04/25 16:20:14 vruppert Exp $
+/////////////////////////////////////////////////////////////////////////
+
+#ifndef BX_SDL_H
+#define BX_SDL_H
+
 #define BX_HEADERBAR_FG_RED	0x10
 #define BX_HEADERBAR_FG_GREEN	0x10
 #define BX_HEADERBAR_FG_BLUE	0x10
@@ -777,262 +784,4 @@ unsigned char sdl_font8x8[256][8] = {
   {   0,   0,  60,  60,  60,  60,   0,   0 },  // 254
   {   0,   0,   0,   0,   0,   0,   0,   0 }}; // 255
 
-/*
-unsigned char sdl_palette[256][3] = {
-  {   0,   0,   0 },  // 0
-  {   0,   0, 168 },  // 1
-  {   0, 168,   0 },  // 2
-  {   0, 168, 168 },  // 3
-  { 168,   0,   0 },  // 4
-  { 168,   0, 168 },  // 5
-  { 168,  84,   0 },  // 6
-  { 168, 168, 168 },  // 7
-  {  84,  84,  84 },  // 8
-  {  84,  84, 252 },  // 9
-  {  84, 252,  84 },  // 10
-  {  84, 252, 252 },  // 11
-  { 252,  84,  84 },  // 12
-  { 252,  84, 252 },  // 13
-  { 252, 252,  84 },  // 14
-  { 252, 252, 252 },  // 15
-  {   0,   0,   0 },  // 16
-  {  20,  20,  20 },  // 17
-  {  32,  32,  32 },  // 18
-  {  44,  44,  44 },  // 19
-  {  56,  56,  56 },  // 20
-  {  68,  68,  68 },  // 21
-  {  80,  80,  80 },  // 22
-  {  96,  96,  96 },  // 23
-  { 112, 112, 112 },  // 24
-  { 128, 128, 128 },  // 25
-  { 144, 144, 144 },  // 26
-  { 160, 160, 160 },  // 27
-  { 180, 180, 180 },  // 28
-  { 200, 200, 200 },  // 29
-  { 224, 224, 224 },  // 30
-  { 252, 252, 252 },  // 31
-  {   0,   0, 252 },  // 32
-  {  64,   0, 252 },  // 33
-  { 124,   0, 252 },  // 34
-  { 188,   0, 252 },  // 35
-  { 252,   0, 252 },  // 36
-  { 252,   0, 188 },  // 37
-  { 252,   0, 124 },  // 38
-  { 252,   0,  64 },  // 39
-  { 252,   0,   0 },  // 40
-  { 252,  64,   0 },  // 41
-  { 252, 124,   0 },  // 42
-  { 252, 188,   0 },  // 43
-  { 252, 252,   0 },  // 44
-  { 188, 252,   0 },  // 45
-  { 124, 252,   0 },  // 46
-  {  64, 252,   0 },  // 47
-  {   0, 252,   0 },  // 48
-  {   0, 252,  64 },  // 49
-  {   0, 252, 124 },  // 50
-  {   0, 252, 188 },  // 51
-  {   0, 252, 252 },  // 52
-  {   0, 188, 252 },  // 53
-  {   0, 124, 252 },  // 54
-  {   0,  64, 252 },  // 55
-  { 124, 124, 252 },  // 56
-  { 156, 124, 252 },  // 57
-  { 188, 124, 252 },  // 58
-  { 220, 124, 252 },  // 59
-  { 252, 124, 252 },  // 60
-  { 252, 124, 220 },  // 61
-  { 252, 124, 188 },  // 62
-  { 252, 124, 156 },  // 63
-  { 252, 124, 124 },  // 64
-  { 252, 156, 124 },  // 65
-  { 252, 188, 124 },  // 66
-  { 252, 220, 124 },  // 67
-  { 252, 252, 124 },  // 68
-  { 220, 252, 124 },  // 69
-  { 188, 252, 124 },  // 70
-  { 156, 252, 124 },  // 71
-  { 124, 252, 124 },  // 72
-  { 124, 252, 156 },  // 73
-  { 124, 252, 188 },  // 74
-  { 124, 252, 220 },  // 75
-  { 124, 252, 252 },  // 76
-  { 124, 220, 252 },  // 77
-  { 124, 188, 252 },  // 78
-  { 124, 156, 252 },  // 79
-  { 180, 180, 252 },  // 80
-  { 196, 180, 252 },  // 81
-  { 216, 180, 252 },  // 82
-  { 232, 180, 252 },  // 83
-  { 252, 180, 252 },  // 84
-  { 252, 180, 232 },  // 85
-  { 252, 180, 216 },  // 86
-  { 252, 180, 196 },  // 87
-  { 252, 180, 180 },  // 88
-  { 252, 196, 180 },  // 89
-  { 252, 216, 180 },  // 90
-  { 252, 232, 180 },  // 91
-  { 252, 252, 180 },  // 92
-  { 232, 252, 180 },  // 93
-  { 216, 252, 180 },  // 94
-  { 196, 252, 180 },  // 95
-  { 180, 252, 180 },  // 96
-  { 180, 252, 196 },  // 97
-  { 180, 252, 216 },  // 98
-  { 180, 252, 232 },  // 99
-  { 180, 252, 252 },  // 100
-  { 180, 232, 252 },  // 101
-  { 180, 216, 252 },  // 102
-  { 180, 196, 252 },  // 103
-  {   0,   0, 112 },  // 104
-  {  28,   0, 112 },  // 105
-  {  56,   0, 112 },  // 106
-  {  84,   0, 112 },  // 107
-  { 112,   0, 112 },  // 108
-  { 112,   0,  84 },  // 109
-  { 112,   0,  56 },  // 110
-  { 112,   0,  28 },  // 111
-  { 112,   0,   0 },  // 112
-  { 112,  28,   0 },  // 113
-  { 112,  56,   0 },  // 114
-  { 112,  84,   0 },  // 115
-  { 112, 112,   0 },  // 116
-  {  84, 112,   0 },  // 117
-  {  56, 112,   0 },  // 118
-  {  28, 112,   0 },  // 119
-  {   0, 112,   0 },  // 120
-  {   0, 112,  28 },  // 121
-  {   0, 112,  56 },  // 122
-  {   0, 112,  84 },  // 123
-  {   0, 112, 112 },  // 124
-  {   0,  84, 112 },  // 125
-  {   0,  56, 112 },  // 126
-  {   0,  28, 112 },  // 127
-  {  56,  56, 112 },  // 128
-  {  68,  56, 112 },  // 129
-  {  84,  56, 112 },  // 130
-  {  96,  56, 112 },  // 131
-  { 112,  56, 112 },  // 132
-  { 112,  56,  96 },  // 133
-  { 112,  56,  84 },  // 134
-  { 112,  56,  68 },  // 135
-  { 112,  56,  56 },  // 136
-  { 112,  68,  56 },  // 137
-  { 112,  84,  56 },  // 138
-  { 112,  96,  56 },  // 139
-  { 112, 112,  56 },  // 140
-  {  96, 112,  56 },  // 141
-  {  84, 112,  56 },  // 142
-  {  68, 112,  56 },  // 143
-  {  56, 112,  56 },  // 144
-  {  56, 112,  68 },  // 145
-  {  56, 112,  84 },  // 146
-  {  56, 112,  96 },  // 147
-  {  56, 112, 112 },  // 148
-  {  56,  96, 112 },  // 149
-  {  56,  84, 112 },  // 150
-  {  56,  68, 112 },  // 151
-  {  80,  80, 112 },  // 152
-  {  88,  80, 112 },  // 153
-  {  96,  80, 112 },  // 154
-  { 104,  80, 112 },  // 155
-  { 112,  80, 112 },  // 156
-  { 112,  80, 104 },  // 157
-  { 112,  80,  96 },  // 158
-  { 112,  80,  88 },  // 159
-  { 112,  80,  80 },  // 160
-  { 112,  88,  80 },  // 161
-  { 112,  96,  80 },  // 162
-  { 112, 104,  80 },  // 163
-  { 112, 112,  80 },  // 164
-  { 104, 112,  80 },  // 165
-  {  96, 112,  80 },  // 166
-  {  88, 112,  80 },  // 167
-  {  80, 112,  80 },  // 168
-  {  80, 112,  88 },  // 169
-  {  80, 112,  96 },  // 170
-  {  80, 112, 104 },  // 171
-  {  80, 112, 112 },  // 172
-  {  80, 104, 112 },  // 173
-  {  80,  96, 112 },  // 174
-  {  80,  88, 112 },  // 175
-  {   0,   0,  64 },  // 176
-  {  16,   0,  64 },  // 177
-  {  32,   0,  64 },  // 178
-  {  48,   0,  64 },  // 179
-  {  64,   0,  64 },  // 180
-  {  64,   0,  48 },  // 181
-  {  64,   0,  32 },  // 182
-  {  64,   0,  16 },  // 183
-  {  64,   0,   0 },  // 184
-  {  64,  16,   0 },  // 185
-  {  64,  32,   0 },  // 186
-  {  64,  48,   0 },  // 187
-  {  64,  64,   0 },  // 188
-  {  48,  64,   0 },  // 189
-  {  32,  64,   0 },  // 190
-  {  16,  64,   0 },  // 191
-  {   0,  64,   0 },  // 192
-  {   0,  64,  16 },  // 193
-  {   0,  64,  32 },  // 194
-  {   0,  64,  48 },  // 195
-  {   0,  64,  64 },  // 196
-  {   0,  48,  64 },  // 197
-  {   0,  32,  64 },  // 198
-  {   0,  16,  64 },  // 199
-  {  32,  32,  64 },  // 200
-  {  40,  32,  64 },  // 201
-  {  48,  32,  64 },  // 202
-  {  56,  32,  64 },  // 203
-  {  64,  32,  64 },  // 204
-  {  64,  32,  56 },  // 205
-  {  64,  32,  48 },  // 206
-  {  64,  32,  40 },  // 207
-  {  64,  32,  32 },  // 208
-  {  64,  40,  32 },  // 209
-  {  64,  48,  32 },  // 210
-  {  64,  56,  32 },  // 211
-  {  64,  64,  32 },  // 212
-  {  56,  64,  32 },  // 213
-  {  48,  64,  32 },  // 214
-  {  40,  64,  32 },  // 215
-  {  32,  64,  32 },  // 216
-  {  32,  64,  40 },  // 217
-  {  32,  64,  48 },  // 218
-  {  32,  64,  56 },  // 219
-  {  32,  64,  64 },  // 220
-  {  32,  56,  64 },  // 221
-  {  32,  48,  64 },  // 222
-  {  32,  40,  64 },  // 223
-  {  44,  44,  64 },  // 224
-  {  48,  44,  64 },  // 225
-  {  52,  44,  64 },  // 226
-  {  60,  44,  64 },  // 227
-  {  64,  44,  64 },  // 228
-  {  64,  44,  60 },  // 229
-  {  64,  44,  52 },  // 230
-  {  64,  44,  48 },  // 231
-  {  64,  44,  44 },  // 232
-  {  64,  48,  44 },  // 233
-  {  64,  52,  44 },  // 234
-  {  64,  60,  44 },  // 235
-  {  64,  64,  44 },  // 236
-  {  60,  64,  44 },  // 237
-  {  52,  64,  44 },  // 238
-  {  48,  64,  44 },  // 239
-  {  44,  64,  44 },  // 240
-  {  44,  64,  48 },  // 241
-  {  44,  64,  52 },  // 242
-  {  44,  64,  60 },  // 243
-  {  44,  64,  64 },  // 244
-  {  44,  60,  64 },  // 245
-  {  44,  52,  64 },  // 246
-  {  44,  48,  64 },  // 247
-  {   0,   0,   0 },  // 248
-  {   0,   0,   0 },  // 249
-  {   0,   0,   0 },  // 250
-  {   0,   0,   0 },  // 251
-  {   0,   0,   0 },  // 252
-  {   0,   0,   0 },  // 253
-  {   0,   0,   0 },  // 254
-  { 112,  97, 108 }}; // 255
-*/
+#endif
