@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: win32.cc,v 1.114 2007/12/03 20:38:55 sshwarts Exp $
+// $Id: win32.cc,v 1.120 2008/06/01 10:56:29 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -23,13 +23,14 @@
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+/////////////////////////////////////////////////////////////////////////
 
 //  Much of this file was written by:
 //  David Ross
 //  dross@pobox.com
 
 // Define BX_PLUGGABLE in files that can be compiled into plugins.  For
-// platforms that require a special tag on exported symbols, BX_PLUGGABLE 
+// platforms that require a special tag on exported symbols, BX_PLUGGABLE
 // is used to know when we are exporting symbols and when we are importing.
 #define BX_PLUGGABLE
 
@@ -205,7 +206,7 @@ sharedThreadInfo stInfo;
 
 LRESULT CALLBACK mainWndProc (HWND, UINT, WPARAM, LPARAM);
 LRESULT CALLBACK simWndProc (HWND, UINT, WPARAM, LPARAM);
-VOID UIThread(PVOID);
+VOID CDECL UIThread(PVOID);
 void SetStatusText(int Num, const char *Text, bx_bool active);
 void terminateEmul(int);
 void create_vga_font(void);
@@ -343,20 +344,20 @@ Bit32u win32_to_bx_key[2][0x100] =
     0,
     0,
     /* 0x70 - 0x7f */
-    0,                  /* Todo: "Katakana" key ( ibm 133 ) for Japanese 106 keyboard */
+    0,                  /* Todo: "Katakana" key (ibm 133) for Japanese 106 keyboard */
     0,
     0,
-    0,                  /* Todo: "Ro" key ( ibm 56 ) for Japanese 106 keyboard */
+    0,                  /* Todo: "Ro" key (ibm 56) for Japanese 106 keyboard */
     0,
     0,
     0,
     0,
     0,
-    0,                  /* Todo: "convert" key ( ibm 132 ) for Japanese 106 keyboard */
+    0,                  /* Todo: "convert" key (ibm 132) for Japanese 106 keyboard */
     0,
-    0,                  /* Todo: "non-convert" key ( ibm 131 ) for Japanese 106 keyboard */
+    0,                  /* Todo: "non-convert" key (ibm 131) for Japanese 106 keyboard */
     0,
-    0,                  /* Todo: "Yen" key ( ibm 14 ) for Japanese 106 keyboard */
+    0,                  /* Todo: "Yen" key (ibm 14) for Japanese 106 keyboard */
     0,
     0,
   },
@@ -488,68 +489,67 @@ Bit32u win32_to_bx_key[2][0x100] =
 #if  defined(__MINGW32__) || defined(_MSC_VER)
   VOID CALLBACK MyTimer(HWND,UINT,UINT,DWORD);
   void alarm(int);
-  void bx_signal_handler(int);
 #endif
 
-static void processMouseXY( int x, int y, int z, int windows_state, int implied_state_change)
+static void processMouseXY(int x, int y, int z, int windows_state, int implied_state_change)
 {
   int bx_state;
   int old_bx_state;
-  EnterCriticalSection( &stInfo.mouseCS);
-  bx_state=( ( windows_state & MK_LBUTTON) ? 1 : 0 ) + ( ( windows_state & MK_RBUTTON) ? 2 : 0) +
-           ( ( windows_state & MK_MBUTTON) ? 4 : 0);
+  EnterCriticalSection(&stInfo.mouseCS);
+  bx_state=((windows_state & MK_LBUTTON) ? 1 : 0) + ((windows_state & MK_RBUTTON) ? 2 : 0) +
+           ((windows_state & MK_MBUTTON) ? 4 : 0);
   old_bx_state=bx_state ^ implied_state_change;
-  if ( old_bx_state!=mouse_button_state)
+  if (old_bx_state!=mouse_button_state)
   {
     /* Make up for missing message */
-    BX_INFO(( "&&&missing mouse state change"));
-    EnterCriticalSection( &stInfo.keyCS);
+    BX_INFO(("&&&missing mouse state change"));
+    EnterCriticalSection(&stInfo.keyCS);
     enq_mouse_event();
     mouse_button_state=old_bx_state;
-    enq_key_event( mouse_button_state, MOUSE_PRESSED);
-    LeaveCriticalSection( &stInfo.keyCS);
+    enq_key_event(mouse_button_state, MOUSE_PRESSED);
+    LeaveCriticalSection(&stInfo.keyCS);
   }
   ms_ydelta=ms_savedy-y;
   ms_xdelta=x-ms_savedx;
   ms_zdelta=z;
   ms_lastx=x;
   ms_lasty=y;
-  if ( bx_state!=mouse_button_state)
+  if (bx_state!=mouse_button_state)
   {
-    EnterCriticalSection( &stInfo.keyCS);
+    EnterCriticalSection(&stInfo.keyCS);
     enq_mouse_event();
     mouse_button_state=bx_state;
-    enq_key_event( mouse_button_state, MOUSE_PRESSED);
-    LeaveCriticalSection( &stInfo.keyCS);
+    enq_key_event(mouse_button_state, MOUSE_PRESSED);
+    LeaveCriticalSection(&stInfo.keyCS);
   }
-  LeaveCriticalSection( &stInfo.mouseCS);
+  LeaveCriticalSection(&stInfo.mouseCS);
 }
 
 static void resetDelta()
 {
-  EnterCriticalSection( &stInfo.mouseCS);
+  EnterCriticalSection(&stInfo.mouseCS);
   ms_savedx=ms_lastx;
   ms_savedy=ms_lasty;
   ms_ydelta=ms_xdelta=ms_zdelta=0;
-  LeaveCriticalSection( &stInfo.mouseCS);
+  LeaveCriticalSection(&stInfo.mouseCS);
 }
 
 static void cursorWarped()
 {
-  EnterCriticalSection( &stInfo.mouseCS);
-  EnterCriticalSection( &stInfo.keyCS);
+  EnterCriticalSection(&stInfo.mouseCS);
+  EnterCriticalSection(&stInfo.keyCS);
   enq_mouse_event();
-  LeaveCriticalSection( &stInfo.keyCS);
+  LeaveCriticalSection(&stInfo.keyCS);
   ms_lastx=stretched_x/2;
   ms_lasty=stretched_y/2;
   ms_savedx=ms_lastx;
   ms_savedy=ms_lasty;
-  LeaveCriticalSection( &stInfo.mouseCS);
+  LeaveCriticalSection(&stInfo.mouseCS);
 }
 
 // GUI thread must be dead/done in order to call terminateEmul
-void terminateEmul(int reason) {
-
+void terminateEmul(int reason)
+{
   // We know that Critical Sections were inited when x_tilesize has been set
   // See bx_win32_gui_c::specific_init
   if (x_tilesize != 0) {
@@ -562,7 +562,7 @@ void terminateEmul(int reason) {
   if (MemoryDC) DeleteDC (MemoryDC);
   if (MemoryBitmap) DeleteObject (MemoryBitmap);
 
-  if ( bitmap_info) delete[] (char*)bitmap_info;
+  if (bitmap_info) delete[] (char*)bitmap_info;
 
   for (unsigned b=0; b<bx_bitmap_entries; b++)
     if (bx_bitmaps[b].bmap) DeleteObject(bx_bitmaps[b].bmap);
@@ -638,7 +638,7 @@ void bx_win32_gui_c::specific_init(int argc, char **argv, unsigned
   mouseCaptureMode = FALSE;
   mouseCaptureNew = FALSE;
   mouseToggleReq = FALSE;
-  
+
   mouse_buttons = GetSystemMetrics(SM_CMOUSEBUTTONS);
   BX_INFO(("Number of Mouse Buttons = %d", mouse_buttons));
   if (mouse_buttons == 2) {
@@ -646,7 +646,7 @@ void bx_win32_gui_c::specific_init(int argc, char **argv, unsigned
     szMouseDisable = "CTRL + Lbutton + Rbutton disables mouse";
     szMouseTooltip = "Enable mouse capture\nUse CTRL + Lbutton + Rbutton to release";
   }
-  
+
   // parse win32 specific options
   if (argc > 1) {
     for (i = 1; i < argc; i++) {
@@ -663,13 +663,13 @@ void bx_win32_gui_c::specific_init(int argc, char **argv, unsigned
       }
     }
   }
-  
+
   if (legacyF12) {
     szMouseEnable = "Press F12 to enable mouse ";
     szMouseDisable = "Press F12 to disable mouse";
     szMouseTooltip = "Enable mouse capture\nUse F12 to release";
   }
-  
+
   stInfo.hInstance = GetModuleHandle(NULL);
 
   UNUSED(headerbar_y);
@@ -798,7 +798,8 @@ void resize_main_window()
 }
 
 // This thread controls the GUI window.
-VOID UIThread(PVOID pvoid) {
+VOID CDECL UIThread(PVOID pvoid)
+{
   MSG msg;
   HDC hdc;
   WNDCLASS wndclass;
@@ -873,7 +874,7 @@ VOID UIThread(PVOID pvoid) {
     }
     SetStatusText(0, szMouseEnable, TRUE);
 
-    stInfo.simWnd = CreateWindowEx( sim_exstyle,
+    stInfo.simWnd = CreateWindowEx(sim_exstyle,
                       "SIMWINDOW",
                       "",
                       sim_style,
@@ -956,7 +957,7 @@ void SetStatusText(int Num, const char *Text, bx_bool active)
   UpdateWindow(hwndSB);
 }
 
-void 
+void
 bx_win32_gui_c::statusbar_setitem(int element, bx_bool active)
 {
   if (element < 0) {
@@ -1010,7 +1011,7 @@ LRESULT CALLBACK mainWndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
       int rect_data[] = { 1, 0, IsWindowVisible(hwndTB),
        100, IsWindowVisible(hwndSB), 0x7712, 0, 0 };
       RECT R;
-      GetEffectiveClientRect( hwnd, &R, rect_data );
+      GetEffectiveClientRect(hwnd, &R, rect_data);
       x = R.right - R.left;
       y = R.bottom - R.top;
       MoveWindow(stInfo.simWnd, R.left, R.top, x, y, TRUE);
@@ -1100,7 +1101,7 @@ LRESULT CALLBACK simWndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
       SetMouseCapture();
     }
     // If mouse escaped, bring it back
-    if ( mouseCaptureMode)
+    if (mouseCaptureMode)
     {
       pt.x = 0;
       pt.y = 0;
@@ -1140,7 +1141,7 @@ LRESULT CALLBACK simWndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
 
   case WM_MOUSEMOVE:
     if (!mouseModeChange) {
-      processMouseXY( LOWORD(lParam), HIWORD(lParam), 0, wParam, 0);
+      processMouseXY(LOWORD(lParam), HIWORD(lParam), 0, wParam, 0);
     }
     return 0;
 
@@ -1152,7 +1153,7 @@ LRESULT CALLBACK simWndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
       pt.x = LOWORD(lParam);
       pt.y = HIWORD(lParam);
       ScreenToClient(stInfo.simWnd, &pt);
-      processMouseXY( pt.x, pt.y, (Bit16s) HIWORD(wParam) / 120, LOWORD(wParam), 0);
+      processMouseXY(pt.x, pt.y, (Bit16s) HIWORD(wParam) / 120, LOWORD(wParam), 0);
     }
     return 0;
 
@@ -1167,11 +1168,11 @@ LRESULT CALLBACK simWndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
       } else if (mouseModeChange && (iMsg == WM_LBUTTONUP)) {
         mouseModeChange = FALSE;
       } else {
-        processMouseXY( LOWORD(lParam), HIWORD(lParam), 0, wParam, 1);
+        processMouseXY(LOWORD(lParam), HIWORD(lParam), 0, wParam, 1);
       }
       return 0;
     }
-    processMouseXY( LOWORD(lParam), HIWORD(lParam), 0, wParam, 1);
+    processMouseXY(LOWORD(lParam), HIWORD(lParam), 0, wParam, 1);
     return 0;
 
   case WM_MBUTTONDOWN:
@@ -1184,7 +1185,7 @@ LRESULT CALLBACK simWndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
     } else if (mouseModeChange && (iMsg == WM_MBUTTONUP)) {
       mouseModeChange = FALSE;
     } else {
-      processMouseXY( LOWORD(lParam), HIWORD(lParam), 0, wParam, 4);
+      processMouseXY(LOWORD(lParam), HIWORD(lParam), 0, wParam, 4);
     }
     return 0;
 
@@ -1199,11 +1200,11 @@ LRESULT CALLBACK simWndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
       } else if (mouseModeChange && (iMsg == WM_RBUTTONUP)) {
         mouseModeChange = FALSE;
       } else {
-        processMouseXY( LOWORD(lParam), HIWORD(lParam), 0, wParam, 2);
+        processMouseXY(LOWORD(lParam), HIWORD(lParam), 0, wParam, 2);
       }
       return 0;
     }
-    processMouseXY( LOWORD(lParam), HIWORD(lParam), 0, wParam, 2);
+    processMouseXY(LOWORD(lParam), HIWORD(lParam), 0, wParam, 2);
     return 0;
 
   case WM_CLOSE:
@@ -1233,17 +1234,16 @@ LRESULT CALLBACK simWndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
   case WM_SYSKEYUP:
     // check if it's keyup, alt key, non-repeat
     // see http://msdn2.microsoft.com/en-us/library/ms646267.aspx
-    if (wParam == VK_RETURN) {
-      if ((HIWORD(lParam) & BX_SYSKEY) == (KF_ALTDOWN | KF_UP)) {
-        if (!saveParent) {
-          BX_INFO(("entering fullscreen mode"));
-          theGui->dimension_update(desktop_x, desktop_y,
-                                   0, 0, current_bpp);
-        } else {
-          BX_INFO(("leaving fullscreen mode"));
-          theGui->dimension_update(dimension_x, desktop_y - 1,
+    if ((wParam == VK_RETURN) &&
+        ((HIWORD(lParam) & BX_SYSKEY) == (KF_ALTDOWN | KF_UP))) {
+      if (!saveParent) {
+        BX_INFO(("entering fullscreen mode"));
+        theGui->dimension_update(desktop_x, desktop_y,
                                  0, 0, current_bpp);
-        }
+      } else {
+        BX_INFO(("leaving fullscreen mode"));
+        theGui->dimension_update(dimension_x, desktop_y - 1,
+                               0, 0, current_bpp);
       }
     } else {
       EnterCriticalSection(&stInfo.keyCS);
@@ -1349,7 +1349,7 @@ void enq_key_event(Bit32u key, Bit32u press_release)
     }
   }
   if (((tail+1) % SCANCODE_BUFSIZE) == head) {
-    BX_ERROR(( "enq_scancode: buffer full"));
+    BX_ERROR(("enq_scancode: buffer full"));
     return;
   }
   keyevents[tail].key_event = key | press_release;
@@ -1358,11 +1358,11 @@ void enq_key_event(Bit32u key, Bit32u press_release)
 
 void enq_mouse_event(void)
 {
-  EnterCriticalSection( &stInfo.mouseCS);
-  if ( ms_xdelta || ms_ydelta || ms_zdelta)
+  EnterCriticalSection(&stInfo.mouseCS);
+  if (ms_xdelta || ms_ydelta || ms_zdelta)
   {
     if (((tail+1) % SCANCODE_BUFSIZE) == head) {
-      BX_ERROR(( "enq_scancode: buffer full" ));
+      BX_ERROR(("enq_scancode: buffer full"));
       return;
     }
     QueueEvent& current=keyevents[tail];
@@ -1374,13 +1374,14 @@ void enq_mouse_event(void)
     resetDelta();
     tail = (tail + 1) % SCANCODE_BUFSIZE;
   }
-  LeaveCriticalSection( &stInfo.mouseCS);
+  LeaveCriticalSection(&stInfo.mouseCS);
 }
 
-QueueEvent* deq_key_event(void) {
+QueueEvent* deq_key_event(void)
+{
   QueueEvent* key;
 
-  if ( head == tail ) {
+  if (head == tail) {
     BX_ERROR(("deq_scancode: buffer empty"));
     return((QueueEvent*)0);
   }
@@ -1397,7 +1398,8 @@ QueueEvent* deq_key_event(void) {
 // the gui code can poll for keyboard, mouse, and other
 // relevant events.
 
-void bx_win32_gui_c::handle_events(void) {
+void bx_win32_gui_c::handle_events(void)
+{
   Bit32u key;
   Bit32u key_event;
 
@@ -1410,17 +1412,17 @@ void bx_win32_gui_c::handle_events(void) {
   EnterCriticalSection(&stInfo.keyCS);
   while (head != tail) {
     QueueEvent* queue_event=deq_key_event();
-    if ( ! queue_event)
+    if (! queue_event)
       break;
     key = queue_event->key_event;
-    if ( key==MOUSE_MOTION)
+    if (key==MOUSE_MOTION)
     {
-      DEV_mouse_motion_ext( queue_event->mouse_x,
+      DEV_mouse_motion_ext(queue_event->mouse_x,
         queue_event->mouse_y, queue_event->mouse_z, queue_event->mouse_button_state);
     }
     // Check for mouse buttons first
-    else if ( key & MOUSE_PRESSED) {
-      DEV_mouse_motion_ext( 0, 0, 0, LOWORD(key));
+    else if (key & MOUSE_PRESSED) {
+      DEV_mouse_motion_ext(0, 0, 0, LOWORD(key));
     }
     else if (key & HEADERBAR_CLICKED) {
       headerbar_click(LOWORD(key));
@@ -1446,25 +1448,26 @@ void bx_win32_gui_c::handle_events(void) {
 // Called periodically, requesting that the gui code flush all pending
 // screen update requests.
 
-void bx_win32_gui_c::flush(void) {
-  EnterCriticalSection( &stInfo.drawCS);
+void bx_win32_gui_c::flush(void)
+{
+  EnterCriticalSection(&stInfo.drawCS);
   if (updated_area_valid) {
     // slight bugfix
 	updated_area.right++;
 	updated_area.bottom++;
-	InvalidateRect( stInfo.simWnd, &updated_area, FALSE);
+	InvalidateRect(stInfo.simWnd, &updated_area, FALSE);
 	updated_area_valid = FALSE;
   }
-  LeaveCriticalSection( &stInfo.drawCS);
+  LeaveCriticalSection(&stInfo.drawCS);
 }
 
-
 // ::CLEAR_SCREEN()
 //
 // Called to request that the VGA region is cleared.  Don't
 // clear the area that defines the headerbar.
 
-void bx_win32_gui_c::clear_screen(void) {
+void bx_win32_gui_c::clear_screen(void)
+{
   HGDIOBJ oldObj;
 
   if (!stInfo.UIinited) return;
@@ -1499,18 +1502,17 @@ void bx_win32_gui_c::clear_screen(void) {
 // cursor_y: new y location of cursor
 // tm_info:  this structure contains information for additional
 //           features in text mode (cursor shape, line offset,...)
-// nrows:    number of text rows (unused here)
 
 void bx_win32_gui_c::text_update(Bit8u *old_text, Bit8u *new_text,
 			   unsigned long cursor_x, unsigned long cursor_y,
-                           bx_vga_tminfo_t tm_info, unsigned nrows)
+                           bx_vga_tminfo_t tm_info)
 {
   HDC hdc;
   unsigned char data[64];
   Bit8u *old_line, *new_line;
   Bit8u cAttr, cChar;
   unsigned int curs, hchars, i, offset, rows, x, y, xc, yc;
-  BOOL forceUpdate = FALSE;
+  BOOL forceUpdate = FALSE, blink_state, blink_mode;
 #if !BX_USE_WINDOWS_FONTS
   Bit8u *text_base;
   Bit8u cfwidth, cfheight, cfheight2, font_col, font_row, font_row2;
@@ -1523,7 +1525,12 @@ void bx_win32_gui_c::text_update(Bit8u *old_text, Bit8u *new_text,
 
   EnterCriticalSection(&stInfo.drawCS);
 
-  UNUSED(nrows);
+  blink_mode = (tm_info.blink_flags & BX_TEXT_BLINK_MODE) > 0;
+  blink_state = (tm_info.blink_flags & BX_TEXT_BLINK_STATE) > 0;
+  if (blink_mode) {
+    if (tm_info.blink_flags & BX_TEXT_BLINK_TOGGLE)
+      forceUpdate = 1;
+  }
   if (charmap_updated) {
     for (unsigned c = 0; c<256; c++) {
       if (char_changed[c]) {
@@ -1643,7 +1650,13 @@ void bx_win32_gui_c::text_update(Bit8u *old_text, Bit8u *new_text,
       if (forceUpdate || (old_text[0] != new_text[0])
           || (old_text[1] != new_text[1])) {
         cChar = new_text[0];
-        cAttr = new_text[1];
+        if (blink_mode) {
+          cAttr = new_text[1] & 0x7F;
+          if (!blink_state && (new_text[1] & 0x80))
+            cAttr = (cAttr & 0x70) | (cAttr >> 4);
+        } else {
+          cAttr = new_text[1];
+        }
         DrawBitmap(hdc, vgafont[cChar], xc, yc, cfwidth, cfheight, font_col,
                    font_row, SRCCOPY, cAttr);
         if (offset == curs) {
@@ -1703,7 +1716,13 @@ void bx_win32_gui_c::text_update(Bit8u *old_text, Bit8u *new_text,
       if (forceUpdate || (old_text[0] != new_text[0])
           || (old_text[1] != new_text[1])) {
         cChar = new_text[0];
-        cAttr = new_text[1];
+        if (blink_mode) {
+          cAttr = new_text[1] & 0x7F;
+          if (!blink_state && (new_text[1] & 0x80))
+            cAttr = (cAttr & 0x70) | (cAttr >> 4);
+        } else {
+          cAttr = new_text[1];
+        }
         DrawChar(hdc, cChar, xc, yc, cAttr, 1, 0);
         if (offset == curs) {
           DrawChar(hdc, cChar, xc, yc, cAttr, tm_info.cs_start, tm_info.cs_end);
@@ -1728,8 +1747,7 @@ void bx_win32_gui_c::text_update(Bit8u *old_text, Bit8u *new_text,
   LeaveCriticalSection(&stInfo.drawCS);
 }
 
-  int
-bx_win32_gui_c::get_clipboard_text(Bit8u **bytes, Bit32s *nbytes)
+int bx_win32_gui_c::get_clipboard_text(Bit8u **bytes, Bit32s *nbytes)
 {
   if (OpenClipboard(stInfo.simWnd)) {
     HGLOBAL hg = GetClipboardData(CF_TEXT);
@@ -1750,8 +1768,7 @@ bx_win32_gui_c::get_clipboard_text(Bit8u **bytes, Bit32s *nbytes)
   }
 }
 
-  int
-bx_win32_gui_c::set_clipboard_text(char *text_snapshot, Bit32u len)
+int bx_win32_gui_c::set_clipboard_text(char *text_snapshot, Bit32u len)
 {
   if (OpenClipboard(stInfo.simWnd)) {
     HANDLE hMem = GlobalAlloc(GMEM_ZEROINIT, len);
@@ -1806,7 +1823,8 @@ bx_bool bx_win32_gui_c::palette_change(unsigned index, unsigned red,
 // note: origin of tile and of window based on (0,0) being in the upper
 //       left of the window.
 
-void bx_win32_gui_c::graphics_tile_update(Bit8u *tile, unsigned x0, unsigned y0) {
+void bx_win32_gui_c::graphics_tile_update(Bit8u *tile, unsigned x0, unsigned y0)
+{
   HDC hdc;
   HGDIOBJ oldObj;
 
@@ -1815,7 +1833,7 @@ void bx_win32_gui_c::graphics_tile_update(Bit8u *tile, unsigned x0, unsigned y0)
 
   oldObj = SelectObject(MemoryDC, MemoryBitmap);
 
-  StretchDIBits( MemoryDC, x0, y0, x_tilesize, y_tilesize, 0, 0,
+  StretchDIBits(MemoryDC, x0, y0, x_tilesize, y_tilesize, 0, 0,
     x_tilesize, y_tilesize, tile, bitmap_info, DIB_RGB_COLORS, SRCCOPY);
 
   SelectObject(MemoryDC, oldObj);
@@ -1872,7 +1890,7 @@ void bx_win32_gui_c::dimension_update(unsigned x, unsigned y, unsigned fheight,
 #endif
   }
 
-  if ( x==dimension_x && y==dimension_y && bpp==current_bpp)
+  if (x==dimension_x && y==dimension_y && bpp==current_bpp)
     return;
   dimension_x = x;
   dimension_y = y;
@@ -1883,7 +1901,7 @@ void bx_win32_gui_c::dimension_update(unsigned x, unsigned y, unsigned fheight,
     stretched_x *= 2;
     stretch_factor *= 2;
   }
-  
+
   bitmap_info->bmiHeader.biBitCount = bpp;
   if (bpp == 16)
   {
@@ -1936,7 +1954,8 @@ void bx_win32_gui_c::dimension_update(unsigned x, unsigned y, unsigned fheight,
 // ydim: y dimension of bitmap
 
 unsigned bx_win32_gui_c::create_bitmap(const unsigned char *bmap, unsigned xdim,
-				 unsigned ydim) {
+				 unsigned ydim)
+{
   unsigned char *data;
   TBADDBITMAP tbab;
 
@@ -1981,11 +2000,12 @@ unsigned bx_win32_gui_c::create_bitmap(const unsigned char *bmap, unsigned xdim,
 //     the boundaries of this bitmap.
 
 unsigned bx_win32_gui_c::headerbar_bitmap(unsigned bmap_id, unsigned alignment,
-				    void (*f)(void)) {
+				    void (*f)(void))
+{
   unsigned hb_index;
   TBBUTTON tbb[1];
 
-  if ( (bx_headerbar_entries+1) > BX_MAX_HEADERBAR_ENTRIES )
+  if ((bx_headerbar_entries+1) > BX_MAX_HEADERBAR_ENTRIES)
     terminateEmul(EXIT_HEADER_BITMAP_ERROR);
 
   bx_headerbar_entries++;
@@ -2069,7 +2089,8 @@ void bx_win32_gui_c::replace_bitmap(unsigned hbar_id, unsigned bmap_id)
 //
 // Called before bochs terminates, to allow for a graceful
 // exit from the native GUI mechanism.
-void bx_win32_gui_c::exit(void) {
+void bx_win32_gui_c::exit(void)
+{
   printf("# In bx_win32_gui_c::exit(void)!\n");
 
   // kill thread first...
@@ -2082,7 +2103,8 @@ void bx_win32_gui_c::exit(void) {
 }
 
 
-void create_vga_font(void) {
+void create_vga_font(void)
+{
   unsigned char data[64];
 
   // VGA font is 8 or 9 wide and up to 32 high
@@ -2097,7 +2119,8 @@ void create_vga_font(void) {
 }
 
 
-unsigned char reverse_bitorder(unsigned char b) {
+unsigned char reverse_bitorder(unsigned char b)
+{
   unsigned char ret=0;
 
   for (unsigned i=0; i<8; i++) {
@@ -2118,7 +2141,8 @@ COLORREF GetColorRef(unsigned char attr)
 
 
 void DrawBitmap(HDC hdc, HBITMAP hBitmap, int xStart, int yStart, int width,
-                int height, int fcol, int frow, DWORD dwRop, unsigned char cColor) {
+                int height, int fcol, int frow, DWORD dwRop, unsigned char cColor)
+{
   BITMAP bm;
   HDC hdcMem;
   POINT ptSize, ptOrg;
@@ -2141,11 +2165,6 @@ void DrawBitmap(HDC hdc, HBITMAP hBitmap, int xStart, int yStart, int width,
 
   oldObj = SelectObject(MemoryDC, MemoryBitmap);
 
-  // The highest background bit usually means blinking characters. No idea
-  // how to implement that so for now it's just implemented as color.
-  // Note: it is also possible to program the VGA controller to have the
-  // high bit for the foreground color enable blinking characters.
-
   COLORREF crFore = SetTextColor(MemoryDC, GetColorRef((cColor>>4)&0xf));
   COLORREF crBack = SetBkColor(MemoryDC, GetColorRef(cColor&0xf));
   BitBlt(MemoryDC, xStart, yStart, ptSize.x, ptSize.y, hdcMem, ptOrg.x,
@@ -2161,7 +2180,8 @@ void DrawBitmap(HDC hdc, HBITMAP hBitmap, int xStart, int yStart, int width,
 }
 
 
-void updateUpdated(int x1, int y1, int x2, int y2) {
+void updateUpdated(int x1, int y1, int x2, int y2)
+{
   x1*=stretch_factor;
   x2*=stretch_factor;
   if (!updated_area_valid) {
@@ -2241,7 +2261,8 @@ void bx_win32_gui_c::show_ips(Bit32u ips_count)
 #if BX_USE_WINDOWS_FONTS
 
 void DrawChar (HDC hdc, unsigned char c, int xStart, int yStart,
-               unsigned char cColor, int cs_start, int cs_end) {
+               unsigned char cColor, int cs_start, int cs_end)
+{
   HDC hdcMem;
   POINT ptSize, ptOrg;
   HGDIOBJ oldObj;
@@ -2263,11 +2284,6 @@ void DrawChar (HDC hdc, unsigned char c, int xStart, int yStart,
   oldObj = SelectObject(MemoryDC, MemoryBitmap);
   hFontOld=(HFONT)SelectObject(MemoryDC, hFont[FontId]);
 
-  // The highest background bit usually means blinking characters. No idea
-  // how to implement that so for now it's just implemented as color.
-  // Note: it is also possible to program the VGA controller to have the
-  // high bit for the foreground color enable blinking characters.
-
   COLORREF crFore = SetTextColor(MemoryDC, GetColorRef(cColor&0xf));
   COLORREF crBack = SetBkColor(MemoryDC, GetColorRef((cColor>>4)&0xf));
   str[0]=c;
