@@ -69,10 +69,44 @@
 #define RT_USBDEV_DLG 2700
 #define IDUSBLBL1  2711
 #define IDUSBLBL2  2712
-#define IDUSBLBL3  2713
-#define IDUSBLBL4  2714
 #define IDUSBWARN  2715
 #define IDUSBDEV1  2620
-#define IDUSBOPT1  2630
 #define IDUSBDEV2  2640
-#define IDUSBOPT2  2650
+#define DEBUGGER_DLG 3000
+#define DEBUG_MSG  3010
+#define DEBUG_CMD  3020
+#define IDEXEC     3030
+#define IDSTOP     3040
+#define IDSHOWCPU  3050
+#define IDCPULBL1  3061
+#define IDCPULBL2  3062
+#define IDCPULBL3  3063
+#define IDCPULBL4  3064
+#define IDCPULBL5  3065
+#define IDCPULBL6  3066
+#define IDCPULBL7  3067
+#define IDCPULBL8  3068
+#define IDCPULBL9  3069
+#define IDCPULBL10 3070
+#define IDCPULBL11 3071
+#define IDCPULBL12 3072
+#define IDCPULBL13 3073
+#define IDCPULBL14 3074
+#define IDCPULBL15 3075
+#define IDCPULBL16 3076
+#define IDCPUVAL1  3081
+#define IDCPUVAL2  3082
+#define IDCPUVAL3  3083
+#define IDCPUVAL4  3084
+#define IDCPUVAL5  3085
+#define IDCPUVAL6  3086
+#define IDCPUVAL7  3087
+#define IDCPUVAL8  3088
+#define IDCPUVAL9  3089
+#define IDCPUVAL10 3090
+#define IDCPUVAL11 3091
+#define IDCPUVAL12 3092
+#define IDCPUVAL13 3093
+#define IDCPUVAL14 3094
+#define IDCPUVAL15 3095
+#define IDCPUVAL16 3096
