@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////
-// $Id: wx.cc,v 1.90 2007/09/28 19:51:58 sshwarts Exp $
+// $Id: wx.cc,v 1.94 2008/02/15 22:05:40 sshwarts Exp $
 /////////////////////////////////////////////////////////////////
 //
 // wxWidgets VGA display for Bochs.  wx.cc implements a custom
@@ -16,23 +16,19 @@
 //   that wxmain.cc does NOT include bochs.h.  All interactions
 //   between the CI and the simulator are through the siminterface
 //   object.
-// - wx.cc implements a VGA display screen using wxWidgets.  It is 
+// - wx.cc implements a VGA display screen using wxWidgets.  It is
 //   is the wxWidgets equivalent of x.cc, win32.cc, macos.cc, etc.
 //   wx.cc includes bochs.h and has access to all Bochs devices.
 //   The VGA panel accepts only paint, key, and mouse events.  As it
-//   receives events, it builds BxEvents and places them into a 
+//   receives events, it builds BxEvents and places them into a
 //   thread-safe BxEvent queue.  The simulation thread periodically
 //   processes events from the BxEvent queue (bx_wx_gui_c::handle_events)
 //   and notifies the appropriate emulated I/O device.
 //
 /////////////////////////////////////////////////////////////////
 
-//////////////////////////////////////////////////////////////
-// includes
-//////////////////////////////////////////////////////////////
-
 // Define BX_PLUGGABLE in files that can be compiled into plugins.  For
-// platforms that require a special tag on exported symbols, BX_PLUGGABLE 
+// platforms that require a special tag on exported symbols, BX_PLUGGABLE
 // is used to know when we are exporting symbols and when we are importing.
 #define BX_PLUGGABLE
 
@@ -138,7 +134,7 @@ END_EVENT_TABLE()
 MyPanel::MyPanel(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style, const wxString& name)
   : wxPanel (parent, id, pos, size, style, name)
 {
-  wxLogDebug (wxT ("MyPanel constructor")); 
+  wxLogDebug (wxT ("MyPanel constructor"));
   refreshTimer.SetOwner (this);
   refreshTimer.Start (100);
   needRefresh = true;
@@ -239,19 +235,19 @@ void MyPanel::OnMouse(wxMouseEvent& event)
     }
   )
 
-  if (event.MiddleDown () && event.ControlDown ()) {
+  if (event.MiddleDown() && event.ControlDown()) {
     ToggleMouse (false);
     return;
   }
 
-  if (!mouse_captured) 
+  if (!mouse_captured)
     return;  // mouse disabled, ignore the event
 
   // process buttons and motion together
   Bit32u buttons;
-  buttons  = event.LeftIsDown ()  ? 1 : 0;
-  buttons |= event.RightIsDown () ? 2 : 0;
-  buttons |= event.MiddleIsDown () ? 4 : 0;
+  buttons  = event.LeftIsDown()  ? 1 : 0;
+  buttons |= event.RightIsDown() ? 2 : 0;
+  buttons |= event.MiddleIsDown() ? 4 : 0;
   if (x==mouseSavedX && y==mouseSavedY && !event.IsButton ()) {
     // nothing happened.  This could have been generated by the WarpPointer.
     return;
@@ -282,8 +278,7 @@ void MyPanel::OnMouse(wxMouseEvent& event)
   // will move the cursor to (mouseSavedX, mouseSavedY).
 }
 
-void 
-MyPanel::MyRefresh ()
+void MyPanel::MyRefresh ()
 {
   IFDBG_VGA (wxLogDebug (wxT ("set needRefresh=true")));
   needRefresh = true;
@@ -310,10 +305,8 @@ void MyPanel::OnKeyUp(wxKeyEvent& event)
   }
 }
 
-
 /// copied right out of gui/x.cc
-static char 
-wxAsciiKey[0x5f] = {
+static char wxAsciiKey[0x5f] = {
   //  !"#$%&'
   BX_KEY_SPACE,
   BX_KEY_1,
@@ -437,8 +430,7 @@ wxAsciiKey[0x5f] = {
 };
 
 // copied from gui/win32.cc
-Bit32u
-wxMSW_to_bx_key[0x59] = {
+Bit32u wxMSW_to_bx_key[0x59] = {
   /* 0x00 - 0x0f */
   0,
   BX_KEY_ESC,
@@ -544,8 +536,7 @@ wxMSW_to_bx_key[0x59] = {
 #endif
 
 // MS Windows specific key mapping, which uses wxKeyEvent::m_rawCode & 2.
-bx_bool
-MyPanel::fillBxKeyEvent_MSW (wxKeyEvent& wxev, BxKeyEvent& bxev, bx_bool release)
+bx_bool MyPanel::fillBxKeyEvent_MSW (wxKeyEvent& wxev, BxKeyEvent& bxev, bx_bool release)
 {
 #if defined(wxHAS_RAW_KEY_CODES) && defined(__WXMSW__)
   IFDBG_KEY(wxLogDebug (wxT ("fillBxKeyEvent_MSW. key code %d, raw codes %d %d", wxev.m_keyCode, wxev.m_rawCode, wxev.m_rawFlags)));
@@ -572,8 +563,7 @@ MyPanel::fillBxKeyEvent_MSW (wxKeyEvent& wxev, BxKeyEvent& bxev, bx_bool release
 #endif
 
 // GTK specific key mapping, which uses wxKeyEvent::m_rawCode.
-bx_bool
-MyPanel::fillBxKeyEvent_GTK (wxKeyEvent& wxev, BxKeyEvent& bxev, bx_bool release)
+bx_bool MyPanel::fillBxKeyEvent_GTK (wxKeyEvent& wxev, BxKeyEvent& bxev, bx_bool release)
 {
 #if defined (wxHAS_RAW_KEY_CODES) && defined(__WXGTK__)
   IFDBG_KEY(wxLogDebug (wxT ("fillBxKeyEvent_GTK. key code %ld, raw codes %d %d", wxev.m_keyCode, wxev.m_rawCode, wxev.m_rawFlags)));
@@ -729,14 +719,14 @@ MyPanel::fillBxKeyEvent_GTK (wxKeyEvent& wxev, BxKeyEvent& bxev, bx_bool release
 #endif
 
         default:
-          wxLogError(wxT ("fillBxKeyEvent_GTK(): keysym %x unhandled!"), (unsigned) keysym );
+          wxLogError(wxT("fillBxKeyEvent_GTK(): keysym %x unhandled!"), (unsigned) keysym);
           return BX_KEY_UNHANDLED;
     }
   } else {
     /* use mapping */
     BXKeyEntry *entry = bx_keymap.findHostKey (keysym);
     if (!entry) {
-      BX_ERROR(( "fillBxKeyEvent_GTK(): keysym %x unhandled!", (unsigned) keysym ));
+      BX_ERROR(("fillBxKeyEvent_GTK(): keysym %x unhandled!", (unsigned) keysym));
       return BX_KEY_UNHANDLED;
     }
     key_event = entry->baseKey;
@@ -749,8 +739,7 @@ MyPanel::fillBxKeyEvent_GTK (wxKeyEvent& wxev, BxKeyEvent& bxev, bx_bool release
 #endif
 }
 
-bx_bool
-MyPanel::fillBxKeyEvent (wxKeyEvent& wxev, BxKeyEvent& bxev, bx_bool release)
+bx_bool MyPanel::fillBxKeyEvent (wxKeyEvent& wxev, BxKeyEvent& bxev, bx_bool release)
 {
   // Use raw codes if they are available.  Raw codes are a nonstandard addition
   // to the wxWidgets library.  At present, the only way to use the "RAW_CODES"
@@ -771,7 +760,7 @@ MyPanel::fillBxKeyEvent (wxKeyEvent& wxev, BxKeyEvent& bxev, bx_bool release)
   IFDBG_KEY (wxLogDebug (wxT ("fillBxKeyEvent. key code %ld", wxev.m_keyCode)));
   Bit32u key = wxev.m_keyCode;
   Bit32u bx_key;
-  
+
   if(key >= WXK_SPACE && key < WXK_DELETE) {
     bx_key = wxAsciiKey[key - WXK_SPACE];
   } else {
@@ -821,7 +810,7 @@ MyPanel::fillBxKeyEvent (wxKeyEvent& wxev, BxKeyEvent& bxev, bx_bool release)
     case WXK_NUMLOCK:              bx_key = BX_KEY_NUM_LOCK;     break;
     case WXK_SCROLL:               bx_key = BX_KEY_SCRL_LOCK;    break;
     case WXK_DECIMAL:              bx_key = BX_KEY_PERIOD;       break;
-    case WXK_SUBTRACT:             bx_key = BX_KEY_MINUS;        break; 
+    case WXK_SUBTRACT:             bx_key = BX_KEY_MINUS;        break;
     case WXK_ADD:                  bx_key = BX_KEY_EQUALS;       break;
     case WXK_MULTIPLY:             bx_key = BX_KEY_KP_MULTIPLY;  break;
     case WXK_DIVIDE:               bx_key = BX_KEY_KP_DIVIDE;    break;
@@ -873,13 +862,11 @@ MyPanel::fillBxKeyEvent (wxKeyEvent& wxev, BxKeyEvent& bxev, bx_bool release)
   return true;
 }
 
-
 //////////////////////////////////////////////////////////////
 // fill in methods of bx_gui
 //////////////////////////////////////////////////////////////
 
-  void
-bx_wx_gui_c::specific_init(int argc, char **argv, unsigned tilewidth, unsigned tileheight,
+void bx_wx_gui_c::specific_init(int argc, char **argv, unsigned tilewidth, unsigned tileheight,
                      unsigned headerbar_y)
 {
   int b,i,j;
@@ -1051,10 +1038,9 @@ void bx_wx_gui_c::handle_events(void)
   num_events = 0;
 }
 
-void 
-bx_wx_gui_c::statusbar_setitem(int element, bx_bool active)
+void bx_wx_gui_c::statusbar_setitem(int element, bx_bool active)
 {
-#if defined( __WXMSW__)
+#if defined(__WXMSW__)
   char status_text[10];
 #endif
 
@@ -1062,7 +1048,7 @@ bx_wx_gui_c::statusbar_setitem(int element, bx_bool active)
   if (element < 0) {
     for (unsigned i = 0; i < statusitem_count; i++) {
       if (active) {
-#if defined( __WXMSW__)
+#if defined(__WXMSW__)
         status_text[0] = 9;
         strcpy(status_text+1, statusitem_text[i]);
         theFrame->SetStatusText(status_text, i+1);
@@ -1075,7 +1061,7 @@ bx_wx_gui_c::statusbar_setitem(int element, bx_bool active)
     }
   } else if ((unsigned)element < statusitem_count) {
     if (active) {
-#if defined( __WXMSW__)
+#if defined(__WXMSW__)
         status_text[0] = 9;
         strcpy(status_text+1, statusitem_text[element]);
         theFrame->SetStatusText(status_text, element+1);
@@ -1095,19 +1081,16 @@ bx_wx_gui_c::statusbar_setitem(int element, bx_bool active)
 // Called periodically, requesting that the gui code flush all pending
 // screen update requests.
 
-  void
-bx_wx_gui_c::flush(void)
+void bx_wx_gui_c::flush(void)
 {
 }
 
-
 // ::CLEAR_SCREEN()
 //
 // Called to request that the VGA region is cleared.  Don't
 // clear the area that defines the headerbar.
 
-  void
-bx_wx_gui_c::clear_screen(void)
+void bx_wx_gui_c::clear_screen(void)
 {
   IFDBG_VGA(wxLogDebug (wxT ("MyPanel::clear_screen trying to get lock. wxScreen=%p", wxScreen)));
   wxCriticalSectionLocker lock(wxScreen_lock);
@@ -1116,8 +1099,7 @@ bx_wx_gui_c::clear_screen(void)
   thePanel->MyRefresh ();
 }
 
-static void 
-UpdateScreen(unsigned char *newBits, int x, int y, int width, int height) 
+static void UpdateScreen(unsigned char *newBits, int x, int y, int width, int height)
 {
   IFDBG_VGA(wxLogDebug (wxT ("MyPanel::UpdateScreen trying to get lock. wxScreen=%p", wxScreen)));
   wxCriticalSectionLocker lock(wxScreen_lock);
@@ -1147,8 +1129,7 @@ UpdateScreen(unsigned char *newBits, int x, int y, int width, int height)
   }
 }
 
-static void 
-DrawBochsBitmap(int x, int y, int width, int height, char *bmap, char color, int fontx, int fonty, bx_bool gfxchar)
+static void DrawBochsBitmap(int x, int y, int width, int height, char *bmap, char color, int fontx, int fonty, bx_bool gfxchar)
 {
   static unsigned char newBits[9 * 32];
   unsigned char mask;
@@ -1196,11 +1177,10 @@ DrawBochsBitmap(int x, int y, int width, int height, char *bmap, char color, int
 // cursor_y: new y location of cursor
 // tm_info:  this structure contains information for additional
 //           features in text mode (cursor shape, line offset,...)
-// nrows:    number of text rows (unused here)
 
 void bx_wx_gui_c::text_update(Bit8u *old_text, Bit8u *new_text,
                       unsigned long cursor_x, unsigned long cursor_y,
-          bx_vga_tminfo_t tm_info, unsigned nrows)
+          bx_vga_tminfo_t tm_info)
 {
   IFDBG_VGA(wxLogDebug (wxT ("text_update")));
 
@@ -1209,14 +1189,19 @@ void bx_wx_gui_c::text_update(Bit8u *old_text, Bit8u *new_text,
   unsigned int curs, hchars, offset, rows, x, y, xc, yc, yc2, cs_y;
   Bit8u cfwidth, cfheight, cfheight2, font_col, font_row, font_row2;
   Bit8u split_textrow, split_fontrows;
-  bx_bool forceUpdate = 0, gfxchar, split_screen;
+  bx_bool forceUpdate = 0, gfxchar, split_screen, blink_state, blink_mode;
 
-  UNUSED(nrows);
+  // first check if the screen needs to be redrawn completely
+  blink_mode = (tm_info.blink_flags & BX_TEXT_BLINK_MODE) > 0;
+  blink_state = (tm_info.blink_flags & BX_TEXT_BLINK_STATE) > 0;
+  if (blink_mode) {
+    if (tm_info.blink_flags & BX_TEXT_BLINK_TOGGLE)
+      forceUpdate = 1;
+  }
   if(charmap_updated) {
     forceUpdate = 1;
     charmap_updated = 0;
   }
-
   if((tm_info.h_panning != h_panning) || (tm_info.v_panning != v_panning)) {
     forceUpdate = 1;
     h_panning = tm_info.h_panning;
@@ -1227,7 +1212,7 @@ void bx_wx_gui_c::text_update(Bit8u *old_text, Bit8u *new_text,
     line_compare = tm_info.line_compare;
   }
 
-  // first invalidate character at previous and new cursor location
+  // invalidate character at previous and new cursor location
   if((wxCursorY < text_rows) && (wxCursorX < text_cols)) {
     curs = wxCursorY * tm_info.line_offset + wxCursorX * 2;
     old_text[curs] = ~new_text[curs];
@@ -1309,7 +1294,13 @@ void bx_wx_gui_c::text_update(Bit8u *old_text, Bit8u *new_text,
       if(forceUpdate || (old_text[0] != new_text[0])
          || (old_text[1] != new_text[1])) {
         cChar = new_text[0];
-        cAttr = new_text[1];
+        if (blink_mode) {
+          cAttr = new_text[1] & 0x7F;
+          if (!blink_state && (new_text[1] & 0x80))
+            cAttr = (cAttr & 0x70) | (cAttr >> 4);
+        } else {
+          cAttr = new_text[1];
+        }
         gfxchar = tm_info.line_graphics && ((cChar & 0xE0) == 0xC0);
         DrawBochsBitmap(xc, yc, cfwidth, cfheight, (char *)&vga_charmap[cChar<<5],
                         cAttr, font_col, font_row, gfxchar);
@@ -1358,10 +1349,9 @@ void bx_wx_gui_c::text_update(Bit8u *old_text, Bit8u *new_text,
   wxCursorX = cursor_x;
   wxCursorY = cursor_y;
 
-  thePanel->MyRefresh ();
+  thePanel->MyRefresh();
 }
 
-
 // ::PALETTE_CHANGE()
 //
 // Allocate a color in the native GUI, for this color, and put
@@ -1369,8 +1359,7 @@ void bx_wx_gui_c::text_update(Bit8u *old_text, Bit8u *new_text,
 // returns: 0=no screen update needed (color map change has direct effect)
 //          1=screen update needed (redraw using current colormap)
 
-  bx_bool
-bx_wx_gui_c::palette_change(unsigned index, unsigned red, unsigned green, unsigned blue)
+bx_bool bx_wx_gui_c::palette_change(unsigned index, unsigned red, unsigned green, unsigned blue)
 {
   IFDBG_VGA(wxLogDebug (wxT ("palette_change")));
   wxBochsPalette[index].red = red;
@@ -1399,11 +1388,10 @@ void bx_wx_gui_c::graphics_tile_update(Bit8u *tile, unsigned x0, unsigned y0)
 {
   IFDBG_VGA (wxLogDebug (wxT ("graphics_tile_update")));
   UpdateScreen(tile, x0, y0, wxTileX, wxTileY);
-  thePanel->MyRefresh ();
+  thePanel->MyRefresh();
 }
 
-  bx_svga_tileinfo_t *
-bx_wx_gui_c::graphics_tile_info(bx_svga_tileinfo_t *info)
+bx_svga_tileinfo_t *bx_wx_gui_c::graphics_tile_info(bx_svga_tileinfo_t *info)
 {
   if (!info) {
     info = (bx_svga_tileinfo_t *)malloc(sizeof(bx_svga_tileinfo_t));
@@ -1424,10 +1412,9 @@ bx_wx_gui_c::graphics_tile_info(bx_svga_tileinfo_t *info)
   info->is_little_endian = 1;
 
   return info;
-};
+}
 
-  Bit8u *
-bx_wx_gui_c::graphics_tile_get(unsigned x0, unsigned y0,
+Bit8u *bx_wx_gui_c::graphics_tile_get(unsigned x0, unsigned y0,
                             unsigned *w, unsigned *h)
 {
   if (x0+wxTileX > (unsigned)wxScreenX) {
@@ -1447,8 +1434,7 @@ bx_wx_gui_c::graphics_tile_get(unsigned x0, unsigned y0,
   return (Bit8u *)wxScreen + y0 * wxScreenX * 3 + x0 * 3;
 }
 
-  void
-bx_wx_gui_c::graphics_tile_update_in_place(unsigned x0, unsigned y0,
+void bx_wx_gui_c::graphics_tile_update_in_place(unsigned x0, unsigned y0,
                                         unsigned w, unsigned h)
 {
   thePanel->MyRefresh();
@@ -1493,7 +1479,7 @@ void bx_wx_gui_c::dimension_update(unsigned x, unsigned y, unsigned fheight, uns
   wxScreen_lock.Leave ();
   IFDBG_VGA(wxLogDebug (wxT ("MyPanel::dimension_update gave up lock. wxScreen=%p", wxScreen)));
   // Note: give up wxScreen_lock before calling SetClientSize.  I did
-  // this because I was sometimes seeing thread deadlock in win32, apparantly 
+  // this because I was sometimes seeing thread deadlock in win32, apparantly
   // related to wxStreen_lock.  The wxWidgets GUI thread was sitting in OnPaint
   // trying to get the wxScreen_lock, and the simulation thread was stuck in some
   // native win32 function called by SetClientSize (below).  As with many
@@ -1522,8 +1508,7 @@ void bx_wx_gui_c::dimension_update(unsigned x, unsigned y, unsigned fheight, uns
 // xdim: x dimension of bitmap
 // ydim: y dimension of bitmap
 
-  unsigned
-bx_wx_gui_c::create_bitmap(const unsigned char *bmap, unsigned xdim, unsigned ydim)
+unsigned bx_wx_gui_c::create_bitmap(const unsigned char *bmap, unsigned xdim, unsigned ydim)
 {
   UNUSED(bmap);
   UNUSED(xdim);
@@ -1546,8 +1531,7 @@ bx_wx_gui_c::create_bitmap(const unsigned char *bmap, unsigned xdim, unsigned yd
 // f: a 'C' function pointer to callback when the mouse is clicked in
 //     the boundaries of this bitmap.
 
-  unsigned
-bx_wx_gui_c::headerbar_bitmap(unsigned bmap_id, unsigned alignment, void (*f)(void))
+unsigned bx_wx_gui_c::headerbar_bitmap(unsigned bmap_id, unsigned alignment, void (*f)(void))
 {
   UNUSED(bmap_id);
   UNUSED(alignment);
@@ -1555,18 +1539,15 @@ bx_wx_gui_c::headerbar_bitmap(unsigned bmap_id, unsigned alignment, void (*f)(vo
   return(0);
 }
 
-
 // ::SHOW_HEADERBAR()
 //
 // Show (redraw) the current headerbar, which is composed of
 // currently installed bitmaps.
 
-  void
-bx_wx_gui_c::show_headerbar(void)
+void bx_wx_gui_c::show_headerbar(void)
 {
 }
 
-
 // ::REPLACE_BITMAP()
 //
 // Replace the bitmap installed in the headerbar ID slot 'hbar_id',
@@ -1603,9 +1584,7 @@ void bx_wx_gui_c::mouse_enabled_changed_specific(bx_bool val)
   mouse_captured = val;
 }
 
-
-  int
-bx_wx_gui_c::get_clipboard_text(Bit8u **bytes, Bit32s *nbytes)
+int bx_wx_gui_c::get_clipboard_text(Bit8u **bytes, Bit32s *nbytes)
 {
   int ret = 0;
   wxMutexGuiEnter();
@@ -1620,7 +1599,7 @@ bx_wx_gui_c::get_clipboard_text(Bit8u **bytes, Bit32s *nbytes)
       *bytes = buf;
       *nbytes = len;
       ret = 1;
-      // buf will be freed in bx_keyb_c::paste_bytes or 
+      // buf will be freed in bx_keyb_c::paste_bytes or
       // bx_keyb_c::service_paste_buf, using delete [].
     } else {
       BX_ERROR (("paste: could not open wxWidgets clipboard"));
@@ -1631,8 +1610,7 @@ bx_wx_gui_c::get_clipboard_text(Bit8u **bytes, Bit32s *nbytes)
   return ret;
 }
 
-  int
-bx_wx_gui_c::set_clipboard_text(char *text_snapshot, Bit32u len)
+int bx_wx_gui_c::set_clipboard_text(char *text_snapshot, Bit32u len)
 {
   wxMutexGuiEnter();
   int ret = 0;
