@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: instrument.h,v 1.20 2007/12/13 21:53:53 sshwarts Exp $
+// $Id: instrument.h,v 1.27 2008/04/19 10:12:09 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -34,17 +34,22 @@
 #define BX_INSTR_INVD         20
 #define BX_INSTR_WBINVD       21
 
-#define BX_INSTR_IS_CALL  10
-#define BX_INSTR_IS_RET   11
-#define BX_INSTR_IS_IRET  12
-#define BX_INSTR_IS_JMP   13
-#define BX_INSTR_IS_INT   14
-
-#define BX_INSTR_PREFETCH_NTA 00
-#define BX_INSTR_PREFETCH_T0  01
-#define BX_INSTR_PREFETCH_T1  02
-#define BX_INSTR_PREFETCH_T2  03
+// possible types passed to BX_INSTR_FAR_BRANCH()
+#define BX_INSTR_IS_CALL      10
+#define BX_INSTR_IS_RET       11
+#define BX_INSTR_IS_IRET      12
+#define BX_INSTR_IS_JMP       13
+#define BX_INSTR_IS_INT       14
+#define BX_INSTR_IS_SYSCALL   15
+#define BX_INSTR_IS_SYSRET    16
+#define BX_INSTR_IS_SYSENTER  17
+#define BX_INSTR_IS_SYSEXIT   18
 
+// possible types passed to BX_INSTR_PREFETCH_HINT()
+#define BX_INSTR_PREFETCH_NTA 0
+#define BX_INSTR_PREFETCH_T0  1
+#define BX_INSTR_PREFETCH_T1  2
+#define BX_INSTR_PREFETCH_T2  3
 
 
 #if BX_INSTRUMENTATION
@@ -62,8 +67,8 @@ void bx_instr_cnear_branch_not_taken(unsigned cpu);
 void bx_instr_ucnear_branch(unsigned cpu, unsigned what, bx_address new_eip);
 void bx_instr_far_branch(unsigned cpu, unsigned what, Bit16u new_cs, bx_address new_eip);
 
-void bx_instr_opcode(unsigned cpu, Bit8u *opcode, unsigned len, bx_bool is32, bx_bool is64);
-void bx_instr_fetch_decode_completed(unsigned cpu, const bxInstruction_c *i);
+void bx_instr_opcode(unsigned cpu, const Bit8u *opcode, unsigned len, bx_bool is32, bx_bool is64);
+void bx_instr_fetch_decode_completed(unsigned cpu, bxInstruction_c *i);
 
 void bx_instr_prefix(unsigned cpu, Bit8u prefix);
 
@@ -71,21 +76,21 @@ void bx_instr_interrupt(unsigned cpu, unsigned vector);
 void bx_instr_exception(unsigned cpu, unsigned vector);
 void bx_instr_hwinterrupt(unsigned cpu, unsigned vector, Bit16u cs, bx_address eip);
 
-void bx_instr_mem_data(unsigned cpu, bx_address lin, unsigned size, unsigned rw);
+void bx_instr_mem_data_access(unsigned cpu, unsigned seg, bx_address offset, unsigned len, unsigned rw);
 
 /* simulation init, shutdown, reset */
 #  define BX_INSTR_INIT(cpu_id)            bx_instr_init(cpu_id)
-#  define BX_INSTR_SHUTDOWN(cpu_id)
+#  define BX_INSTR_EXIT(cpu_id)
 #  define BX_INSTR_RESET(cpu_id)           bx_instr_reset(cpu_id)
 #  define BX_INSTR_HLT(cpu_id)
 #  define BX_INSTR_MWAIT(cpu_id, addr, len, flags)
 #  define BX_INSTR_NEW_INSTRUCTION(cpu_id) bx_instr_new_instruction(cpu_id)
 
 /* called from command line debugger */
-#  define BX_INSTR_DEBUG_PROMPT()        
-#  define BX_INSTR_START()               
-#  define BX_INSTR_STOP()                
-#  define BX_INSTR_PRINT()               
+#  define BX_INSTR_DEBUG_PROMPT()
+#  define BX_INSTR_START()
+#  define BX_INSTR_STOP()
+#  define BX_INSTR_PRINT()
 
 /* branch resoultion */
 #  define BX_INSTR_CNEAR_BRANCH_TAKEN(cpu_id, new_eip)       bx_instr_cnear_branch_taken(cpu_id, new_eip)
@@ -98,7 +103,7 @@ void bx_instr_mem_data(unsigned cpu, bx_address lin, unsigned size, unsigned rw)
                        bx_instr_opcode(cpu_id, opcode, len, is32, is64)
 #  define BX_INSTR_FETCH_DECODE_COMPLETED(cpu_id, i) \
                        bx_instr_fetch_decode_completed(cpu_id, i)
-     
+
 /* prefix byte decoded */
 #  define BX_INSTR_PREFIX(cpu_id, prefix)  bx_instr_prefix(cpu_id, prefix)
 
@@ -108,6 +113,7 @@ void bx_instr_mem_data(unsigned cpu, bx_address lin, unsigned size, unsigned rw)
 #  define BX_INSTR_HWINTERRUPT(cpu_id, vector, cs, eip) bx_instr_hwinterrupt(cpu_id, vector, cs, eip)
 
 /* TLB/CACHE control instruction executed */
+#  define BX_INSTR_CLFLUSH(cpu_id, laddr, paddr)
 #  define BX_INSTR_CACHE_CNTRL(cpu_id, what)
 #  define BX_INSTR_TLB_CNTRL(cpu_id, what, new_cr3)
 #  define BX_INSTR_PREFETCH_HINT(cpu_id, what, seg, offset)
@@ -121,8 +127,8 @@ void bx_instr_mem_data(unsigned cpu, bx_address lin, unsigned size, unsigned rw)
 #  define BX_INSTR_LIN_ACCESS(cpu_id, lin, phy, len, rw)
 
 /* memory access */
-#  define BX_INSTR_MEM_CODE(cpu_id, linear, size)
-#  define BX_INSTR_MEM_DATA(cpu_id, linear, size, rw)  bx_instr_mem_data(cpu_id, linear, size, rw)
+#  define BX_INSTR_MEM_DATA_ACCESS(cpu_id, seg, offset, len, rw) \
+                bx_instr_mem_data_access(cpu_id, seg, offset, len, rw)
 
 /* called from memory object */
 #  define BX_INSTR_PHY_WRITE(cpu_id, addr, len)
@@ -137,11 +143,11 @@ void bx_instr_mem_data(unsigned cpu, bx_address lin, unsigned size, unsigned rw)
 /* wrmsr callback */
 #  define BX_INSTR_WRMSR(cpu_id, addr, value)
 
-#else   
+#else
 
 /* simulation init, shutdown, reset */
 #  define BX_INSTR_INIT(cpu_id)
-#  define BX_INSTR_SHUTDOWN(cpu_id)
+#  define BX_INSTR_EXIT(cpu_id)
 #  define BX_INSTR_RESET(cpu_id)
 #  define BX_INSTR_HLT(cpu_id)
 #  define BX_INSTR_MWAIT(cpu_id, addr, len, flags)
@@ -162,7 +168,7 @@ void bx_instr_mem_data(unsigned cpu, bx_address lin, unsigned size, unsigned rw)
 /* decoding completed */
 #  define BX_INSTR_OPCODE(cpu_id, opcode, len, is32, is64)
 #  define BX_INSTR_FETCH_DECODE_COMPLETED(cpu_id, i)
-     
+
 /* prefix byte decoded */
 #  define BX_INSTR_PREFIX(cpu_id, prefix)
 
@@ -172,6 +178,7 @@ void bx_instr_mem_data(unsigned cpu, bx_address lin, unsigned size, unsigned rw)
 #  define BX_INSTR_HWINTERRUPT(cpu_id, vector, cs, eip)
 
 /* TLB/CACHE control instruction executed */
+#  define BX_INSTR_CLFLUSH(cpu_id, laddr, paddr)
 #  define BX_INSTR_CACHE_CNTRL(cpu_id, what)
 #  define BX_INSTR_TLB_CNTRL(cpu_id, what, new_cr3)
 #  define BX_INSTR_PREFETCH_HINT(cpu_id, what, seg, offset)
@@ -185,8 +192,7 @@ void bx_instr_mem_data(unsigned cpu, bx_address lin, unsigned size, unsigned rw)
 #  define BX_INSTR_LIN_ACCESS(cpu_id, lin, phy, len, rw)
 
 /* memory access */
-#  define BX_INSTR_MEM_CODE(cpu_id, linear, size)      
-#  define BX_INSTR_MEM_DATA(cpu_id, linear, size, rw)
+#  define BX_INSTR_MEM_DATA_ACCESS(cpu_id, seg, offset, len, rw)
 
 /* called from memory object */
 #  define BX_INSTR_PHY_WRITE(cpu_id, addr, len)
@@ -201,4 +207,4 @@ void bx_instr_mem_data(unsigned cpu, bx_address lin, unsigned size, unsigned rw)
 /* wrmsr callback */
 #  define BX_INSTR_WRMSR(cpu_id, addr, value)
 
-#endif  
+#endif
