@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: instrument.cc,v 1.14 2006/06/17 12:09:55 sshwarts Exp $
+// $Id: instrument.cc,v 1.18 2008/04/19 10:12:09 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -61,26 +61,26 @@ void bxInstrumentation::bx_instr_new_instruction()
     char disasm_tbuf[512];	// buffer for instruction disassembly
     unsigned length = opcode_size, n;
     bx_disassembler.disasm(is32, is64, 0, 0, opcode, disasm_tbuf);
-    if(length != 0)	
+    if(length != 0)
     {
       fprintf(stderr, "----------------------------------------------------------\n");
       fprintf(stderr, "CPU: %d: %s\n", cpu_id, disasm_tbuf);
       fprintf(stderr, "LEN: %d\tPREFIXES: %d\tBYTES: ", length, nprefixes);
       for(n=0;n<length;n++) fprintf(stderr, "%02x", opcode[n]);
-      if(is_branch) 
+      if(is_branch)
       {
         fprintf(stderr, "\tBRANCH ");
 
-        if(is_taken) 
+        if(is_taken)
           fprintf(stderr, "TARGET " FMT_ADDRX " (TAKEN)", target_linear);
         else
           fprintf(stderr, "(NOT TAKEN)");
-      }   
+      }
       fprintf(stderr, "\n");
       for(n=0;n < num_data_accesses;n++)
       {
         fprintf(stderr, "MEM ACCESS[%u]: " FMT_ADDRX " (linear) 0x%08x (physical) %s SIZE: %d\n", n,
-                      data_access[n].laddr, 
+                      data_access[n].laddr,
                       data_access[n].paddr,
                       data_access[n].op == BX_READ ? "RD":"WR",
                       data_access[n].size);
@@ -98,14 +98,14 @@ void bxInstrumentation::branch_taken(bx_address new_eip)
   if (!active || !valid) return;
 
   // find linear address
-  bx_address laddr = BX_CPU(cpu_id)->get_segment_base(BX_SEG_REG_CS) + new_eip;
+  bx_address laddr = BX_CPU(cpu_id)->get_laddr(BX_SEG_REG_CS, new_eip);
 
   is_branch = 1;
   is_taken = 1;
   target_linear = laddr;
 }
 
-void bxInstrumentation::bx_instr_cnear_branch_taken(bx_address new_eip) 
+void bxInstrumentation::bx_instr_cnear_branch_taken(bx_address new_eip)
 {
   branch_taken(new_eip);
 }
@@ -128,23 +128,23 @@ void bxInstrumentation::bx_instr_far_branch(unsigned what, Bit16u new_cs, bx_add
   branch_taken(new_eip);
 }
 
-void bxInstrumentation::bx_instr_opcode(Bit8u *opcode_bytes, unsigned len, bx_bool is32, bx_bool is64)
+void bxInstrumentation::bx_instr_opcode(const Bit8u *opcode_bytes, unsigned len, bx_bool is32, bx_bool is64)
 {
   if (!active) return;
 
-  for(unsigned i=0;i<len;i++) 
+  for(unsigned i=0;i<len;i++)
   {
     opcode[i] = opcode_bytes[i];
   }
-  
+
   is32 = is32;
   is64 = is64;
   opcode_size = len;
 }
 
-void bxInstrumentation::bx_instr_fetch_decode_completed(const bxInstruction_c *i)
+void bxInstrumentation::bx_instr_fetch_decode_completed(bxInstruction_c *i)
 {
-  if(active) valid = 1; 
+  if(active) valid = 1;
 }
 
 void bxInstrumentation::bx_instr_prefix(Bit8u prefix)
@@ -176,23 +176,24 @@ void bxInstrumentation::bx_instr_hwinterrupt(unsigned vector, Bit16u cs, bx_addr
   }
 }
 
-void bxInstrumentation::bx_instr_mem_data(bx_address lin, unsigned size, unsigned rw)
+void bxInstrumentation::bx_instr_mem_data_access(unsigned seg, bx_address offset, unsigned len, unsigned rw)
 {
   bx_phy_address phy;
 
   if(!active || !valid) return;
 
-  if (num_data_accesses >= MAX_DATA_ACCESSES) 
+  if (num_data_accesses >= MAX_DATA_ACCESSES)
   {
     return;
   }
 
+  bx_address lin = BX_CPU(cpu_id)->get_laddr(seg, offset);
   bx_bool page_valid = BX_CPU(cpu_id)->dbg_xlate_linear2phy(lin, &phy);
   phy = A20ADDR(phy);
 
   // If linear translation doesn't exist, a paging exception will occur.
   // Invalidate physical address data for now.
-  if (!page_valid) 
+  if (!page_valid)
   {
     phy = 0;
   }
@@ -200,6 +201,6 @@ void bxInstrumentation::bx_instr_mem_data(bx_address lin, unsigned size, unsigne
   data_access[num_data_accesses].laddr = lin;
   data_access[num_data_accesses].paddr = phy;
   data_access[num_data_accesses].op    = rw;
-  data_access[num_data_accesses].size  = size;
+  data_access[num_data_accesses].size  = len;
   num_data_accesses++;
 }
