@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: instrument.cc,v 1.20 2007/12/13 21:53:55 sshwarts Exp $
+// $Id: instrument.cc,v 1.26 2008/04/19 10:12:09 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -30,7 +30,7 @@
 #if BX_INSTRUMENTATION
 
 void bx_instr_init(unsigned cpu) {}
-void bx_instr_shutdown(unsigned cpu) {}
+void bx_instr_exit(unsigned cpu) {}
 void bx_instr_reset(unsigned cpu) {}
 void bx_instr_hlt(unsigned cpu) {}
 void bx_instr_mwait(unsigned cpu, bx_phy_address addr, unsigned len, Bit32u flags) {}
@@ -46,8 +46,8 @@ void bx_instr_cnear_branch_not_taken(unsigned cpu) {}
 void bx_instr_ucnear_branch(unsigned cpu, unsigned what, bx_address new_eip) {}
 void bx_instr_far_branch(unsigned cpu, unsigned what, Bit16u new_cs, bx_address new_eip) {}
 
-void bx_instr_opcode(unsigned cpu, Bit8u *opcode, unsigned len, bx_bool is32, bx_bool is64) {}
-void bx_instr_fetch_decode_completed(unsigned cpu, const bxInstruction_c *i) {}
+void bx_instr_opcode(unsigned cpu, const Bit8u *opcode, unsigned len, bx_bool is32, bx_bool is64) {}
+void bx_instr_fetch_decode_completed(unsigned cpu, bxInstruction_c *i) {}
 
 void bx_instr_prefix(unsigned cpu, Bit8u prefix) {}
 
@@ -56,21 +56,20 @@ void bx_instr_exception(unsigned cpu, unsigned vector) {}
 void bx_instr_hwinterrupt(unsigned cpu, unsigned vector, Bit16u cs, bx_address eip) {}
 
 void bx_instr_tlb_cntrl(unsigned cpu, unsigned what, bx_phy_address new_cr3) {}
+void bx_instr_clflush(unsigned cpu, bx_address laddr, bx_phy_address paddr) {}
 void bx_instr_cache_cntrl(unsigned cpu, unsigned what) {}
 void bx_instr_prefetch_hint(unsigned cpu, unsigned what, unsigned seg, bx_address offset) {}
 
-void bx_instr_before_execution(unsigned cpu, const bxInstruction_c *i) {}
-void bx_instr_after_execution(unsigned cpu, const bxInstruction_c *i) {}
-void bx_instr_repeat_iteration(unsigned cpu, const bxInstruction_c *i) {}
+void bx_instr_before_execution(unsigned cpu, bxInstruction_c *i) {}
+void bx_instr_after_execution(unsigned cpu, bxInstruction_c *i) {}
+void bx_instr_repeat_iteration(unsigned cpu, bxInstruction_c *i) {}
 
 void bx_instr_inp(Bit16u addr, unsigned len) {}
 void bx_instr_outp(Bit16u addr, unsigned len) {}
 void bx_instr_inp2(Bit16u addr, unsigned len, unsigned val) {}
 void bx_instr_outp2(Bit16u addr, unsigned len, unsigned val) {}
 
-void bx_instr_mem_code(unsigned cpu, bx_address linear, unsigned len) {}
-void bx_instr_mem_data(unsigned cpu, bx_address linear, unsigned len, unsigned rw) {}
-
+void bx_instr_mem_data_access(unsigned cpu, unsigned seg, bx_address offset, unsigned len, unsigned rw) {}
 void bx_instr_lin_access(unsigned cpu, bx_address lin, bx_address phy, unsigned len, unsigned rw) {}
 
 void bx_instr_phy_write(unsigned cpu, bx_address addr, unsigned len) {}
