@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: instrument.h,v 1.24 2007/12/13 21:53:55 sshwarts Exp $
+// $Id: instrument.h,v 1.33 2008/04/19 11:08:06 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -34,12 +34,18 @@
 #define BX_INSTR_INVD         20
 #define BX_INSTR_WBINVD       21
 
-#define BX_INSTR_IS_CALL  10
-#define BX_INSTR_IS_RET   11
-#define BX_INSTR_IS_IRET  12
-#define BX_INSTR_IS_JMP   13
-#define BX_INSTR_IS_INT   14
+// possible types passed to BX_INSTR_FAR_BRANCH()
+#define BX_INSTR_IS_CALL      10
+#define BX_INSTR_IS_RET       11
+#define BX_INSTR_IS_IRET      12
+#define BX_INSTR_IS_JMP       13
+#define BX_INSTR_IS_INT       14
+#define BX_INSTR_IS_SYSCALL   15
+#define BX_INSTR_IS_SYSRET    16
+#define BX_INSTR_IS_SYSENTER  17
+#define BX_INSTR_IS_SYSEXIT   18
 
+// possible types passed to BX_INSTR_PREFETCH_HINT()
 #define BX_INSTR_PREFETCH_NTA 0
 #define BX_INSTR_PREFETCH_T0  1
 #define BX_INSTR_PREFETCH_T1  2
@@ -53,7 +59,7 @@ class bxInstruction_c;
 // called from the CPU core
 
 void bx_instr_init(unsigned cpu);
-void bx_instr_shutdown(unsigned cpu);
+void bx_instr_exit(unsigned cpu);
 void bx_instr_reset(unsigned cpu);
 void bx_instr_hlt(unsigned cpu);
 void bx_instr_mwait(unsigned cpu, bx_phy_address addr, unsigned len, Bit32u flags);
@@ -69,8 +75,8 @@ void bx_instr_cnear_branch_not_taken(unsigned cpu);
 void bx_instr_ucnear_branch(unsigned cpu, unsigned what, bx_address new_eip);
 void bx_instr_far_branch(unsigned cpu, unsigned what, Bit16u new_cs, bx_address new_eip);
 
-void bx_instr_opcode(unsigned cpu, Bit8u *opcode, unsigned len, bx_bool is32, bx_bool is64);
-void bx_instr_fetch_decode_completed(unsigned cpu, const bxInstruction_c *i);
+void bx_instr_opcode(unsigned cpu, const Bit8u *opcode, unsigned len, bx_bool is32, bx_bool is64);
+void bx_instr_fetch_decode_completed(unsigned cpu, bxInstruction_c *i);
 
 void bx_instr_prefix(unsigned cpu, Bit8u prefix);
 
@@ -82,18 +88,16 @@ void bx_instr_tlb_cntrl(unsigned cpu, unsigned what, bx_phy_address new_cr3);
 void bx_instr_cache_cntrl(unsigned cpu, unsigned what);
 void bx_instr_prefetch_hint(unsigned cpu, unsigned what, unsigned seg, bx_address offset);
 
-void bx_instr_before_execution(unsigned cpu, const bxInstruction_c *i);
-void bx_instr_after_execution(unsigned cpu, const bxInstruction_c *i);
-void bx_instr_repeat_iteration(unsigned cpu, const bxInstruction_c *i);
+void bx_instr_before_execution(unsigned cpu, bxInstruction_c *i);
+void bx_instr_after_execution(unsigned cpu, bxInstruction_c *i);
+void bx_instr_repeat_iteration(unsigned cpu, bxInstruction_c *i);
 
 void bx_instr_inp(Bit16u addr, unsigned len);
 void bx_instr_outp(Bit16u addr, unsigned len);
 void bx_instr_inp2(Bit16u addr, unsigned len, unsigned val);
 void bx_instr_outp2(Bit16u addr, unsigned len, unsigned val);
 
-void bx_instr_mem_code(unsigned cpu, bx_address linear, unsigned size);
-void bx_instr_mem_data(unsigned cpu, bx_address linear, unsigned size, unsigned rw);
-
+void bx_instr_mem_data_access(unsigned cpu, unsigned seg, bx_address offset, unsigned len, unsigned rw);
 void bx_instr_lin_access(unsigned cpu, bx_address lin, bx_address phy, unsigned len, unsigned rw);
 
 void bx_instr_phy_write(unsigned cpu, bx_address addr, unsigned len);
@@ -103,12 +107,12 @@ void bx_instr_wrmsr(unsigned cpu, unsigned addr, Bit64u value);
 
 /* simulation init, shutdown, reset */
 #  define BX_INSTR_INIT(cpu_id)            bx_instr_init(cpu_id)
-#  define BX_INSTR_SHUTDOWN(cpu_id)        bx_instr_shutdown(cpu_id)
+#  define BX_INSTR_EXIT(cpu_id)            bx_instr_exit(cpu_id)
 #  define BX_INSTR_RESET(cpu_id)           bx_instr_reset(cpu_id)
 #  define BX_INSTR_HLT(cpu_id)             bx_instr_hlt(cpu_id)
 
 #  define BX_INSTR_MWAIT(cpu_id, addr, len, flags) \
-                       bx_instr_hlt(cpu_id, addr, len, flags)
+                       bx_instr_mwait(cpu_id, addr, len, flags)
 
 #  define BX_INSTR_NEW_INSTRUCTION(cpu_id) bx_instr_new_instruction(cpu_id)
 
@@ -129,7 +133,7 @@ void bx_instr_wrmsr(unsigned cpu, unsigned addr, Bit64u value);
                        bx_instr_opcode(cpu_id, opcode, len, is32, is64)
 #  define BX_INSTR_FETCH_DECODE_COMPLETED(cpu_id, i) \
                        bx_instr_fetch_decode_completed(cpu_id, i)
-     
+
 /* prefix byte decoded */
 #  define BX_INSTR_PREFIX(cpu_id, prefix)  bx_instr_prefix(cpu_id, prefix)
 
@@ -139,26 +143,26 @@ void bx_instr_wrmsr(unsigned cpu, unsigned addr, Bit64u value);
 #  define BX_INSTR_HWINTERRUPT(cpu_id, vector, cs, eip) bx_instr_hwinterrupt(cpu_id, vector, cs, eip)
 
 /* TLB/CACHE control instruction executed */
-#  define BX_INSTR_CACHE_CNTRL(cpu_id, what)            bx_instr_cache_cntrl(cpu_id, what)
-#  define BX_INSTR_TLB_CNTRL(cpu_id, what, new_cr3)     bx_instr_tlb_cntrl(cpu_id, what, new_cr3)
+#  define BX_INSTR_CLFLUSH(cpu_id, laddr, paddr)    bx_instr_clflush(cpu_id, laddr, paddr)
+#  define BX_INSTR_CACHE_CNTRL(cpu_id, what)        bx_instr_cache_cntrl(cpu_id, what)
+#  define BX_INSTR_TLB_CNTRL(cpu_id, what, new_cr3) bx_instr_tlb_cntrl(cpu_id, what, new_cr3)
 #  define BX_INSTR_PREFETCH_HINT(cpu_id, what, seg, offset) \
                        bx_instr_prefetch_hint(cpu_id, what, seg, offset)
 
 /* execution */
-#  define BX_INSTR_BEFORE_EXECUTION(cpu_id, i)          bx_instr_before_execution(cpu_id, i)
-#  define BX_INSTR_AFTER_EXECUTION(cpu_id, i)           bx_instr_after_execution(cpu_id, i)
-#  define BX_INSTR_REPEAT_ITERATION(cpu_id, i)          bx_instr_repeat_iteration(cpu_id, i)
+#  define BX_INSTR_BEFORE_EXECUTION(cpu_id, i)  bx_instr_before_execution(cpu_id, i)
+#  define BX_INSTR_AFTER_EXECUTION(cpu_id, i)   bx_instr_after_execution(cpu_id, i)
+#  define BX_INSTR_REPEAT_ITERATION(cpu_id, i)  bx_instr_repeat_iteration(cpu_id, i)
 
 /* memory access */
 #  define BX_INSTR_LIN_ACCESS(cpu_id, lin, phy, len, rw)  bx_instr_lin_access(cpu_id, lin, phy, len, rw)
 
 /* memory access */
-#  define BX_INSTR_MEM_CODE(cpu_id, linear, size)       bx_instr_mem_code(cpu_id, linear, size)
-#  define BX_INSTR_MEM_DATA(cpu_id, linear, size, rw)   bx_instr_mem_data(cpu_id, linear, size, rw)
+#  define BX_INSTR_MEM_DATA_ACCESS(cpu_id, seg, offset, len, rw) bx_instr_mem_data_access(cpu_id, seg, offset, len, rw)
 
 /* called from memory object */
-#  define BX_INSTR_PHY_WRITE(cpu_id, addr, len)         bx_instr_phy_write(cpu_id, addr, len)
-#  define BX_INSTR_PHY_READ(cpu_id, addr, len)          bx_instr_phy_read(cpu_id, addr, len)
+#  define BX_INSTR_PHY_WRITE(cpu_id, addr, len) bx_instr_phy_write(cpu_id, addr, len)
+#  define BX_INSTR_PHY_READ(cpu_id, addr, len)  bx_instr_phy_read(cpu_id, addr, len)
 
 /* feedback from device units */
 #  define BX_INSTR_INP(addr, len)               bx_instr_inp(addr, len)
@@ -167,13 +171,13 @@ void bx_instr_wrmsr(unsigned cpu, unsigned addr, Bit64u value);
 #  define BX_INSTR_OUTP2(addr, len, val)        bx_instr_outp2(addr, len, val)
 
 /* wrmsr callback */
-#  define BX_INSTR_WRMSR(cpu_id, addr, value)           bx_instr_wrmsr(cpu_id, addr, value)
+#  define BX_INSTR_WRMSR(cpu_id, addr, value)   bx_instr_wrmsr(cpu_id, addr, value)
 
-#else   
+#else
 
 /* simulation init, shutdown, reset */
 #  define BX_INSTR_INIT(cpu_id)
-#  define BX_INSTR_SHUTDOWN(cpu_id)
+#  define BX_INSTR_EXIT(cpu_id)
 #  define BX_INSTR_RESET(cpu_id)
 #  define BX_INSTR_HLT(cpu_id)
 #  define BX_INSTR_MWAIT(cpu_id, addr, len, flags)
@@ -194,7 +198,7 @@ void bx_instr_wrmsr(unsigned cpu, unsigned addr, Bit64u value);
 /* decoding completed */
 #  define BX_INSTR_OPCODE(cpu_id, opcode, len, is32, is64)
 #  define BX_INSTR_FETCH_DECODE_COMPLETED(cpu_id, i)
-     
+
 /* prefix byte decoded */
 #  define BX_INSTR_PREFIX(cpu_id, prefix)
 
@@ -204,6 +208,7 @@ void bx_instr_wrmsr(unsigned cpu, unsigned addr, Bit64u value);
 #  define BX_INSTR_HWINTERRUPT(cpu_id, vector, cs, eip)
 
 /* TLB/CACHE control instruction executed */
+#  define BX_INSTR_CLFLUSH(cpu_id, laddr, paddr)
 #  define BX_INSTR_CACHE_CNTRL(cpu_id, what)
 #  define BX_INSTR_TLB_CNTRL(cpu_id, what, new_cr3)
 #  define BX_INSTR_PREFETCH_HINT(cpu_id, what, seg, offset)
@@ -217,8 +222,7 @@ void bx_instr_wrmsr(unsigned cpu, unsigned addr, Bit64u value);
 #  define BX_INSTR_LIN_ACCESS(cpu_id, lin, phy, len, rw)
 
 /* memory access */
-#  define BX_INSTR_MEM_CODE(cpu_id, linear, size)
-#  define BX_INSTR_MEM_DATA(cpu_id, linear, size, rw)
+#  define BX_INSTR_MEM_DATA_ACCESS(cpu_id, seg, offset, len, rw)
 
 /* called from memory object */
 #  define BX_INSTR_PHY_WRITE(cpu_id, addr, len)
@@ -233,4 +237,4 @@ void bx_instr_wrmsr(unsigned cpu, unsigned addr, Bit64u value);
 /* wrmsr callback */
 #  define BX_INSTR_WRMSR(cpu_id, addr, value)
 
-#endif  
+#endif
