@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: pci.h,v 1.21 2005/11/15 17:19:28 vruppert Exp $
+// $Id: pci.h,v 1.27 2007/09/28 19:52:03 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -24,14 +24,13 @@
 //  License along with this library; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 
+#ifndef BX_IODEV_PCI_BRIDGE_H
+#define BX_IODEV_PCI_BRIDGE_H
 
 #define BX_MAX_PCI_DEVICES 20
 
 #define BX_PCI_DEVICE(device, function) ((device)<<3 | (function))
 
-typedef Bit32u (*bx_pci_read_handler_t)(void *, Bit8u, unsigned);
-typedef void   (*bx_pci_write_handler_t)(void *, Bit8u, Bit32u, unsigned);
-
 #if BX_USE_PCI_SMF
 #  define BX_PCI_SMF  static
 #  define BX_PCI_THIS thePciBridge->
@@ -40,7 +39,6 @@ typedef void   (*bx_pci_write_handler_t)(void *, Bit8u, Bit32u, unsigned);
 #  define BX_PCI_THIS this->
 #endif
 
-
 #define BX_PCI_INTA 1
 #define BX_PCI_INTB 2
 #define BX_PCI_INTC 3
@@ -50,20 +48,19 @@ typedef struct {
   Bit32u confAddr;
   Bit32u confData;
   Bit8u  pci_conf[256];
-  } bx_def440fx_t;
+} bx_def440fx_t;
 
+class bx_pci_device_stub_c;
 
-
-class bx_pci_c : public bx_pci_stub_c {
-
+class bx_pci_bridge_c : public bx_pci_bridge_stub_c {
 public:
-  bx_pci_c(void);
-  ~bx_pci_c(void);
-  virtual void   init(void);
-  virtual void   reset(unsigned type);
-  virtual bx_bool register_pci_handlers(void *this_ptr,
-                                        bx_pci_read_handler_t f1,
-                                        bx_pci_write_handler_t f2,
+  bx_pci_bridge_c();
+  virtual ~bx_pci_bridge_c();
+  virtual void init(void);
+  virtual void reset(unsigned type);
+  virtual void register_state(void);
+  virtual void after_restore_state(void);
+  virtual bx_bool register_pci_handlers(bx_pci_device_stub_c *device,
                                         Bit8u *devfunc, const char *name,
                                         const char *descr);
   virtual bx_bool is_pci_device(const char *name);
@@ -74,34 +71,35 @@ public:
                                   bx_write_handler_t f2, Bit32u *addr,
                                   Bit8u *pci_conf, unsigned size,
                                   const Bit8u *iomask, const char *name);
-  virtual void   print_i440fx_state(void);
+  virtual void  print_i440fx_state(void);
   virtual Bit8u rd_memType (Bit32u addr);
   virtual Bit8u wr_memType (Bit32u addr);
 
+  virtual Bit32u pci_read_handler(Bit8u address, unsigned io_len);
+  virtual void   pci_write_handler(Bit8u address, Bit32u value, unsigned io_len);
+
 private:
   Bit8u pci_handler_id[0x100];  // 256 devices/functions
   struct {
-    bx_pci_read_handler_t  read;
-    bx_pci_write_handler_t write;
-    void             *this_ptr;
+    bx_pci_device_stub_c *handler;
   } pci_handler[BX_MAX_PCI_DEVICES];
-  unsigned num_pci_handles;
+  unsigned num_pci_handlers;
 
   bx_bool slot_used[BX_N_PCI_SLOTS];
   bx_bool slots_checked;
 
   struct {
     bx_def440fx_t i440fx;
-    } s;
+  } s;
+
+  void smram_control(Bit8u value);
 
   static Bit32u read_handler(void *this_ptr, Bit32u address, unsigned io_len);
   static void   write_handler(void *this_ptr, Bit32u address, Bit32u value, unsigned io_len);
-  static Bit32u pci_read_handler(void *this_ptr, Bit8u address, unsigned io_len);
-  static void   pci_write_handler(void *this_ptr, Bit8u address, Bit32u value, unsigned io_len);
 #if !BX_USE_PCI_SMF
   Bit32u read(Bit32u address, unsigned io_len);
   void   write(Bit32u address, Bit32u value, unsigned io_len);
-  Bit32u pci_read(Bit8u address, unsigned io_len);
-  void   pci_write(Bit8u address, Bit32u value, unsigned io_len);
 #endif
-  };
+};
+
+#endif
