@@ -1,5 +1,5 @@
 ///////////////////////////////////////////////////////////////////////
-// $Id: pit_wrap.cc,v 1.67 2007/09/28 19:52:05 sshwarts Exp $
+// $Id: pit_wrap.cc,v 1.69 2008/02/15 22:05:43 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -75,8 +75,8 @@ bx_pit_c bx_pit;
 
 //PIT tick to usec conversion functions:
 //Direct conversions:
-#define TICKS_TO_USEC(a) ( ((a)*USEC_PER_SECOND)/TICKS_PER_SECOND )
-#define USEC_TO_TICKS(a) ( ((a)*TICKS_PER_SECOND)/USEC_PER_SECOND )
+#define TICKS_TO_USEC(a) (((a)*USEC_PER_SECOND)/TICKS_PER_SECOND)
+#define USEC_TO_TICKS(a) (((a)*TICKS_PER_SECOND)/USEC_PER_SECOND)
 
 bx_pit_c::bx_pit_c()
 {
@@ -123,7 +123,7 @@ int bx_pit_c::init(void)
   bx_virt_timer.deactivate_timer(BX_PIT_THIS s.timer_handle[0]);
   BX_DEBUG(("deactivated timer."));
   if (BX_PIT_THIS s.timer.get_next_event_time()) {
-    bx_virt_timer.activate_timer(BX_PIT_THIS s.timer_handle[0], 
+    bx_virt_timer.activate_timer(BX_PIT_THIS s.timer_handle[0],
 				(Bit32u)BX_MAX(1,TICKS_TO_USEC(BX_PIT_THIS s.timer.get_next_event_time())),
 				0);
     BX_DEBUG(("activated timer."));
@@ -186,18 +186,16 @@ void bx_pit_c::handle_timer()
     periodic(time_passed32);
   }
   BX_PIT_THIS s.last_usec=BX_PIT_THIS s.last_usec + time_passed;
-  if(time_passed ||
-     (BX_PIT_THIS s.last_next_event_time
-      != BX_PIT_THIS s.timer.get_next_event_time())
-     ) {
-    BX_DEBUG(("pit: RESETting timer."));
+  if(time_passed || (BX_PIT_THIS s.last_next_event_time != BX_PIT_THIS s.timer.get_next_event_time()))
+  {
+    BX_DEBUG(("pit: RESETting timer"));
     bx_virt_timer.deactivate_timer(BX_PIT_THIS s.timer_handle[0]);
-    BX_DEBUG(("deactivated timer."));
+    BX_DEBUG(("deactivated timer"));
     if(BX_PIT_THIS s.timer.get_next_event_time()) {
-      bx_virt_timer.activate_timer(BX_PIT_THIS s.timer_handle[0], 
+      bx_virt_timer.activate_timer(BX_PIT_THIS s.timer_handle[0],
 				  (Bit32u)BX_MAX(1,TICKS_TO_USEC(BX_PIT_THIS s.timer.get_next_event_time())),
 				  0);
-      BX_DEBUG(("activated timer."));
+      BX_DEBUG(("activated timer"));
     }
     BX_PIT_THIS s.last_next_event_time = BX_PIT_THIS s.timer.get_next_event_time();
   }
@@ -249,10 +247,10 @@ Bit32u bx_pit_c::read(Bit32u address, unsigned io_len)
     case 0x61:
       /* AT, port 61h */
       BX_PIT_THIS s.refresh_clock_div2 = (bx_bool)((my_time_usec / 15) & 1);
-      return( (BX_PIT_THIS s.timer.read_OUT(2)<<5) |
-              (BX_PIT_THIS s.refresh_clock_div2<<4) |
-              (BX_PIT_THIS s.speaker_data_on<<1) |
-              (BX_PIT_THIS s.timer.read_GATE(2)?1:0) );
+      return (BX_PIT_THIS s.timer.read_OUT(2)<<5) |
+             (BX_PIT_THIS s.refresh_clock_div2<<4) |
+             (BX_PIT_THIS s.speaker_data_on<<1) |
+             (BX_PIT_THIS s.timer.read_GATE(2)?1:0);
       break;
 
     default:
@@ -288,7 +286,7 @@ void bx_pit_c::write(Bit32u address, Bit32u dvalue, unsigned io_len)
   }
   BX_PIT_THIS s.last_usec=BX_PIT_THIS s.last_usec + time_passed;
 
-  value = (Bit8u  ) dvalue;
+  value = (Bit8u) dvalue;
 
   if (bx_dbg.pit)
     BX_INFO(("pit: write to port %04x = %02x",
@@ -296,24 +294,24 @@ void bx_pit_c::write(Bit32u address, Bit32u dvalue, unsigned io_len)
 
   switch (address) {
     case 0x40: /* timer 0: write count register */
-      BX_PIT_THIS s.timer.write(0,value);
+      BX_PIT_THIS s.timer.write(0, value);
       break;
 
     case 0x41: /* timer 1: write count register */
-      BX_PIT_THIS s.timer.write( 1,value );
+      BX_PIT_THIS s.timer.write(1, value);
       break;
 
     case 0x42: /* timer 2: write count register */
-      BX_PIT_THIS s.timer.write( 2,value );
+      BX_PIT_THIS s.timer.write(2, value);
       break;
 
     case 0x43: /* timer 0-2 mode control */
-      BX_PIT_THIS s.timer.write( 3,value );
+      BX_PIT_THIS s.timer.write(3, value);
       break;
 
     case 0x61:
       BX_PIT_THIS s.speaker_data_on = (value >> 1) & 0x01;
-      if ( BX_PIT_THIS s.speaker_data_on ) {
+      if (BX_PIT_THIS s.speaker_data_on) {
 	  DEV_speaker_beep_on((float)(1193180.0 / this->get_timer(2)));
       } else {
 	  DEV_speaker_beep_off();
@@ -327,18 +325,16 @@ void bx_pit_c::write(Bit32u address, Bit32u dvalue, unsigned io_len)
         (unsigned) address, (unsigned) value));
   }
 
-  if(time_passed ||
-     (BX_PIT_THIS s.last_next_event_time
-      != BX_PIT_THIS s.timer.get_next_event_time())
-     ) {
-    BX_DEBUG(("pit: RESETting timer."));
+  if(time_passed || (BX_PIT_THIS s.last_next_event_time != BX_PIT_THIS s.timer.get_next_event_time()))
+  {
+    BX_DEBUG(("pit: RESETting timer"));
     bx_virt_timer.deactivate_timer(BX_PIT_THIS s.timer_handle[0]);
-    BX_DEBUG(("deactivated timer."));
+    BX_DEBUG(("deactivated timer"));
     if(BX_PIT_THIS s.timer.get_next_event_time()) {
-      bx_virt_timer.activate_timer(BX_PIT_THIS s.timer_handle[0], 
+      bx_virt_timer.activate_timer(BX_PIT_THIS s.timer_handle[0],
 				  (Bit32u)BX_MAX(1,TICKS_TO_USEC(BX_PIT_THIS s.timer.get_next_event_time())),
 				  0);
-      BX_DEBUG(("activated timer."));
+      BX_DEBUG(("activated timer"));
     }
     BX_PIT_THIS s.last_next_event_time = BX_PIT_THIS s.timer.get_next_event_time();
   }
